/* Changes recomb. fracts. into centi-Morgans Haldane  */
double cmH (double xx)
{
   double a;
   a = -50 * log (1 - 2 * xx);
   return (a);
}

/* Changes genetic distance in Haldane centi-Morgans to 
   recombination fractions.                                         */

double rf (double yy)
{
   double b;
   b = .5 * (1 - exp (-yy / 50));
   return (b);
}

/* Reset the values of the global varibles after finding starting 
   value of the map for SA algorithm.                               */

void reset_globs (int sxs_flg, int nscans)
{
   if (sxs_flg)
   {
      if (NAscentIt < nscans)
         NAscentIt = nscans;
      if (NCurvIt < (5 * nscans))
         NCurvIt = 5 * nscans;
      if (NGradIt < (nscans / 5))
         NGradIt = nscans / 5;
      if (Obs_Info_Spc < (10 * nscans))
         Obs_Info_Spc = 10 * nscans;
      if (Glob_Max < (20 * nscans))
         Glob_Max = 20 * nscans;
   }
   else
   {
      if (NAscentIt < nscans)
         NAscentIt = nscans;
      if (NCurvIt < (5 * nscans))
         NCurvIt = 5 * nscans;
      if (NGradIt < (nscans / 5))
         NGradIt = nscans / 5;
      if (Obs_Info_Avg < (10 * nscans))
         Obs_Info_Avg = 10 * nscans;
      if (Glob_Max < (20 * nscans))
         Glob_Max = 20 * nscans;
   }
}

/* Generates initial parameter estiamtes   */
void get_init (int sxs_flg, double **map1, double *rate, int err)
{
   double u, x, ii = 0, F = 0;          /* rate ~ Beta(2,k)
                                         * if using error model */
   int   i, j, nloci, k = 75;

   nloci = NLoci - 1;
   u = rans ();
   if (err)
   {
      while (F < u)
      {
         ii++;
         x = ii / 1000;
         F = (1 - pow ((1 - x), (k + 1))) - ((k + 1) * pow ((1 - x), k) * x);
      }
      *rate = x - .0005;
   }

   for (j = 1; j < nloci; j++)          /* Add random error to par.file map */
   {
      if (sxs_flg)
      {
         for (j = 1; j < nloci; j++)
            for (i = 0; i < 2; i++)
               Marek[i][j] = Alpha * map1[i][j] + (1 - Alpha) * u * 0.5;
      }
      else
      {
         Marek[0][j] = rf (0.5 * (cmH (map1[0][j]) + cmH (map1[1][j])));
         Marek[0][j] = Alpha * Marek[0][j] + (1 - Alpha) * u * 0.5;
         Marek[1][j] = Marek[0][j];
      }
   }
}

/* Copy S.j in to S.j^  */
void swcpy (int *a, int *b, int *c, int *d)
{
   int   i;
   for (i = 0; i < pedsize; i++)
   {
      c[i] = a[i];
      d[i] = b[i];
   }
}

/* Compute and evaluate the Hastings Ratio */
void decide (double *x)
{
   double u;
   x[0] = exp (x[1] + x[2] - x[3] - x[4]);
   u = rans ();
#if 0
   int   i;
   for (i = 1; i <= 4; i++)
      printf (" x[%d] = %6.4f \n", i, x[i]);
   printf (" u is: %6.4f, logprob: %6.4f, hratio is: %6.4f \n",
           u, (x[1] + x[2] - x[3] - x[4]), x[0]);
   printf ("\n runif = %6.6f, hratio = %6.6f, ", u, x[0]);
#endif
   if (u < x[0])
      x[0] = 1;
   else
      x[0] = 0;
/*  printf(" accept = %1.0f \n", x[0]); */
}

/* Prints a map estimation section header to the screen */
void pr_section (int type)
{
   if (type == 1)
   {
      printf ("**********************************************************\n");
      printf ("**********************************************************\n");
      printf ("\n Estimating sex-averaged recombination fractions (RF).");
      printf ("\n Starting value(s) for the hybrid algorithm: \n");
      printf ("\n Interval   Sex-Averaged (RF)");
      printf ("\n --------   -----------------");
   }
   else
   {
      printf ("**********************************************************\n");
      printf ("**********************************************************\n");
      printf ("\n Estimating sex-specific recombination fractions (RF).");
      printf ("\n Starting value(s) for the hybrid algorithm: \n");
      printf ("\n Interval   Female (RF)   Male (RF)");
      printf ("\n --------   -----------   ---------");
   }
}

/* Part of matrix inversion routine  */
void ludcmp (double **anr, int nnr, int *indxnr, double *d)
{
   int   i, imax, j, k;
   double big, dum, sum, temp;
   double *vv;

   vv = malloc ((nnr + 2) * sizeof (double));
   alloc_chk (WHERE, vv);
   for (i = 0; i <= (nnr + 1); i++)
      vv[i] = 0;

   d[0] = 1.0;
   for (i = 1; i <= nnr; i++)
   {
      big = 0.0;
      for (j = 1; j <= nnr; j++)
         if ((temp = fabs (anr[i][j])) > big)
            big = temp;
      if (big == 0.0)
         printf ("\n Singular matrix in routine LUDCMP");
      vv[i] = 1.0 / big;
   }

   for (j = 1; j <= nnr; j++)
   {
      for (i = 1; i < j; i++)
      {
         sum = anr[i][j];
         for (k = 1; k < i; k++)
            sum -= anr[i][k] * anr[k][j];
         anr[i][j] = sum;
      }
      big = 0.0;
      for (i = j; i <= nnr; i++)
      {
         sum = anr[i][j];
         for (k = 1; k < j; k++)
            sum -= anr[i][k] * anr[k][j];
         anr[i][j] = sum;
         if ((dum = vv[i] * fabs (sum)) >= big)
         {
            big = dum;
            imax = i;
         }
      }
      if (j != imax)
      {
         for (k = 1; k <= nnr; k++)
         {
            dum = anr[imax][k];
            anr[imax][k] = anr[j][k];
            anr[j][k] = dum;
         }
         d[0] = -(d[0]);
         vv[imax] = vv[j];
      }
      indxnr[j] = imax;
      if (anr[j][j] == 0.0)
         anr[j][j] = TINY;
      if (j != nnr)
      {
         dum = 1.0 / (anr[j][j]);
         for (i = j + 1; i <= nnr; i++)
            anr[i][j] *= dum;
      }
   }
   free (vv);
}

/* Part of matrix inversion routine   */
void lubksb (double **anr, int nnr, int *indxnr, double *bnr)
{
   int   i, ii = 0, ip, j;
   double sum;

   for (i = 1; i <= nnr; i++)
   {
      ip = indxnr[i];
      sum = bnr[ip];
      bnr[ip] = bnr[i];
      if (ii)
         for (j = ii; j <= i - 1; j++)
            sum -= anr[i][j] * bnr[j];
      else if (sum)
         ii = i;
      bnr[i] = sum;
   }
   for (i = nnr; i >= 1; i--)
   {
      sum = bnr[i];
      for (j = i + 1; j <= nnr; j++)
         sum -= anr[i][j] * bnr[j];
      bnr[i] = sum / anr[i][i];
   }
}

 /* Determines the increment between source and destination maps to move to.  */
double find_intv (double xx, double yy, int *z)
{
   double a, b, c;
   double b_1 = 0.012, b_2 = 0.905, b_3 = 1.1905;

   a = cmH (xx);
   b = cmH (yy);

   if (b > a)                           /* if the (dn) interval > (se) interval,
                                         * then expand (se)   */
   {
      c = b_1 * a * a + b_2 * a + b_3;
      if (c > b)
         c = b;
   }
   else                                 /* if (dn) interval <= (se) interval,
                                         * then shrink (se)   */
   {
      c = (sqrt (b_2 * b_2 - 4 * b_1 * (b_3 - a)) - b_2) / (2 * b_1);
      if (c < b)
         c = b;
   }
   if (c != b)
      (*z) = 1;                         /* (i.e. continue to accumulate)    */
   c = rf (c);
   return (c);
}

/* Calculates the left and right comp.data log likelihood-ratios
   from a single realized matrix S.                                  */
void lr_cdllr (int nloc, double *cdllr, double **se, double **mt, double **tt,
               int ***scount, int ***recounts)
{
   int   c, j, jj;

   double f_r = 0.0;                    /* stores total F recombs at a locus  */
   double m_r = 0.0;                    /* stores total M recombs at a locus  */
   double f_nr = 0.0;                   /* stores total F non-reco at a locus */
   double m_nr = 0.0;                   /* stores total M non-reco at a locus */

   cdllr[0] = 0;
   cdllr[1] = 0;
   for (j = 1; j < nloc; j++)
      for (c = 0; c < n_compnts; c++)
         for (jj = 0; jj < 6; jj++)
            scount[c][j][jj] = recounts[c][j][jj] - scount[c][j][jj];
   for (j = 1; j < nloc; j++)
   {
      m_r = 0;
      f_r = 0;
      m_nr = 0;
      f_nr = 0;
      for (c = 0; c < n_compnts; c++)
      {
         m_r += scount[c][j][1];
         f_r += scount[c][j][4];
         m_nr += scount[c][j][2];
         f_nr += scount[c][j][5];
      }
      cdllr[0] += (f_r * log (se[0][j] / mt[0][j])
                   + m_r * log (se[1][j] / mt[1][j])
                   + f_nr * log ((1 - se[0][j]) / (1 - mt[0][j]))
                   + m_nr * log ((1 - se[1][j]) / (1 - mt[1][j])));

      cdllr[1] += (f_r * log (tt[0][j] / mt[0][j])
                   + m_r * log (tt[1][j] / mt[1][j])
                   + f_nr * log ((1 - tt[0][j]) / (1 - mt[0][j]))
                   + m_nr * log ((1 - tt[1][j]) / (1 - mt[1][j])));
   }
   for (j = 1; j < nloc; j++)
      for (c = 0; c < n_compnts; c++)
         for (jj = 0; jj < 6; jj++)
            scount[c][j][jj] = recounts[c][j][jj];
}

/*    Resets the marker map and performs burn-in if burn = 1.   */
void change_map (int burn, int nsij, double **map, double **probs)
{
   int   sex, i, a = 0, nloci;
   double z = 0.5;

   nloci = NLoci - 1;
   for (sex = FEM; sex <= MAL; sex++)
      set_chromosomes (nloci, map[sex], 0, &a, &z, sex);
   set_positions (nloci);
   if (burn)
      for (i = 1; i <= NBrnIt; i++)
         sampling (0, 1, nloci, nsij, nFGL, probs);
}

/* Checks to see if two maps are 'close'   */
int comp_map (double z, double **map_a, double **map_b)
{
   int   nloci, i, j, k = 1;
   double temp;

   nloci = NLoci - 1;
   for (i = 0; i < 2; i++)
      for (j = 1; j < nloci; j++)
      {
         temp = fabs (map_a[i][j] - map_b[i][j]) / map_b[i][j];
         if (temp > z)
            k = 0;
      }
   return (k);
}

/* Puts the contents of map_a into map_b   */
void sw_map (double **map_a, double **map_b)
{
   int   i, j, nloci;

   nloci = NLoci - 1;
   for (i = 0; i < 2; i++)
      for (j = 1; j < nloci; j++)
         map_b[i][j] = map_a[i][j];

}

/* Prints a map to the screen   */
void pr_map (int sxs_flg, double **map)
{
   int   j, nloci;

   printf ("\n");
   nloci = NLoci - 1;

   if (sxs_flg == 0)                    /* Then print a sex-averaged map.  */
      for (j = 1; j < nloci; j++)
         printf (" %4d  %15.4f \n", j, map[0][j]);
   else
      for (j = 1; j < nloci; j++)
         printf ("%4d %15.4f %11.4f\n", j, map[0][j], map[1][j]);
}

/* Initializes the recomb counts to zero and sums over components  */
void set_rc (int locus, int ***rcounts, double **rcs)
{
   int   c, i, k, n_comps;

   n_comps = n_compnts;
   for (k = 0; k < 3; k++)
      for (i = 0; i < 2; i++)
         rcs[i][k] = 0;

   for (c = 0; c < n_comps; c++)
   {
      rcs[1][2] += rcounts[c][locus][0]; /* totals      */
      rcs[1][0] += rcounts[c][locus][1]; /* recombs     */
      rcs[1][1] += rcounts[c][locus][2]; /* non-recombs */
      rcs[0][2] += rcounts[c][locus][3];
      rcs[0][0] += rcounts[c][locus][4];
      rcs[0][1] += rcounts[c][locus][5];
   }
}
