/* Computes a sequence of MCMC-EM estimates 
   of the map. An error model is *** NOT *** used.                  */

int mcemit (int sxs_flg, int sz_flg, int nsij,
            int ***Recounts, double **Temp, double **mle, double **probs)
{
   int   i, j, c, jj, t;
   int   nscans, nloci;
   double **rcz;

   rcz = malloc (2 * sizeof (double *));
   alloc_chk (WHERE, rcz);
   for (i = 0; i < 2; i++)
   {
      rcz[i] = malloc (3 * sizeof (double));
      alloc_chk (WHERE, rcz[i]);
      for (j = 0; j < 3; j++)
         rcz[i][j] = 0;
   }

   nscans = NIt;
   nloci = NLoci - 1;
   while (!sz_flg)
   {
      nscans = nscans * 2;

      for (j = 0; j < nloci; j++)
         for (c = 0; c < n_compnts; c++)
            for (jj = 0; jj < 6; ++jj)
               Recounts[c][j][jj] = 0;

      for (t = 0; t < nscans; ++t)
      {
         sampling (0, 1, nloci, nsij, nFGL, probs);
         for (c = 0; c < n_compnts; ++c)
            full_seg_prob (nloci, c, Recounts, Temp);
      }
      for (j = 1; j < nloci; j++)
      {
         set_rc (j, Recounts, rcz);
         if (sxs_flg)                   /* generate sxs map estimates */
         {
            for (i = 0; i < 2; i++)
            {
               mle[i][j] = rcz[i][0] / rcz[i][2];
               if (mle[i][j] < RF_bound)
                  mle[i][j] = RF_bound;
               if (mle[i][j] > (.5 - RF_bound))
                  mle[i][j] = .5 - RF_bound;
            }
         }
         else                           /* generate sxa map estimates */
         {
            mle[0][j] = (rcz[0][0] + rcz[1][0]) / (rcz[0][2] + rcz[1][2]);
            if (mle[0][j] < RF_bound)
               mle[0][j] = RF_bound;
            if (mle[0][j] > (.5 - RF_bound))
               mle[0][j] = .5 - RF_bound;
            mle[1][j] = mle[0][j];
         }
      }
      sz_flg = comp_map (EM_bound, mle, Marek);
      sw_map (mle, Marek);
      change_map (1, nsij, Marek, probs);
   }
   for (i = 0; i < 2; i++)
      free (rcz[i]);
   free (rcz);
   return (nscans);
}

/* Computes the observed information. An error model is NOT used.   */

void obs_info (int sxs_flg, double **oi_mat, int ***Scount,
               int ***Recounts, int nsij, double **probs,
               double **Temp, double **mle, int T, int R)
{
   int   c, i, j, t, jj, nloci, Nloci;
   double **rcz;

   rcz = malloc (2 * sizeof (double *));
   alloc_chk (WHERE, rcz);
   for (i = 0; i < 2; i++)
   {
      rcz[i] = malloc (3 * sizeof (double));
      alloc_chk (WHERE, rcz[i]);
      for (j = 0; j < 3; j++)
         rcz[i][j] = 0;
   }

   nloci = NLoci - 1;
   Nloci = 2 * nloci;

   for (j = 0; j < nloci; j++)
      for (i = 0; i < n_compnts; i++)
         for (jj = 0; jj < 6; jj++)
            Scount[i][j][jj] = Recounts[i][j][jj] = 0;
   for (i = 0; i < Nloci; i++)
      for (j = 0; j < Nloci; j++)
         oi_mat[i][j] = 0;

   for (t = 0; t < T; t++)
   {
      sampling (0, 1, nloci, nsij, nFGL, probs);
      for (c = 0; c < n_compnts; ++c)
         full_seg_prob (nloci, c, Recounts, Temp);
      for (j = 1; j < nloci; j++)
         for (c = 0; c < n_compnts; c++)
            for (jj = 0; jj < 6; jj++)
               Scount[c][j][jj] = Recounts[c][j][jj] - Scount[c][j][jj];
      if (sxs_flg)                      /* handle the sex-specific case */
      {
         for (j = 1; j < nloci; j++)
         {
            set_rc (j, Scount, rcz);
            for (i = 0; i < 2; i++)
               oi_mat[(2 * j - 1 + i)][0] = rcz[i][0] / mle[i][j]
                  - rcz[i][1] / (1 - mle[i][j]);
         }
         for (j = 1; j < (Nloci - 1); j++)
            for (i = 1; i < (Nloci - 1); i++)
               oi_mat[j][i] += oi_mat[j][0] * oi_mat[i][0];

         for (j = 1; j < nloci; j++)
            for (c = 0; c < n_compnts; c++)
               for (jj = 0; jj < 6; jj++)
                  Scount[c][j][jj] = Recounts[c][j][jj];
      }
      else                              /* handle the sex-averaged case */
      {
         for (j = 1; j < nloci; j++)
         {
            set_rc (j, Scount, rcz);
            oi_mat[j][0] = (rcz[0][0] + rcz[1][0]) / mle[0][j]
               - (rcz[0][1] + rcz[1][1]) / (1 - mle[0][j]);
         }
         for (j = 1; j < nloci; j++)
            for (i = 1; i < nloci; i++)
               oi_mat[j][i] += oi_mat[j][0] * oi_mat[i][0];

         for (j = 1; j < nloci; j++)
            for (c = 0; c < n_compnts; c++)
               for (jj = 0; jj < 6; jj++)
                  Scount[c][j][jj] = Recounts[c][j][jj];
      }
   }
   if (sxs_flg)                         /* more sex-specific components of obs_info */
   {
      for (i = 1; i < (Nloci - 1); i++) /* compute E(delta^2)    */
         for (j = 1; j < (Nloci - 1); j++)
            oi_mat[i][j] = oi_mat[i][j] / T;

      for (j = 1; j < nloci; j++)       /* add E(delta(delta))        */
      {
         set_rc (j, Recounts, rcz);
         for (i = 0; i < 2; i++)
         {
            oi_mat[(2 * j - 1 + i)][(2 * j - 1 + i)] -=
               (rcz[i][0] / pow (mle[i][j], 2) + rcz[i][1] /
                pow ((1 - mle[i][j]), 2)) / T;
            oi_mat[(2 * j - 1 + i)][0] =
               (rcz[i][0] / mle[i][j] - rcz[i][1] / (1 - mle[i][j])) / T;
         }
      }
      if (R == Redo)
      {
         printf
            ("\n\n Partial derivatives taken in order: (F1, M1, F2, M2, ...) RFs");
         printf ("\n\n Estimated gradient logL(mle): ");
      }
      for (j = 1; j < (Nloci - 1); j++) /* subtract [E(delta)]^2  */
      {
         if (R == Redo)
            printf (" %6.4f ", oi_mat[j][0]);
         for (i = 1; i < (Nloci - 1); i++)
            oi_mat[j][i] -= oi_mat[j][0] * oi_mat[i][0];
      }
      for (i = 1; i < (Nloci - 1); i++) /* negate obs_info_mat  */
         for (j = 1; j < (Nloci - 1); j++)
            oi_mat[i][j] = -oi_mat[i][j]; /*observed information done */
   }
   else                                 /* more sex-averaged compnoents of obs_info */
   {
      for (i = 1; i < nloci; i++)
         for (j = 1; j < nloci; j++)
            oi_mat[i][j] = oi_mat[i][j] / T; /* E((delta)^2) done */

      for (j = 1; j < nloci; j++)       /* add E(delta(delta))    */
      {
         set_rc (j, Recounts, rcz);
         oi_mat[j][j] -= ((rcz[0][0] + rcz[1][0]) / pow (mle[0][j], 2)
                          + (rcz[0][1] + rcz[1][1]) / pow ((1 - mle[0][j]),
                                                           2)) / T;

         oi_mat[j][0] = ((rcz[0][0] + rcz[1][0]) / mle[0][j] -
                         (rcz[0][1] + rcz[1][1]) / (1 - mle[0][j])) / T;
      }
      if (R == Redo)
         printf ("\n\n Estimated gradient of logL(mle): ");
      for (j = 1; j < nloci; j++)       /* subtract [E(delta)]^2  */
      {
         if (R == Redo)
            printf (" %6.4f ", oi_mat[j][0]);
         for (i = 1; i < nloci; i++)
            oi_mat[j][i] -= oi_mat[j][0] * oi_mat[i][0];
      }
      for (i = 1; i < nloci; i++)       /* negate obs_info_mat */
         for (j = 1; j < nloci; j++)
            oi_mat[i][j] = -oi_mat[i][j]; /* observed information done  */
   }
   for (i = 0; i < 2; i++)
      free (rcz[i]);
   free (rcz);
}

/* Inverts a matrix of doubles (created dynamically) [nnr x nnr], 
the input and the result are both one indexed.                      */

void d_invert (double **oi_mat, double **ioi_mat, int rows)
{
   /* ioi_mat holds inv_obs_info */
   int   Nloci, jnr, inr, *indxnr;
   double d = 0, *bnr;

   Nloci = 2 * (NLoci - 1);

   indxnr = malloc (Nloci * sizeof (int));
   alloc_chk (WHERE, indxnr);
   bnr = malloc (Nloci * sizeof (double));
   alloc_chk (WHERE, bnr);

   ludcmp (oi_mat, rows, indxnr, &d);
   for (jnr = 1; jnr <= rows; jnr++)
   {
      for (inr = 1; inr <= rows; inr++)
         bnr[inr] = 0.0;
      bnr[jnr] = 1.0;
      lubksb (oi_mat, rows, indxnr, bnr);
      for (inr = 1; inr <= rows; inr++)
         ioi_mat[inr][jnr] = bnr[inr];
   }
   free (indxnr);
   free (bnr);
}

/* Calculate a stochastic approx. iteration                         */

void sait (int sxs_flg, int vv, int ***Recounts, int nsij,
           double **probs, double **Temp, double **ioi_mat, double **mle)
{
   int   i, j, jj, c, nloci, Nloci, t;
   double **rcz, *temp, gama;

   nloci = NLoci - 1;
   Nloci = 2 * nloci;

   temp = malloc (Nloci * sizeof (double));
   rcz = malloc (2 * sizeof (double *));
   alloc_chk (WHERE, rcz);
   for (i = 0; i < 2; i++)
   {
      rcz[i] = malloc (3 * sizeof (double));
      alloc_chk (WHERE, rcz[i]);
      for (j = 0; j < 3; j++)
         rcz[i][j] = 0;
   }
   for (j = 0; j < nloci; j++)
      for (c = 0; c < n_compnts; c++)
         for (jj = 0; jj < 6; jj++)
            Recounts[c][j][jj] = 0;

   for (t = 0; t < NGradIt; ++t)
   {
      sampling (0, 1, nloci, nsij, nFGL, probs);
      for (c = 0; c < n_compnts; ++c)
         full_seg_prob (nloci, c, Recounts, Temp);
   }
   if (sxs_flg)                         /* generate SA sex-specific est. */
   {
      for (j = 1; j < nloci; j++)
      {
         set_rc (j, Recounts, rcz);
         for (i = 0; i < 2; i++)
         {
            gama = (rcz[i][0] / Marek[i][j] - rcz[i][1] /
                    (1 - Marek[i][j])) / NGradIt;
            temp[2 * j - 1 + i] = gama / (vv + 1);
         }
      }
      for (j = 1; j < (Nloci - 1); j++)
      {
         gama = 0;
         for (i = 1; i < (Nloci - 1); i++)
            gama += (temp[i] * ioi_mat[j][i]);
         if (j % 2)
         {
            i = (j + 1) / 2;
            mle[0][i] += gama;
            if (mle[0][i] < RF_bound)
               mle[0][i] = RF_bound;
            if (mle[0][i] > (.5 - RF_bound))
               mle[0][i] = .5 - RF_bound;
         }
         else
         {
            i = j / 2;
            mle[1][i] += gama;
            if (mle[1][i] < RF_bound)
               mle[1][i] = RF_bound;
            if (mle[1][i] > (.5 - RF_bound))
               mle[1][i] = .5 - RF_bound;
         }
      }
   }
   else                                 /* generate SA sex-averaged est. */
   {
      for (j = 1; j < nloci; j++)
      {
         set_rc (j, Recounts, rcz);
         gama = ((rcz[0][0] + rcz[1][0]) / Marek[0][j] -
                 (rcz[0][1] + rcz[1][1]) / (1 - Marek[0][j])) / NGradIt;
         temp[j] = gama / (vv + 1);
      }
      for (j = 1; j < nloci; j++)
      {
         gama = 0;
         for (i = 1; i < nloci; i++)
            gama += (temp[i] * ioi_mat[j][i]);
         mle[0][j] += gama;
         if (mle[0][j] < RF_bound)
            mle[0][j] = RF_bound;
         if (mle[0][j] > (.5 - RF_bound))
            mle[0][j] = .5 - RF_bound;
         mle[1][j] = mle[0][j];
      }
   }
   free (temp);
   for (i = 0; i < 2; i++)
      free (rcz[i]);
   free (rcz);
}

/* Computes the log likelihood ratio of two map2 over map1.         */

double logLR (double **map1, double **map2, int nsij,
              int ***Scount, int ***Recounts, double **probs,
              double **Temp, int *alf, int ss)
{
   int   c, i, j, jj, t, nloci;
   double **tt, **mt, sum_cdlr[2], cdllr[2];

   nloci = NLoci - 1;

   tt = malloc (2 * sizeof (double *));
   alloc_chk (WHERE, tt);
   mt = malloc (2 * sizeof (double *));
   alloc_chk (WHERE, mt);
   for (i = 0; i < 2; i++)
   {
      tt[i] = malloc ((nloci + 1) * sizeof (double));
      alloc_chk (WHERE, tt[i]);
      mt[i] = malloc ((nloci + 1) * sizeof (double));
      alloc_chk (WHERE, mt[i]);
   }

   for (i = 0; i < 2; i++)              /* compute tt, and mt  */
      for (j = 1; j < nloci; j++)
      {
         tt[i][j] = find_intv (map1[i][j], map2[i][j], alf);
         mt[i][j] = rf ((0.5 * (cmH (tt[i][j]) + cmH (map1[i][j]))));
      }
   change_map (1, nsij, mt, probs);
   for (i = 0; i < n_compnts; i++)
      for (j = 0; j < nloci; j++)
         for (jj = 0; jj < 6; jj++)
         {
            Scount[i][j][jj] = Recounts[i][j][jj] = 0;
         }
   sum_cdlr[0] = 0;
   sum_cdlr[1] = 0;
   for (t = 0; t < ss; ++t)
   {
      sampling (0, 1, nloci, nsij, nFGL, probs);
      for (c = 0; c < n_compnts; ++c)
         full_seg_prob (nloci, c, Recounts, Temp);

      lr_cdllr (nloci, cdllr, map1, mt, tt, Scount, Recounts);

      sum_cdlr[0] += exp (cdllr[0]);
      sum_cdlr[1] += exp (cdllr[1]);
   }
   sw_map (tt, map1);

   for (i = 0; i < 2; i++)
   {
      free (tt[i]);
      free (mt[i]);
   }
   free (tt);
   free (mt);

   return (log (sum_cdlr[1] / sum_cdlr[0]));
}
