/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */

typedef struct node_tag
{
   double value;
   struct node_tag *next;
}
val_node;

typedef struct dist_tag
{
   int   frequency;
   double lower_limit;
   double upper_limit;
   struct dist_tag *next;
}
dist_node;

void print_linked_list (val_node * headPtr)
{
   if (headPtr == NULL)
   {
      printf ("Empty list\n");
   }
   while (headPtr)
   {
      val_node *tempPtr = headPtr->next;
      printf ("%f ", headPtr->value);
      headPtr = tempPtr;
   }
}
void print_freq_dist (dist_node * headPtr)
{
   if (headPtr == NULL)
   {
      printf ("Empty list\n");
   }
   while (headPtr)
   {
      dist_node *tempPtr = headPtr->next;
      printf ("(%f - %f)  %d\n", headPtr->lower_limit, headPtr->upper_limit,
              headPtr->frequency);
      headPtr = tempPtr;
   }
}
void free_linked_list (val_node * headPtr)
{
   if (headPtr == NULL)
   {
      printf ("Empty list..Nothing to free\n");
   }
   while (headPtr)
   {
      val_node *tempPtr = headPtr->next;
      free (headPtr);
      headPtr = tempPtr;
   }
}

void free_histogram (dist_node * headPtr)
{
   if (headPtr == NULL)
   {
      printf ("Empty list..Nothing to free\n");
   }
   while (headPtr)
   {
      dist_node *tempPtr = headPtr->next;
      free (headPtr);
      headPtr = tempPtr;
   }
}

void ReadFromList (double *pValueList, int nSize, val_node ** HeadPtr,
                   double *minValue, double *maxValue)
{
   int   i;
   double minVal = (double) INT_MIN;
   double maxVal = (double) INT_MAX;
   int   bFirst = 1;
   val_node *headptr = NULL;
   double val = 0;

   if (pValueList == NULL)
   {
      *HeadPtr = NULL;
      printf ("Something wrong here at in reading from NULL List (%d)\n",
              __LINE__);
      exit (EXIT_FAILURE);
   }

   for (i = 0; i < nSize; i++)
   {
      val = pValueList[i];
      val_node *tempPtr = (val_node *) malloc (sizeof (val_node));
      alloc_chk (WHERE, tempPtr);
      if (tempPtr)
      {
         if (bFirst)
         {
            minVal = val;
            maxVal = val;
            bFirst = 0;
         }
         if (val > maxVal)
         {
            maxVal = val;
         }
         if (val < minVal)
         {
            minVal = val;
         }
         tempPtr->value = val;
         /*printf("%f=value",val); */
         tempPtr->next = headptr;
         headptr = tempPtr;
      }
   }
   *minValue = minVal;
   *maxValue = maxVal;
   *HeadPtr = headptr;
}

void readFile (FILE * read_fp, val_node ** HeadPtr, double *minValue,
               double *maxValue, int *listSize)
{
   double minVal = (double) INT_MIN;
   double maxVal = (double) INT_MAX;
   int   bFirst = 1;
   int   lSize = 0;
   val_node *headptr = NULL;

   if (read_fp)
   {
      double val = 0;
      while (fscanf (read_fp, "%lf\n", &val) != EOF)
      {
         /*printf("%f---val\n",val); */
         val_node *tempPtr = (val_node *) malloc (sizeof (val_node));
         alloc_chk (WHERE, tempPtr);
         if (tempPtr)
         {
            if (bFirst)
            {
               minVal = val;
               maxVal = val;
               bFirst = 0;
            }
            if (val > maxVal)
            {
               maxVal = val;
            }
            if (val < minVal)
            {
               minVal = val;
            }
            tempPtr->value = val;
            /*printf("%f=value",val); */
            tempPtr->next = headptr;
            headptr = tempPtr;
            lSize++;
         }
      }

   }
   else
   {
      printf ("Invalid file pointer..Exiting..\n");
   }
   *listSize = lSize;
   *minValue = minVal;
   *maxValue = maxVal;
   *HeadPtr = headptr;
}

void create_hostogram_table (val_node * HeadPtr, dist_node ** distributionPtr,
                             double maxValue, double minValue, int size,
                             int noOfInterval)
{
   double range;
   double width;
   dist_node *distPtr = NULL;
   dist_node *origDistPtr = NULL;
   int   i;
   range = maxValue - minValue;
   if (noOfInterval != 0)
   {
      width = range / noOfInterval;
   }

   if (HeadPtr == NULL)
   {
      printf ("No Information passed to create Histogram table (%d)\n",
              __LINE__);
      exit (EXIT_FAILURE);
   }
   /* create the entire list here */
   if (noOfInterval == 0)
   {
      dist_node *tempPtr = NULL;
      tempPtr = (dist_node *) malloc (sizeof (dist_node));
      alloc_chk (WHERE, tempPtr);

      tempPtr->lower_limit = minValue;
      tempPtr->upper_limit = maxValue + 1;
      tempPtr->frequency = size;
      tempPtr->next = distPtr;
      distPtr = tempPtr;
      origDistPtr = tempPtr;
   }
   else
   {
      for (i = 0; i < noOfInterval; i++)
      {
         dist_node *tempPtr = NULL;
         tempPtr = (dist_node *) malloc (sizeof (dist_node));
         alloc_chk (WHERE, tempPtr);

         tempPtr->frequency = 0;
         tempPtr->next = distPtr;
         /*   tempPtr->lower_limit = minValue + (i*width);
            tempPtr->upper_limit = minValue + ((i+1)*width)); */
         if (i == 0)
         {
            tempPtr->upper_limit = (maxValue + 1);
         }
         else
         {
            tempPtr->upper_limit = (maxValue - (i * width));
         }
         if (i == (noOfInterval - 1))
         {
            tempPtr->lower_limit = (minValue);
         }
         else
         {
            tempPtr->lower_limit = (maxValue - ((i + 1) * width));
         }
         distPtr = tempPtr;
         origDistPtr = tempPtr;
      }
   }
   while (HeadPtr)
   {
      if (distPtr == NULL)
      {
         printf ("Invalid distribution ptr passed, Exiting ... (%d) (%s)\n",
                 __LINE__, __FILE__);
         exit (EXIT_FAILURE);
      }

      while (distPtr)
      {
         if ((distPtr->lower_limit <= HeadPtr->value)
             && (distPtr->upper_limit > HeadPtr->value))
         {
            distPtr->frequency = (distPtr->frequency) + 1;
            break;
         }
         distPtr = distPtr->next;
      }
      HeadPtr = HeadPtr->next;
      distPtr = origDistPtr;
   }
   *distributionPtr = distPtr;
}

void writeToOutputFile (FILE * out_fp, dist_node * headPtr)
{
   char  buffer[512];
   if (out_fp == NULL)
   {
      printf ("Invalid file pointer passed for writting %d-%s\n", __LINE__,
              __FILE__);
      exit (EXIT_FAILURE);
   }
   if (headPtr == NULL)
   {
      printf ("Empty nformation passed for writing..Exiting (%d)\n", __LINE__);
      exit (EXIT_FAILURE);
   }

   while (headPtr)
   {
      dist_node *tempPtr = headPtr->next;
      if (headPtr->frequency > 0)
      {
         int   len = sprintf (buffer, "%f    %d",
                              (headPtr->lower_limit + headPtr->upper_limit) / 2,
                              headPtr->frequency);
         fwrite (buffer, 1, len, out_fp);
      }
      /*headPtr = tempPtr ; */
      if ((headPtr->next) && (headPtr->frequency > 0))
      {
         fwrite ("    ", 1, 4, out_fp);
      }
      headPtr = tempPtr;
   }
   fwrite ("    \n", 1, 5, out_fp);
}

int PerformHistogramAnalysis (int noOfIteration, int noOfLocus, int noOfStats,
                              char *outputFilename)
{
   val_node *headPtr = NULL;
   dist_node *histogramPtr = NULL;
   double minval, maxval;
   int   listsize;
   int   i, j, k, len;
   char  buffer[500];
   FILE *out_fp = NULL;
   FILE *read_fp = NULL;
   if (outputFilename == NULL)
   {
      printf ("Invalid output file name \n");
      return 0;
   }
   out_fp = open_chkq (WHERE, outputFilename, "w");

   for (i = 0; i < noOfIteration; i++)
   {
      for (j = 0; j < noOfLocus; j++)
      {
         for (k = 0; k < noOfStats; k++)
         {
            len = sprintf (buffer, "./file_%d_%d_%d", i, j, k);
            read_fp = open_chkq (WHERE, buffer, "r");

            /* read the input file and create the linked list here */
            readFile (read_fp, &headPtr, &minval, &maxval, &listsize);
            if (read_fp)
            {
               fclose (read_fp);
            }
            if (remove (buffer) == -1)
            {
               printf ("Failed to delete the file\n");
            }
            read_fp = NULL;
            /*print_linked_list(headPtr) ; */
            /* create the histogram table from the linked list */
            create_hostogram_table (headPtr, &histogramPtr, maxval, minval,
                                    listsize, 1);
            /*print_freq_dist(histogramPtr); */
            /* write the histogram table in file */
            writeToOutputFile (out_fp, histogramPtr);
            free_linked_list (headPtr);
            free_histogram (histogramPtr);
            headPtr = NULL;
            histogramPtr = NULL;
         }
      }
   }
   if (out_fp)
   {
      fclose (out_fp);
   }
   return 1;
}

int PerformHistogramAnalysisNew (int nSample, int nFileLimit, int noOfIteration,
                                 int noOfLocus, int noOfStats)
{
   val_node *headPtr = NULL;
   dist_node *histogramPtr = NULL;
   double minval, maxval;
   int   i, j, k, len;
   char  buffer[500];
   FILE *out_fp = NULL;
   FILE *read_fp = NULL;
   FILE **read_fpList = NULL;
   double *pValList = NULL;
   int  *pFilePosList = NULL;
   int   nLineCounter = 0;
   int   nFileCounterDiff = 0;
   int   nOpenedFile = 0;
   out_fp = open_chkq (WHERE, "Histogram.txt", "w");

   pValList = (double *) calloc (nSample, sizeof (double));
   alloc_chk (WHERE, pValList);

   nFileCounterDiff = nSample - nFileLimit;
   nOpenedFile = nFileCounterDiff > 0 ? nFileLimit : nSample;

   read_fpList = (FILE **) calloc (nOpenedFile, sizeof (FILE *));
   alloc_chk (WHERE, read_fpList);

   for (i = 0; i < nOpenedFile; i++)
   {
      len = sprintf (buffer, "./file_%d", i);
      buffer[len] = 0;

      read_fpList[i] = open_chkq (WHERE, buffer, "r");

   }
   if (nFileCounterDiff > 0)
   {
      pFilePosList = (int *) calloc (nFileCounterDiff, sizeof (int));
      alloc_chk (WHERE, pFilePosList);
   }

   nLineCounter = noOfIteration * noOfLocus * noOfStats;
   for (j = 0; j < nLineCounter; j++)
   {
      for (k = 0; k < nSample; k++)
      {
         pValList[k] = 0;
      }
      for (i = 0; i < nSample; i++)
      {
         double val = 0;
         if (i >= nFileLimit)
         {
            /* open the new file and set the current pos */
            len = sprintf (buffer, "./file_%d", i);
            buffer[len] = 0;
            read_fp = open_chkq (WHERE, buffer, "r");

            fseek (read_fp, pFilePosList[i - nFileLimit], SEEK_CUR);
            if (fscanf (read_fp, "%lf\n", &val) != EOF)
            {
               pValList[i] = val;
               pFilePosList[i - nFileLimit] = ftell (read_fp);
            }
            fclose (read_fp);
            read_fp = NULL;
         }
         else
         {
            if (fscanf (read_fpList[i], "%lf\n", &val) != EOF)
            {
               pValList[i] = val;
            }
         }
      }
      ReadFromList (pValList, nSample, &headPtr, &minval, &maxval);
      create_hostogram_table (headPtr, &histogramPtr, maxval, minval, nSample,
                              100);
      /*print_freq_dist(histogramPtr); */
      /* write the histogram table in file */
      writeToOutputFile (out_fp, histogramPtr);
      free_linked_list (headPtr);
      free_histogram (histogramPtr);
      headPtr = NULL;
      histogramPtr = NULL;
   }
   if (pFilePosList)
   {
      free (pFilePosList);
   }
   if (pValList)
      free (pValList);

   for (i = 0; i < nOpenedFile; i++)
   {
      fclose (read_fpList[i]);
   }
   free (read_fpList);

   if (out_fp)
   {
      fclose (out_fp);
   }
   for (i = 0; i < nSample; i++)
   {
      len = sprintf (buffer, "./file_%d", i);
      if (remove (buffer) == -1)
      {
         printf ("Failed to delete the file\n");
      }
   }
   return 1;
}
