/*******************************************************************************
  program name: lm_map (see Autozyg/MapTester/README_userdoc) for overview.
********************************************************************************/
/**
 * @defgroup lm_auto lm_auto
 * @ingroup autozyg
 * @{
 */

#define PNAM  "lm_map"

#include "autozyg.h"
#include "./Utils/map.h"
#include "setupAutozyg.c"
#include "./Utils/map_fncts.c"
#include "./Utils/map_ops.c"

int main (int argc, char *argv[])
{
   int   i, j, k, t, c;
   int   nloc;
   int   nsw;
   int   tot_count = 0;                 /* score by component and locus  */
   int ***recounts;                     /* [nc][nl][6] counts of recombs */
   double **tot_log_prob_seg;
   double **tot_log_prob_pen;
   double **temp_log_prob_seg;
   double **prob;
   FILE *oSeedF;

#include "./Utils/variable.defs"        /* defines variables 
                                           specific to lm_map       */
#if 0
   ChkIt = 2;                           /* parfile statement: check progress 
                                           MC iteration K (not implemented  
                                           for lm_map)                      */
#endif

   /* Allows peeling seqs to be entered at the          */
   /* command line, and toggles flag for error analysis */
   Awe = 0;
   if (argc > 2 && !(strcmp (argv[argc - 2], "seq")))
   {
      Awe = 1;
      peelseq = argv[argc - 1];
      argc = argc - 2;
   }

   nsw = setupAutozyg (argc, argv, PNAM, 0);

   nloc = NLoci - 1;                    /* nloc is the cardinal # of 
                                           markers for which marker
                                           phenotypes (or genotypes if 
                                           using no error) are provided   */
   Nlocy = 2 * nloc;

   alloc_loglik (&recounts, &tot_log_prob_seg, &temp_log_prob_seg,
                 &tot_log_prob_pen);
   tot_count = 0;

#include "./Utils/alloc.mem"            /* dynamically allocates memory  */

   alloc_logprob ();
   t = 0;

   if (!Awe)      /*--------ESTIMATE THE MAP W/0       -------------*/
   {
      prob = setupInherit (1);
      if (MapTyp == SEX_AVG || MapTyp == BOTH) /* SXA MAP W/O          */
      {
         pr_section (1);
         for (r = 0; r < Redo; r++)
         {
            get_init (0, simple_null, &Alpha, Awe);
            if (!r)
               pr_map (0, Marek);
            change_map (1, nsw, Marek, prob);
            size_flag = 0;

            n_scans = mcemit (0, size_flag, nsw, recounts,
                              temp_log_prob_seg, mle_sxa, prob);

            reset_globs (0, n_scans);

            if (!r)
            {
               sw_map (Marek, best);
               fish_inv_flag = 1;
            }
            else
               fish_inv_flag = 1 - comp_map (EM_bound * 2, mle_sxa, best);

            if (fish_inv_flag)          /* EST. INV OBS INFO: SXA W/O        */
            {
               fish_inv_flag = 0;

               obs_info (0, obs_info_mat, scount, recounts, nsw,
                         prob, temp_log_prob_seg, mle_sxa, NCurvIt, r);

               nnr = nloc - 1;
               d_invert (obs_info_mat, ymat, nnr);
               /* ymat holds the invs. obs. infor.      */
            }

            v = 0;                      /* PROPOSE INIT. VALUE FOR SA: MLE_SXA W/O       */
            size_flag = 1 - SA;
            llr_flag = 0;
            while (!size_flag)          /* while not close          */
            {
               v++;
               sait (0, v, recounts, nsw, prob, temp_log_prob_seg,
                     ymat, mle_sxa);

               size_flag = comp_map (SA_bound, mle_sxa, Marek);

               if (!(size_flag) && !(llr_flag))
                  /* if not close and not ascending, then estimate llr.
                     otherwise,  accept SA proposal                    */
               {
                  dn = mle_sxa;
                  accum_lr_flag = 1;
                  llr = 0;
                  sw_map (Marek, se);
                  while (accum_lr_flag)
                  {
                     accum_lr_flag = 0;
                     llr += logLR (se, dn, nsw, scount, recounts,
                                   prob, temp_log_prob_seg,
                                   &accum_lr_flag, NAscentIt);
                  }
                  if (llr > 0)
                  {
                     llr_flag = 1;
                     sw_map (mle_sxa, Marek); /* ACCEPT SA PROPOSAL */
                  }
                  else
                     sw_map (Marek, mle_sxa); /* DO NOT ACCEPT   */
                  change_map (1, nsw, Marek, prob);
               }
               else                     /* ACCEPT SA PROPOSAL IMMEDIATELY     */
               {
                  sw_map (mle_sxa, Marek);
                  change_map (0, nsw, Marek, prob);
               }
            }

            dn = best;                  /* COMP. CANDIDATE MLE -v- CURR.MLE: SXA W/O */
            accum_lr_flag = 1;
            llr = 0;
            sw_map (Marek, se);

            while (accum_lr_flag)
            {
               accum_lr_flag = 0;
               if (r == 0)
                  Glob_Max = 10;
               llr += logLR (se, dn, nsw, scount, recounts,
                             prob, temp_log_prob_seg, &accum_lr_flag, Glob_Max);
            }
            if (llr < 0 || r == 0)
               for (i = 0; i < 2; i++)
                  for (j = 1; j < nloc; j++)
                     best[i][j] = Marek[i][j];
            printf ("\n Interval   Sex-Averaged (RF)  ");
            printf ("Pass Number %3d", r + 1);
            printf ("\n --------   -----------------");
            pr_map (0, best);
         }

         sw_map (best, mle_sxa);
         change_map (1, nsw, mle_sxa, prob);
         printf ("\n Estimating the variance of the ");
         printf ("sex-averaged mle");

         obs_info (0, obs_info_mat, scount, recounts, nsw,
                   prob, temp_log_prob_seg, mle_sxa, Obs_Info_Avg, r);

         nnr = nloc - 1;
         d_invert (obs_info_mat, ymat, nnr);
         /*    ymat holds the invs. obs. infor.      */

         printf ("\n\n");
         for (c = 0; c < n_compnts; c++)
            pr_reco_counts (0, nloc, recounts[c]);

         printf ("\n\n\n       MAXIMUM LIKELIHOOD ESTIMATES\n");
         printf ("\n  Interval        Sex-Averaged (RF)");
         printf ("\n  --------        ----------------\n");
         for (j = 1; j < nloc; j++)
            printf ("    %d         %15.4f \n", j, mle_sxa[0][j]);
         printf ("\n\n ESTIMATED VARIANCE OF SEX-AVERAGED (RF)");
         printf ("\n-----------------------------------------------");

         for (inr = 1; inr <= nnr; inr++)
         {
            printf ("\n  ");
            for (jnr = 1; jnr <= nnr; jnr++)
            {
               if (ymat[inr][jnr] < 0)
                  printf (" %2.6f ", ymat[inr][jnr]);
               else
                  printf ("  %2.6f ", ymat[inr][jnr]);
            }
         }
         printf ("\n\n\n EFFECTIVE NUMBER OF MEIOSES\n");
         printf ("------------------------------\n");
         for (i = 1; i < nloc; i++)
            printf ("     Interval %d: %1.0f \n", i,
                    ceil (mle_sxa[0][i] * (1 - mle_sxa[0][i]) / ymat[i][i]));
         printf ("\n\n");
      }
      /*============================================================*/
      if (MapTyp == SEX_SPEC || MapTyp == BOTH) /*  SXS MAP: W/O        */
      {
         pr_section (2);

         for (r = 0; r < Redo; r++)
         {
            get_init (1, simple_null, &Alpha, Awe);
            if (!r)
               pr_map (1, Marek);
            change_map (1, nsw, Marek, prob);
            size_flag = 0;

            n_scans = mcemit (1, size_flag, nsw, recounts,
                              temp_log_prob_seg, mle_sxs, prob);

            reset_globs (1, n_scans);

            if (!r)                     /*  BEGIN CRUDE ESTIMATE OF INV OBS INFO SXS W/O */
            {
               sw_map (Marek, best);
               fish_inv_flag = 1;
            }
            else
               fish_inv_flag = 1 - comp_map (EM_bound, mle_sxs, best);
            if (fish_inv_flag)          /* EST. INV OBS INFO: SXS W/O        */
            {
               obs_info (1, obs_info_mat, scount, recounts, nsw,
                         prob, temp_log_prob_seg, mle_sxs, NCurvIt, r);

               nnr = 2 * nloc - 2;

               d_invert (obs_info_mat, ymat, nnr);
            }

            v = 0;                      /* PROPOSE INIT. VALUE FOR SA: MLE_SXS W/O       */
            size_flag = 1 - SA;
            llr_flag = 0;
            while (!size_flag)
            {
               v++;
               sait (1, v, recounts, nsw, prob, temp_log_prob_seg,
                     ymat, mle_sxs);

               size_flag = comp_map (SA_bound, mle_sxs, Marek);
               if (!(size_flag) && !(llr_flag)) /* if not close nor ascent   */
               {
                  dn = mle_sxs;
                  accum_lr_flag = 1;
                  llr = 0;
                  sw_map (Marek, se);

                  while (accum_lr_flag)
                  {
                     accum_lr_flag = 0;
                     llr += logLR (se, dn, nsw, scount, recounts,
                                   prob, temp_log_prob_seg,
                                   &accum_lr_flag, NAscentIt);
                  }
                  if (llr > 0)
                  {
                     llr_flag = 1;
                     sw_map (mle_sxs, Marek);
                  }
                  else
                     sw_map (Marek, mle_sxs);

                  change_map (1, nsw, Marek, prob);
               }
               else
               {
                  size_flag = comp_map (SA_bound, mle_sxs, Marek);
                  sw_map (mle_sxs, Marek);
                  change_map (0, nsw, Marek, prob);
               }
            }

            dn = best;                  /* COMP. CANDIDATE MLE -v- CURR.MLE: SXS W/O */
            accum_lr_flag = 1;
            llr = 0;
            sw_map (Marek, se);

            while (accum_lr_flag)
            {
               accum_lr_flag = 0;
               if (r == 0)
                  Glob_Max = 10;
               llr += logLR (se, dn, nsw, scount, recounts,
                             prob, temp_log_prob_seg, &accum_lr_flag, Glob_Max);
            }
            if (llr < 0 || r == 0)
               for (i = 0; i < 2; i++)
                  for (j = 1; j < nloc; j++)
                     best[i][j] = Marek[i][j];
            printf
               ("\n Interval     Female (RF)     Male (RF)  Pass Number %3d",
                r + 1);
            printf ("\n --------   --------------   -----------");
            pr_map (1, best);
         }

         sw_map (best, mle_sxs);

         change_map (1, nsw, mle_sxs, prob);
         printf ("\n Estimating the variance of the sex-specific mle");

         obs_info (1, obs_info_mat, scount, recounts, nsw,
                   prob, temp_log_prob_seg, mle_sxs, Obs_Info_Spc, r);

         nnr = 2 * nloc - 2;
         d_invert (obs_info_mat, ymat, nnr);
         /*    ymat holds the invs. obs. infor.      */

         printf ("\n\n");
         for (c = 0; c < n_compnts; c++)
            pr_reco_counts (0, nloc, recounts[c]);

         printf ("\n");
         printf ("\n\n       MAXIMUM LIKELIHOOD ESTIMATES\n");
         printf ("\n  Interval    Female (RF)         Male (RF)");
         printf ("\n  --------    -----------         ---------\n");
         for (j = 1; j < nloc; j++)
            printf ("     %d     %12.4f   %15.4f \n", j, mle_sxs[0][j],
                    mle_sxs[1][j]);
         printf ("\n\n");
         printf (" ESTIMATED VARIANCE OF SEX-SPECIFIC (RF) ");
         printf ("[F1,M1,F2,M2,... x F1,M1,F2,M2,...]\n");

         for (inr = 1; inr <= nnr; inr++)
         {
            printf ("\n  ");
            for (jnr = 1; jnr <= nnr; jnr++)
            {
               if (ymat[inr][jnr] < 0)
                  printf (" %1.4f ", ymat[inr][jnr]);
               else
                  printf ("  %1.4f ", ymat[inr][jnr]);
            }
         }

         printf ("\n\n EFFECTIVE NUMBER OF MEIOSES");
         printf ("\n\n Interval  Female      Male");
         printf ("\n--------  --------    -------\n");
         for (i = 1; i < nloc; i++)
         {
            j = 2 * i - 1;
            printf ("     %d: %7.0f  ", i,
                    ceil (mle_sxs[0][i] * (1 - mle_sxs[0][i]) / ymat[j][j]));
            printf ("  %7.0f \n",
                    ceil (mle_sxs[1][i] * (1 - mle_sxs[1][i]) /
                          ymat[j + 1][j + 1]));
         }
      }

     /*=============================================================*/
      printf ("\n Estimating the likelihood ratio statistic(s)");
      vv = NChiSqIt / 2;                /*  marks progress of total cpu time  */
      tests = 1;
      dn = mle_sxa;
      accum_lr_flag = 1;
      llr = 0;
      for (j = 1; j < nloc; j++)
      {
         Marek[0][j] =
            rf (.5 * (cmH (simple_null[0][j]) + cmH (simple_null[1][j])));
         Marek[1][j] = Marek[0][j];
      }
      sw_map (Marek, se);

      if (MapTyp == BOTH)
         tests = 2;

      for (lrt = 0; lrt < tests; lrt++)
      {
         if (MapTyp == SEX_SPEC && lrt == 0)
         {
            dn = mle_sxs;
            accum_lr_flag = 1;
            llr = 0;
            sw_map (simple_null, se);
         }
         if (lrt)
         {
            dn = mle_sxs;
            accum_lr_flag = 1;
            llr = 0;
            sw_map (mle_sxa, se);
         }
         while (accum_lr_flag)
         {
            accum_lr_flag = 0;

            llr += logLR (se, dn, nsw, scount, recounts,
                          prob, temp_log_prob_seg, &accum_lr_flag, NChiSqIt);
         }
         chi_square[lrt] = 2 * llr;
      }
      switch (MapTyp)
      {
          case SEX_AVG:
             printf
                ("\n\n Estimate of twice the log likelihood ratio of the\n");
             printf (" sex-averaged mle to the averaged given map: ");
             printf ("%6.4f\n", chi_square[0]);
             break;

          case SEX_SPEC:
             printf ("\n\n Estimate of twice the log likelihood ratio \n");
             printf (" between the sex-specific mle and the given map: ");
             printf ("%6.4f\n", chi_square[0]);
             break;

          case BOTH:
             printf
                ("\n\n Estimate of twice the log likelihood ratio of the\n");
             printf (" sex-averaged mle and the averaged given map: ");
             printf ("%6.4f\n", chi_square[0]);

             printf
                ("\n Estimate of twice the log likelihood ratio between \n");
             printf (" the sex-specific and the sex-averaged ML estimates: ");
             printf ("%7.4f\n", (chi_square[1]));
             break;

          default:
             printf ("\n An ERROR occurred in estimating: 2*log(L(.,.))\n");
             break;
      }
   }

   printf ("--------------------------------------------------------\n\n");
   fflush (stdout);

   if (SamplSeedsOut)
   {
      getSeeds (SAMPL);
      if (!(oSeedF = open_fil (OSEED)))
         give_up (ERR_qit);
      writSeeds (oSeedF);
      fclose (oSeedF);
   }

#include "Utils/free.mem"               /* frees all allocated memory */

   exit (EXIT_SUCCESS);
}
