/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */
/**************************************************************************
program name: ibddrop

      Program to simulate IBD to estimate probs
      The state (X) consists of segregation indicators--

         X determines the IBD pattern among the specified descendant gametes.
         Up to 10 gametes can be scored for IBD

      Two methods of scoring IBD -- 
          (1) by a specified (group of) pattern(s) over windows, and
          (2) by pattern marginally over loci

      Original program modified from jntauto,         Fall 1999, EAT
      New scoring routines addedi                     Fall,1999
      Combined into single program,                   May 2000, EAT

      Cleaning up and reformatting the code.
      Unused variables deleted and pointers initialized.
      Can now be built with MORGAN V2.3 libraries.    August 2000, N.L.

      Modified for Morgan parameter statements and setup by proc_pars()
      and proc_auto()                                 Mar 2002, MAJ

***************************************************************************/
/* $Id: ibddrop.c,v 1.16 2005/08/14 00:19:28 thompson Exp $ */
/**
 * @defgroup ibddrop ibddrop
 * @ingroup GDrop
 * @{
 */

#define PNAM "ibddrop"
#define IBD

#include "genedrop.h"
#include "genedrop_fncs.c"

int main (int argc, char **argv)
{
   int   j, k, sx, s, jj, t, c, ng, pt;
   double ss;

   int   nsw;                           /* no. of switches                    */

   double ****nscore;                   /* (1) [nc][ns][nw][wpt]  ibd counts:
                                         *       ea component, ea scoreset,
                                         *       ea window, ea pattern
                                         * (2) [nc][ns][nl][pt]  ibd counts:
                                         *       ea component, ea scoreset,
                                         *       ea locus, ea pattern         */

   int   nwpt;                          /* (1) no. of patterns                */
   int  *gw;                            /* (1) [w]                            */
   int  *wloc;                          /* (1) [nl] binary vector score over loci */

   int  *gaga;                          /* (2) [g]                            */
   int  *npt;                           /* (2) [g+1]  no. of patterns 
                                           for gametes                        */
   int **index;                         /* (2) [g+1][cd+1]                    */
   int **label;                         /* (2) [g+1][pt+1]                    */
   int **list;                          /* (2) [g+1][pt+1]                    */
   int **count;                         /* (2) [nc][nl]  no. of steps
                                           on ea locus on ea component
                                           (NIt used instead)                 */
   double *sss;                         /* (2) [nl]                           */

   /* where,                               (1)  variables for window over loci
    *                                           (method 1)
    *                                      (2)  variables for pattern over loci 
    *                                           (method 2)
    *                                      w   - no. of loci in window 
    *                                      g   - no. of gametes in largest set
    *                                      nc  - no. pedigree components
    *                                      ns  - no. of gamete sets
    *                                      pt  - no. of patterns
    *                                      nl  - no. of loci
    *                                            (Nloci, which is NMarks + 1)
    *                                      nw  - no. of windows over loci     */

   FILE *oSeedF;

   nsw = setupGenDrp (argc, argv, PNAM, 0);

   printf (" Simulation recombinations:\n");

   for (sx = FEM; sx <= MAL; ++sx)
   {
      if (sx == MAL)
         printf ("    M: ");
      if (sx == FEM)
         printf ("    F: ");

      for (j = 0; j < NMarks; ++j)
      {
         if (!(j % 7) && j)
            printf ("\n       ");
         printf ("%3d%s", chromo[0][j], dub2nlz (xecomo[sx][0][j], 7, 3));
      }
      printf ("%3d\n", chromo[0][NMarks]);
   }
   printf ("\n");
   fflush (stdout);

   /* check probands for gene scoring  */
   /* if the last component for which there is a scoreset is less than the
      number of components read, reset the number of components to score      */

   NScCmp = set_probgam (n_compnts);

   if (NLocWndw)
   {
      /* alloc and init locus window arrays  */

      for (j = 0, nwpt = 1; j < NLocWndw; ++j)
         nwpt *= 2;

      alloc_wndw (&gw, &wloc, &nscore, NWndw, nwpt);

      for (c = 0; c < NScCmp; ++c)
         for (s = 0; s < NScSet[c]; ++s)
            for (j = 0; j < NWndw; ++j)
               for (pt = 0; pt < nwpt; ++pt)
                  nscore[c][s][j][pt] = 0.0;
   }
   else
   {
      /* alloc and init ibd pattern arrays  */

      alloc_ibd (&gaga, &index, &label, &list, &npt, &count, &nscore, &sss);

      for (c = 0; c < NScCmp; ++c)
         for (s = 0; s < NScSet[c]; ++s)
            for (j = 0; j < NLoci; ++j)
               for (pt = 0; pt <= PTgg[NPrGam[c][s]]; ++pt)
                  nscore[c][s][j][pt] = 0.0;
   }

   /* realise the initial switches if not already read above */

   realise (NLoci - 1, xecomo, nsw);

   /* set genes (gg0, gg1) from initial switches */

   for (j = 0; j < NLoci; ++j)
      set_FGL (j);

   if (NLocWndw)
   {
      /* count total number of patterns - scoring over loci - 2**window-size  */

      printf (" %d pattern sets are to be scored for each proband gamete "
              "pair\n\n", nwpt);
      printf (" Autozygosity will be scored over %d windows of %d loci\n\n",
              NWndw, NLocWndw);
   }
   else
   {
      /* count total number of patterns */

      for (j = 1; j <= MxGam; ++j)
         npt[j] = index_ibd (j, 0, 0, index[j], label[j], list[j], NULL);

      pr_npatsets (npt);
   }

   /* now test switches and do MC sampling, NIt iterations */

   for (t = 0; t < NIt; ++t)
   {
      realise (NLoci - 1, xecomo, nsw);

      for (j = 0; j < NLoci; ++j)
      {
         jj = chromo[0][j];
         set_FGL (jj);
      }
      for (c = 0; c < NScCmp; ++c)
      {
         if (NLocWndw)
         {
            /* score ibd pattern over window in compt c */

            for (s = 0; s < NScSet[c]; ++s)
            {
               for (j = 0; j < NLoci; ++j)
                  wloc[j] = ibd_pattern (j, ProbIx[c][s], ProbGam[c][s]);

               for (j = 0; j < NWndw; ++j)
               {
                  pt = bin_pattern (j, NLocWndw, wloc);
                  ++nscore[c][s][j][pt];
               }
            }
         }
         else
         {
            /* score ibd pattern over loci in compt c */

            for (j = 0; j < NLoci; ++j)
            {
               jj = chromo[0][j];
               for (s = 0; s < NScSet[c]; ++s)
                  (void) score_genes (jj, NPrGam[c][s], gaga,
                                      index[NPrGam[c][s]], list[NPrGam[c][s]],
                                      ProbIx[c][s], ProbGam[c][s],
                                      nscore[c][s][j], 0, wloc);
            }
         }
      }
   }

   /* now print the ibd probabilities for each component */

   for (c = 0; c < NScCmp; ++c)
   {
      if (NScSet[c])
      {
         pr_cmpnthead (c);

         if (NLocWndw)
         {
            /* for ea set, print probabilities for ea pattern for ea window  */

            pr_prbihead ();

            for (s = 0; s < NScSet[c]; ++s)
            {
               /* format and print headings  */

               pr_pgamset (c, s);

               pr_wndwhead (NWndw);

               /* format and print data  */

               for (pt = 0; pt < nwpt; ++pt)
               {
                  ss = 0.;
                  for (j = 0; j < NWndw; ++j)
                     if (nscore[c][s][j][pt] > ss)
                        ss = nscore[c][s][j][pt];
                  if (ss / NIt > 0.0001)
                  {
                     binary (pt, NLocWndw, gw);
                     pr_wndwprob (c, s, NWndw, pt, gw, nscore);
                  }
               }
               printf ("\n");
            }
         }
         else
         {
            /* for ea set, print probabilities for ea pattern for ea window  */

            for (s = 0; s < NScSet[c]; ++s)
            {
               /* format and print headings  */

               pr_probhead ();
               pr_pgamset (c, s);

               pr_lochead (chromo);

               /* format and print data  */

               ng = NPrGam[c][s];
               for (k = 0; k < npt[ng]; ++k)
               {
                  ss = 0.0;
                  pt = list[ng][k];
                  for (j = 0; j < NLoci; ++j)
                  {
                     sss[j] = nscore[c][s][j][pt] / NIt;
                     if (sss[j] > ss)
                        ss = sss[j];
                  }
                  if (ss > 0.0001)
                  {
                     invert_label (label[ng][k], ng, gaga);
                     pr_locprob (ng, gaga, sss, label[ng][k]);
                  }
               }
               printf ("\n");
            }
         }
      }
   }
   fflush (stdout);

   if (SamplSeedsOut)
   {
      getSeeds (SAMPL);
      if (!(oSeedF = open_fil (OSEED)))
         give_up (ERR_qit);
      writSeeds (oSeedF);
      fclose (oSeedF);
   }
   free_chromo (0);
   free_xxgg ();

   if (NLocWndw)
      free_wndw (&gw, &wloc, &nscore, NWndw, nwpt);
   else
      free_ibd (&gaga, &index, &label, &list, &npt, &count, &nscore, &sss);

   exit (EXIT_SUCCESS);
}

/** @} */
