/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */
int   setupMarkerdrop (int, char **, char *, int);

static double **quant = NULL;

/**
 * Calls functions to do the setup steps for the Morgan programs
 * which use a Monte Carlo method.
 * This includes setting up parameters and data structures for the
 * marker and trait model,
 * reading the pedigree file and setting up the pedigree structure,
 * storing observed marker and trait data for pedigree members.
 */
int setupMarkerdrop (int argc, char **argv, char *ppn, int nh)
{
   int   nSwtch;

   /* initialize exit procedure, print headings. 
    * fncs:  atexit(), print_head() 
    * dirs:  Stuff 
    */
   initMorgProg (argv);

   /* process parameter statements, set up globals resulting from these.
    * fncs:  proc_pars(), prntStatusPP(), proc_auto(), proc_fils(), 
    *        proc_seeds(), prntStatusCP() 
    * vars:  globals in globs.h
    * dirs:  Pars, Stuff
    */
   procParStat (argc, argv, ppn);

   /* check, and massage if necessary, pedigree file.
    * fncs:  chk_pedfil(), prntStatusCPF(), mak_opedfil()
    * vars:  indKeys[PedSize], compntList[nCompnt]
    * dirs:  PedComp, Stuff
    */
   chkPedFil ();

   /* read in pedigree data and set up pedigree structure.
    * fncs:  alloc_ped(), open_ped(), input_ped(), nghds(), count_founders()
    * vars:  int data[PedSize][NColData], Ind nghd[PedSize+1]
    *        double quant[NTraits][PedSize], Ind nghd[].q_dat[2*(NMarks+1)]
    * dirs:  Nghds, Quant, Stuff
    */
   setPed (quant);

   nSwtch = setSwitches ();

   /* set observed marker and trait data for pedigree members.
    * fncs:  set_markdat(), set_traitdat(), prnt_traitdat(),
    *        set_affind(), set_qdat()
    * vars:  Ind nghd[].g_dat[2*(NMarks+1)],
    *        Ind nghd[].d_dat[2*(NMarks+1)] 
    * dirs:  Markers
    */
   setObsDat ();

   /* set up chromosome map for markers and trait. */
   /* fncs:  alloc_chromo(), set_chromosomes()
    * vars:  int chromo[nh+1][NMarks+1], revchro[nh+1][NMarks+1],
    *        double xecomo[2][nh+1][NMarks+1]
    * dirs:  Markers
    */
   setMaps (nh);

   /* set up type of each locus.
    * fncs:  set_traits(),  print_traits() 
    * vars:  Trait traits[NMarks+1] 
    * dirs:  Markers
    */
   setLocTyp ();

   /* set up data structures for locus and meiosis sampling. 
    * fncs:  alloc_xxgg(), allocate_loci(), set_quant_trait(), alloc_msmpl()
    * vars:  int xx0[NMarks+1][PedSize], xx1[NMarks+1][PedSize] 
    *        int gg0[NMarks+1][PedSize], gg1[NMarks+1][PedSize]
    *        Locus loci[NMarks+1].*  
    *        int new_segs[NMarks+1],
    *        double pp2[NMarks][2], pcf [NMarks][2]
    * dirs:  Sample
    */
   setSampling ();

   /* initialize seeds for rans().
    * vars:  pSeedC, pSeedT   
    * dirs:  Rans
    */
   setSeeds (TRAIT);

   return nSwtch;
}
