/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */
/*******************************************************************************
 program name: lm_lods.c

 Program to do lod scores (replaces homozyg.c from old Autozyg)
 The state (X) consists of segregation indicators -- so can generalize
    to cases of more than one observed/pedigree, and multiple loci.

 The data, Y, is the multilocus genotypes of observed individuals
    in the pedigree.

 Obtained by modification of jnt_auto.c                            EAT Dec 1998

 Here is the plan!
   OK this is the PLAN  (12/15/98)

 OK, now we have a general autozyg program, we need to go back to
 lod estimation, for those as want.  We use the same program (modified).

 For k markers, we will have (k+2) simulation points, and (k+2) evaluation
 points  (but maybe it is not worth evaluating all for all).
 To start we will just evaluate relative to a left-point and a right-point
 for each simulation point.

 The simulation points are mid-point of each interval, and r=0.1 to each
 end of chromosome (0.1 can be adjusted later), and "unlinked" which is
 0.5 to either end.

 Since the data are locus-by-locus, the likelihood ratio contribution is
 just ratio of P_theta(S)/P_sim_th(S).

 This is very easy, once we get the chromosome set-ups fixed.

 1) Trait locus data set-up -- assume trait-locus genotypes for observed
 individuals -- done  12/20/98
 2) Likelihood-ratio contributions routine,  -- done 12/18/98
 3) Simulation and evaluation chromosomes for each location. -- done 12/23/98

 4) Need a test run!! -- JV4C maybe.   working! 12/24/98

 New routines:  (12/19/98)
    full_seg_prob() and full_pen_seg_lod() and lik_rat()
       (all in full_logprob.c)
    full_seg_prob modified from old routine -- returns log(prob P(S))
       of all meioses on given component pedigree
    full_pen_seg_lod computes full penetrance P(Y|S) for data y, and
       given meiosis indicators, on given component pedigree.
          calls full_lod and returns full log-probability of (Y,S).
       [lod_next routine is not currently used ]

 Now to integrate the L-sampler, using the new Lsampler and Msampler
    libraries and new routines;                                    EAT Mar 2000

 Converted to MORGAN_V2.3                                          MAJ Jun 2000

 Modifications to specify simulation points, within each interval and
 outside of map.                                                   EAT Nov 2000
 Then converted to MORGAN 2.3                                      MAJ Nov 2000

 Modification of full_seg_prob, and recombinant counts             EAT Nov 2000
    scoring of counts and tot_log_liks by component and locus/interval.

 Modifications to lik_rat, do_lsmpl, etc.                          EAT Dec 2000

 Modifications to use specify intervals for l-ratio updating and use
    more efficient seg_prob updating routines for l-sample steps   EAT Jan 2001

 Added changes for sequential imputation set-up                    AWG Mar 2001

 Converted fixed arrays to dynamically allocated (and freed) arrays
                                                                   MAJ Aug 2001
*******************************************************************************/

/**
 * @defgroup lm_lods lm_lods
 * @ingroup lodscore
 * @{
 */

#define PNAM  "lm_lods"
#define LODS

#include "lodscore.h"
#include "lods_chk.c"
#include "setupAutozyg.c"

int main (int argc, char *argv[])
{
   int   i, j, k;
   int   t, c, h;
   int   nloc;
   int   nsw, sx;
   int  *scounts;                       /* [nc]         */
   int ***recounts;                     /* [nc][nl][6]  */
   int **lsflag;                        /* [3][nl]      flags for lsampler
                                         *              l_ratio adjustments */
   double **l_ratio;                    /* [nc][3]      now this is the
                                         *              LOG-l-ratio EAT 12/00 */
   double **prob;                       /* [nc][nl]    */
   double **cumulik;                    /* [nc][3]     */
   double **temp_log_prob_seg, **temp2_log_prob_seg; /* [nc][nloc]  */
   double **tot_log_prob_seg, **tot_log_prob_pen; /* [nc][nloc]  */
   double **tot_log_left_seg, **tot_log_rght_seg; /* [nc][nloc]  */
   int   tot_count;

   double ***likeratios = NULL;         /* Matrix for likelihood ratios */
   double **lods = NULL;                /* Lods scores */
   int   method = 0;
   double leftratio = 0.0;
   double rightratio = 0.0;

   int   restart_status = 0;

   double rec0[3] = { .5, .5, .5 };
   int   loc0[3] = { 0, 0, 0 };

   FILE *oSeedF;

   /* .................................................. */
   /* nh, the last arg, is 2 */

   nsw = setupAutozyg (argc, argv, PNAM, 2);
   prob = setupInherit (0);

   /* .................................................. */

   nloc = NMarks;

   /* Allocate memory for likeratios and lods scores */
   lods = alloc_dbl2d (3, NTPos);
   likeratios = alloc_dbl3d (n_compnts, NTPos1, NTPos1);

   alloc_loglik (&recounts, &tot_log_prob_seg, &temp_log_prob_seg,
                 &tot_log_prob_pen);
   alloc_lod (&scounts, &lsflag, &l_ratio, &cumulik,
              &temp2_log_prob_seg, &tot_log_left_seg, &tot_log_rght_seg);

   for (h = 1; h <= 2; ++h)             /* for lm_lods ls_flag is now ==1 */
      for (j = 0; j < nloc; ++j)
         lsflag[h][j] = 1;

   alloc_lpflag (nloc, 2);
   alloc_logprob ();

   /* try to restart if a rescue file is specified */
   k = 0;
   restart_status = lods_restart (likeratios, &k);

   if (restart_status == 2)
   {                                    /* restart failed */
      k = 0;
      fprintf (stderr, "A rescue file was specified but couldn't be opened.\n");
      restart_status = 0;
   }

   if (restart_status == 0)
   {                                    /* not restarting */
      prntStatusBMC ();                 /* print status for beginning MCMC */

      /* now do burn-in iterations */

      printf (" Now starting %d iterations of burn-in\n", NBrnIt);
      for (t = 1; t <= NBrnIt; t++)
      {
         if (!(t % ChkIt))
         {
            printf (" Starting burnin iteration %d\n", t);
            fflush (stdout);
         }
         sampling (t, 0, nloc, nsw, nFGL, prob);
      }
   }
   else
   {
      printf (" Restarting.  Skip burn-in.\n");
      fflush (stdout);
   }

   /*                                            *** now start BIG loop *** */

   for (; k < NTPos1; ++k)
   {
      printf ("\n CHROMOSOME SET %d\n\n", k);
      fflush (stdout);

      /* two evaluation sets for each simulation chromo (left/right) */

      printf (" Recombination frequencies:\n\n");

      for (sx = FEM; sx <= MAL; ++sx)
      {
         loc0[0] = IntVal[k];
         rec0[0] = RecVal[sx][k];

         if (k == 0)
         {
            loc0[1] = nloc;
            rec0[1] = RecVal[sx][NTPos];
         }
         else
         {
            loc0[1] = IntVal[k - 1];
            rec0[1] = RecVal[sx][k - 1];
         }

         loc0[2] = IntVal[k + 1];
         rec0[2] = RecVal[sx][k + 1];

         set_chromosomes (nloc, Marek[sx], 2, loc0, rec0, sx); /* nh is 2  */

         set_lpflag (nloc, 2, sx);

         for (i = 0; i <= 2; ++i)
         {
            if (i == 0)
               printf ("    sim,   ");
            else if (i == 1)
               printf ("    left,  ");
            else
               printf ("    right, ");
            if (sx == MAL)
               printf ("M:");
            if (sx == FEM)
               printf ("F:");
            for (j = 0; j < nloc; ++j)
            {
               if (!(j % 6) && j)
                  printf ("\n             ");
               printf ("%3d%s", chromo[i][j], dub2nlz (xecomo[sx][i][j], 7, 3));
            }
            printf ("%3d\n", chromo[i][nloc]);
         }
      }
      printf ("\n");

      /* set locus positions for l-sampler  -- comment corrected EAT 2005 */

      set_positions (nloc);             /* EAT  March 2000    */

      /* now should do an L-sample step on the trait locus (0) to reset  */

      (void) do_lsmpl (nloc, 0, prob, nFGL);

      for (c = 0; c < n_compnts; ++c)
      {
         for (h = 0; h <= 2; ++h)
         {
            l_ratio[c][h] = 0.0;
            cumulik[c][h] = 0.0;
         }

         for (j = 0; j <= nloc; ++j)
         {
            tot_log_prob_seg[c][j] = 0.0;
            tot_log_left_seg[c][j] = 0.0;
            tot_log_rght_seg[c][j] = 0.0;
            tot_log_prob_pen[c][j] = 0.0;
         }
         scounts[c] = 0;
         for (j = 0; j <= nloc; ++j)
         {
            for (h = 0; h < 6; ++h)
               recounts[c][j][h] = 0;
         }
      }
      tot_count = 0;

      /* set initial l_ratio    */

      for (c = 0; c < n_compnts; ++c)
      {
         l_ratio[c][1] = 0.0;
         l_ratio[c][2] = l_ratio[c][1];

         for (h = 1; h <= 2; ++h)
            l_ratio[c][h] +=
               diff_seg_prob (nloc, c, h, lsflag[h], temp_log_prob_seg);

         /* now do MCMC resampling by meiosis or by locus */

         for (t = 0; t < NIt; ++t)
         {
            sampling (t, 0, nloc, nsw, nFGL, prob);
            /* MCMC sampling step or scan */

            if (!((t + 1) % ScorIt))
            {                           /* score likelihood ratio contributns */
               for (c = 0; c < n_compnts; ++c)
               {
                  l_ratio[c][1] = 0.0;
                  l_ratio[c][2] = l_ratio[c][1];

                  for (h = 1; h <= 2; ++h)
                  {
                     l_ratio[c][h] += diff_seg_prob (nloc, c, h, lsflag[h],
                                                     temp_log_prob_seg);
                     cumulik[c][h] += exp (l_ratio[c][h]);
                  }
                  ++scounts[c];
               }

               /* score tot log-liks & recombs */
               for (c = 0; c < n_compnts; ++c)
               {
                  for (j = 0; j <= nloc; ++j)
                     tot_log_prob_pen[c][j] += log (prob[c][j]);

                  full_seg_prob (nloc, c, recounts, temp_log_prob_seg);
                  for (j = 0; j < nloc; ++j)
                     tot_log_prob_seg[c][j] += temp_log_prob_seg[c][j];

                  (void) diff_seg_prob (nloc, c, 1, lsflag[1],
                                        temp2_log_prob_seg);
                  for (j = 0; j < nloc; ++j)
                     tot_log_left_seg[c][j] += temp2_log_prob_seg[c][j] +
                        temp_log_prob_seg[c][j];

                  (void) diff_seg_prob (nloc, c, 2, lsflag[2],
                                        temp2_log_prob_seg);
                  for (j = 0; j < nloc; ++j)
                     tot_log_rght_seg[c][j] += temp2_log_prob_seg[c][j] +
                        temp_log_prob_seg[c][j];
               }
               ++tot_count;
            }
         }
      }
      /* now we are done with this simulation/estimation for this trait locus
         position  */
      /* output results for this simulation/estimation set */

      for (c = 0; c < n_compnts; ++c)
      {
         pr_cmpnthead (c);

         if (scounts[c] == 0)
            printf ("    No likelihood ratios because sampling count = 0\n\n");
         else
         {
            leftratio = cumulik[c][1] / scounts[c];
            rightratio = cumulik[c][2] / scounts[c];
            printf ("    Cumulative likelihood ratios (sampling count %d)\n",
                    scounts[c]);
            printf ("       estimated ratios:   left:sim = %6.4f,"
                    "  right:sim = %6.4f\n\n", leftratio, rightratio);
         }
         /* Fill in the matrices of likehood ratios */
         /* initialization first */

         for (j = 0; j < NTPos1; ++j)
            likeratios[c][k][j] = 0.0;
         if (k == 0)
         {
            likeratios[c][0][NTPos] = leftratio;
            likeratios[c][0][1] = rightratio;
         }
         else if (k == NTPos)
         {
            likeratios[c][k][k - 1] = leftratio;
            likeratios[c][k][0] = rightratio;
         }
         else
         {
            likeratios[c][k][k - 1] = leftratio;
            likeratios[c][k][k + 1] = rightratio;
         }

         pr_log_prob_segl (0, nloc, -cmplist[c + 1].nmeioses,
                           tot_log_left_seg[c], tot_log_prob_seg[c],
                           tot_log_rght_seg[c], tot_count);

         pr_log_prob_pen (0, nloc, tot_log_prob_pen[c], tot_count);

         pr_reco_counts (0, nloc, recounts[c]);
      }

      /* dumping restart information */
      lods_dump (likeratios, k);
   }                                    /* ****    end of BIG loop   **** */

   prntStatusFMC ();                    /* print status for finished with mcmc */

   /* now we are done with simulating for all nloc+2 simulation sets */

   printf ("\n ESTIMATED LOD SCORES\n\n");

   for (c = 0; c < n_compnts; ++c)
   {
      double lodsval[4];
      printf ("  Component %3d\n", c + 1);
      for (method = 0; method < 3; ++method)
      {
         double eigen2 = 0.0;
         lodsval[method] = comput_lods (likeratios[c], lods[method],
                                        NTPos, method, &eigen2);
         if (method == 0)
            lodsval[3] = eigen2;
      }
      printf ("\n    The largest eigenvalue        :%9.5f\n", lodsval[0]);
      printf ("\n    The second largest eigenvalue :%9.5f\n", lodsval[3]);
      printf ("\n    Cumulative from left          :%9.5f\n", lodsval[1]);
      printf ("\n    Cumulative from right         :%9.5f\n", lodsval[2]);

      prntLodSHead ();

      for (k = 0, i = 0; k < NTPos + NMarks; ++k)
      {
         if (!prntLodSPos (k))
            printf (" %10s %10s %10s", "NA", "NA", "NA");
         else
         {
            for (method = 0; method < 3; ++method)
            {
               if (-lods[method][i] >= DBL_MAX || lods[method][i] >= DBL_MAX)
                  printf (" %10s", "*****");
               else
                  printf (" %10.5f", lods[method][i]);

            }
            ++i;
         }
         printf ("\n");
      }
      printf ("\n");
   }
   fflush (stdout);

   /* Free the memory */
   free_2d ((void *) &lods, 3);
   free_3d ((void *) &likeratios, n_compnts, NTPos1);

   /* Free up allocated memory */
   finish_dumping ();
   free_xxgg ();
   locus_peel_free ();
   free_allocate_loci ();
   free_loglik (&recounts, &tot_log_prob_seg, &temp_log_prob_seg,
                &tot_log_prob_pen);
   free_lod (&scounts, &lsflag, &l_ratio, &cumulik,
             &temp2_log_prob_seg, &tot_log_left_seg, &tot_log_rght_seg);
   free_lpflag (2);
   free_chromo (2);
   free_logprob ();
   free_msmpl ();
   free_traits ();
   free_gen_pen ();
   free_seg_pen (nloc + 1, &prob);

   if (SamplSeedsOut)
   {
      getSeeds (SAMPL);
      if (!(oSeedF = open_fil (OSEED)))
         give_up (ERR_qit);
      writSeeds (oSeedF);
      fclose (oSeedF);
   }
   if (Scan)
      free_perm ();

   exit (EXIT_SUCCESS);
}

/** @} */
