/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */
/*  vim: sw=3
 *  This is the file schnell2.c, modified from Simon's loki_test.c to include
 *  the multi_point lod estimation from Shi Li's multi point stuff.  Modified
 *  6/99 GLS this version also includes the joint segregation stuff.
 *
 *  $Id: lm_schnell.c,v 1.17 2005/10/18 23:44:13 myrna Exp $
 */

#define PNAM "lm_schnell"
#define SCHNELL

#include "lodscore.h"
#include "setupAutozyg.c"

/* Header files current directory */
#include "mixds2.c"
#include "shnl_chk.c"

void  free_schnell (int);

int main (int argc, char *argv[])
{
   /* Global varibles */
   extern struct Locus *loci;           /* lsample.h */
/* extern double *ym;                  lsample.h */
   /* Centered trait values */

   /* {{{ Local variable */
   FILE *oSeedF;                        /* Output seed file */

   int   i, l, j, k, jj, c, h, t;

   int   trait_loc0[] = { 0, 0, 0 };
   double trait_rec0[] = { 0.5, 0.5, 0.5 };

   /* The positions of the current trait locus.  Lod score is computed
    * of the mid one relative to the left and right.  Hence 3 positions.
    * See set_chromosomes() in Msample/set_chromosome.c. */
   int   sx = 0;
   /*  variables related to list_switch() */
   int   nsw = 0;
   int  *perm_sw = 0;
   int  *perm_loc = 0;                  /* permutation arrays of switches
                                         * and loci  */
   double mu = 0.0;                     /* Global mean       */

   double *additive = NULL;             /* Additive genetic component,
                                           updated when va > 0 */

   int  *scounts;                       /* [nc]         */
   int ***recounts;                     /* [nc][nl][6]  */
   int **lsflag;                        /* [3][nl]      flags for lsampler
                                         *              l_ratio adjustments */
   double ddtmp;
   double **l_ratio;                    /* [nc][3]      now this is the
                                         *              LOG-l-ratio EAT 12/00 */
   double **prob;                       /* [nc][nl]    */
   double **cumulik;                    /* [nc][3]     */
   double **temp_log_prob_seg, **temp2_log_prob_seg; /* [nc][NMarks]  */
   double **tot_log_prob_seg, **tot_log_prob_pen; /* [nc][NMarks]  */
   double **tot_log_left_seg, **tot_log_rght_seg; /* [nc][NMarks]  */
   int   tot_count;

   double ***likeratios = NULL;         /* Matrix for likelihood ratios */
   double leftratio = 0.0;
   double rightratio = 0.0;

   /* .................................................. */
   /* the fourth argument is the number of haplotypes */

   nsw = setupAutozyg (argc, argv, PNAM, 2);
   prob = setupInherit (0);

   /* .................................................. */

   /* Now, the trait data are in x[NTraits][np+1], where np is the no. of
    * individual with complete trait data.  x[][0] are all zeros.
    * (This implies that we are doing a complete-case analysis.)
    */

   /* set up permutation arrays for loci and switches */
   if (Scan)
   {
      perm_sw = malloc (nsw * sizeof (int));
      alloc_chk (WHERE, perm_sw);
      for (i = 0; i < nsw; ++i)
         perm_sw[i] = i;
      perm_loc = malloc ((NMarks + 1) * sizeof (int));
      alloc_chk (WHERE, perm_loc);
      for (i = 0; i <= NMarks; ++i)
         perm_loc[i] = i;
   }

   /* allocate and set ym[], the set of centered trait values   */
   /* This segment removed:  EAT 08/2004                        */

   set_seg_mode (0);

   if (loci[0].va > 0)
   {
      additive = malloc (PedSize * sizeof *additive);
      alloc_chk (WHERE, additive);
      for (i = 0; i < PedSize; ++i)
         additive[i] = 0.0;
   }

   /* If there is a recovery file, skip all the following finding init config
    * and burn-in stuff.  Also we don't have to sample with trait unlinked
    * because that's the first checkpoint we have.  */

   /* init mixd stuff */
   if (loci[0].va > 0)
      smpl_a (loci, NMarks + 1, loci[0].ve, loci[0].va, mu, np, additive);

   /* Allocate memory for likeratios and lods scores */
   likeratios = alloc_dbl3d (n_compnts, NTPos1, NTPos1);

   alloc_loglik (&recounts, &tot_log_prob_seg, &temp_log_prob_seg,
                 &tot_log_prob_pen);
   alloc_lod (&scounts, &lsflag, &l_ratio, &cumulik,
              &temp2_log_prob_seg, &tot_log_left_seg, &tot_log_rght_seg);

   alloc_lpflag (NMarks, 2);
   alloc_logprob ();

   prntStatusBMC ();

   /* try to restart if a rescue file is specified */
   k = 0;
   schnell_restart (likeratios, &k, loci[0].va, additive);

   for (; k < NTPos1; ++k)              /* THE BIG LOOP */
   {
      printf ("\n\nCHROMOSOME SET %d\n\n", k);
      fflush (stdout);
      /* two evaluation sets for each simulation chromo (left/right) */
      printf (" Recombinations:\n");
      for (sx = FEM; sx <= MAL; ++sx)
      {
         trait_loc0[0] = IntVal[k];
         trait_rec0[0] = RecVal[sx][k];
         if (k == 0)
         {
            trait_loc0[1] = NMarks;
            trait_rec0[1] = RecVal[sx][NTPos];
         }
         else
         {
            trait_loc0[1] = IntVal[k - 1];
            trait_rec0[1] = RecVal[sx][k - 1];
         }
         trait_loc0[2] = IntVal[k + 1];
         trait_rec0[2] = RecVal[sx][k + 1];

         set_chromosomes (NMarks, Marek[sx], 2, trait_loc0, trait_rec0, sx);

         set_lpflag (NMarks, 2, sx);

         for (i = 0; i <= 2; ++i)
         {
            if (i == 0)
               printf ("    sim,   ");
            else if (i == 1)
               printf ("    left,  ");
            else
               printf ("    right, ");
            if (sx == MAL)
               printf ("M:");
            if (sx == FEM)
               printf ("F:");
            for (j = 0; j < NMarks; ++j)
            {
               if (!(j % 6) && j)
                  printf ("\n             ");
               printf ("%3d%s", chromo[i][j], dub2nlz (xecomo[sx][i][j], 7, 3));
            }
            printf ("%3d\n", chromo[i][NMarks]);
         }
      }
      printf ("\n");
      fflush (stdout);

      /* L-sampler set-up */

      set_positions (NMarks);

      /* now should do an L-sampler step on the trait locus (0) to reset */
      /* lm_schnell no uses regular do_lsmpl: do_lsmpl_shnl eliminated   */
      /*                                             EAT 2004            */
      ddtmp = do_lsmpl (NMarks, 0, prob, nFGL);

      for (c = 0; c < n_compnts; ++c)
      {
         for (h = 0; h <= 2; ++h)
         {
            l_ratio[c][h] = 0.0;
            cumulik[c][h] = 0.0;
         }

         for (h = 0; h <= NMarks; ++h)
         {
            tot_log_prob_seg[c][h] = 0.0;
            tot_log_left_seg[c][h] = 0.0;
            tot_log_rght_seg[c][h] = 0.0;
            tot_log_prob_pen[c][h] = 0.0;
         }
         scounts[c] = 0;
         for (j = 0; j <= NMarks; ++j)
            for (jj = 0; jj < 6; ++jj)
               recounts[c][j][jj] = 0;
      }
      tot_count = 0;

      /* set initial l_ratio    */

      for (c = 0; c < n_compnts; ++c)
      {
         l_ratio[c][1] = 0.0;
         l_ratio[c][2] = l_ratio[c][1];

         for (h = 1; h <= 2; ++h)
         {
            for (j = 0; j < NMarks; ++j)
               lsflag[h][j] = 1;
            l_ratio[c][h] +=
               diff_seg_prob (NMarks, c, h, lsflag[h], temp_log_prob_seg);
         }
      }

      /* now do MCMC resampling by meiosis or by locus */
      for (t = 0; t < NIt; ++t)
      {
         if (rans () < LSamplProb)
         {
            /* Do L-sampling steps */
            if (!((t + 1) % ChkIt))
            {
               printf (" Starting L-sampler iteration %d\n", t + 1);
               fflush (stdout);
            }
            if (Scan)                   /* permute loci for L-sampler scan */
            {
               permute (perm_loc, NMarks + 1);
               for (j = 0; j <= NMarks; ++j)
               {
                  jj = perm_loc[j];

                  ddtmp = do_lsmpl (NMarks, jj, prob, nFGL);
               }
               for (h = 1; h <= 2; ++h)
                  for (j = 0; j <= NMarks; ++j)
                     lsflag[h][j] = 1;
               for (c = 0; c < n_compnts; ++c)
               {
                  l_ratio[c][1] = 0.0;
                  l_ratio[c][2] = l_ratio[c][1];

                  for (h = 1; h <= 2; ++h)
                  {
                     for (j = 0; j < NMarks; ++j)
                        lsflag[h][j] = 1;
                     l_ratio[c][h] +=
                        diff_seg_prob (NMarks, c, h, lsflag[h],
                                       temp_log_prob_seg);
                     cumulik[c][h] += exp (l_ratio[c][h]);
                  }
                  ++scounts[c];
               }
            }
            else
            {                           /* random locus, by name */
               jj = intran (0, NMarks);
               /* adjust adjacent intervals out of l-ratio */
               /* note: relevant intervals may depend on h */
               for (h = 1; h <= 2; ++h)
                  for (j = 0; j <= NMarks; ++j)
                     lsflag[h][j] = 0;

               for (h = 1; h <= 2; ++h)
               {
                  j = revchro[0][jj];   /* position under sim (0) */
                  if ((((j == 0) && (lpflag[1][h][j] != 0))
                       || (lpflag[0][h][j] != 0))
                      || (((j == NMarks) && (lpflag[1][h][j - 1] != 0))
                          || (lpflag[0][h][j - 1] != 0))
                      || ((lpflag[1][h][j] != 0) || (lpflag[0][h][j] != 0)
                          || (lpflag[1][h][j - 1] != 0)
                          || (lpflag[0][h][j - 1] != 0)))
                  {
                     if (j > 0)
                        lsflag[h][j - 1] = 1;
                     if (j < NMarks)
                        lsflag[h][j] = 1;
                     if (chromo[0][j] != chromo[h][j])
                     {
                        j = revchro[h][jj];
                        if (j > 0)
                           lsflag[h][j - 1] = 1;
                        if (j < NMarks)
                           lsflag[h][j] = 1;
                     }
                  }
                  for (c = 0; c < n_compnts; ++c)
                     l_ratio[c][h] -= diff_seg_prob (NMarks, c, h, lsflag[h],
                                                     temp_log_prob_seg);
               }
               /* Set up transmission probs */

               ddtmp = do_lsmpl (NMarks, jj, prob, nFGL);
               for (h = 1; h <= 2; ++h)
                  for (c = 0; c < n_compnts; ++c)
                     l_ratio[c][h] += diff_seg_prob (NMarks, c, h, lsflag[h],
                                                     temp_log_prob_seg);
               for (c = 0; c < n_compnts; ++c)
               {
                  for (h = 1; h <= 2; ++h)
                     cumulik[c][h] += exp (l_ratio[c][h]);
                  ++scounts[c];
               }
            }
         }
         else
         {                              /* ***    Do M-sample step    *** */
            if (!((t + 1) % ChkIt))
            {
               printf (" Starting M-sampler iteration %d\n", t + 1);
               fflush (stdout);
            }
            if (Scan)                   /* permute meioses for M-sampler scan */
            {
               permute (perm_sw, nsw);
               for (j = 0; j < nsw; ++j)
               {
                  l = perm_sw[j] + 1;

                  i = (meiosis[l].ind).index;
                  sx = meiosis[l].sex;
                  c = nghd[i].component - 1;

                  do_msmpl (NMarks, 0, c, i, sx, prob, nFGL);
               }
               for (h = 1; h <= 2; ++h)
                  for (j = 0; j <= NMarks; ++j)
                     lsflag[h][j] = 1;
               for (c = 0; c < n_compnts; ++c)
               {
                  l_ratio[c][1] = 0.0;
                  l_ratio[c][2] = l_ratio[c][1];

                  for (h = 1; h <= 2; ++h)
                  {
                     for (j = 0; j < NMarks; ++j)
                        lsflag[h][j] = 1;
                     l_ratio[c][h] += diff_seg_prob (NMarks, c, h, lsflag[h],
                                                     temp_log_prob_seg);
                     cumulik[c][h] += exp (l_ratio[c][h]);
                  }
                  ++scounts[c];
               }
            }
            else
            {                           /*choose a random meiosis */
               l = intran (0, nsw - 1) + 1;
               i = (meiosis[l].ind).index;
               sx = meiosis[l].sex;
               c = nghd[i].component - 1;

               /* adjust meiosis to be updated out of l_ratio */

               for (h = 1; h <= 2; ++h)
                  l_ratio[c][h] -= lik_rat (NMarks, i, sx, h);

               /*  do-msmpl */
               do_msmpl (NMarks, 0, c, i, sx, prob, nFGL);

               /* put updated meiosis back in to l_ratio and
                * update the cumulative ratio                */

               for (h = 1; h <= 2; ++h)
               {
                  l_ratio[c][h] += lik_rat (NMarks, i, sx, h);
                  cumulik[c][h] += exp (l_ratio[c][h]);
               }
               ++scounts[c];
            }
         }
         if (loci[0].va > 0)
            smpl_a (loci, NMarks + 1, loci[0].ve, loci[0].va, mu, np, additive);

         if (!((t + 1) % ScorIt))
         {                              /* score tot log-probs & recombs */
            for (c = 0; c < n_compnts; ++c)
            {
               for (j = 0; j <= NMarks; ++j)
                  tot_log_prob_pen[c][j] += log (prob[c][j]);

               full_seg_prob (NMarks, c, recounts, temp_log_prob_seg);
               for (j = 0; j < NMarks; ++j)
                  tot_log_prob_seg[c][j] += temp_log_prob_seg[c][j];

               for (h = 1; h <= 2; ++h)
                  for (j = 0; j < NMarks; ++j)
                     lsflag[h][j] = 1;

               (void) diff_seg_prob (NMarks, c, 1, lsflag[1],
                                     temp2_log_prob_seg);
               for (j = 0; j < NMarks; ++j)
                  tot_log_left_seg[c][j] += temp2_log_prob_seg[c][j] +
                     temp_log_prob_seg[c][j];

               (void) diff_seg_prob (NMarks, c, 2, lsflag[2],
                                     temp2_log_prob_seg);
               for (j = 0; j < NMarks; ++j)
                  tot_log_rght_seg[c][j] += temp2_log_prob_seg[c][j] +
                     temp_log_prob_seg[c][j];
            }
            ++tot_count;
         }
      }
      /* now we are done with this simulation/estimation for this trait locus
         position  */
      /* output results for this simulation/estimation set */

      for (c = 0; c < n_compnts; ++c)
      {
         printf ("\n Component %d:\n\n", c + 1);
         if (scounts[c] == 0)
            printf ("    No likelihood ratios because sampling count = 0\n\n");
         else
         {
            leftratio = cumulik[c][1] / scounts[c];
            rightratio = cumulik[c][2] / scounts[c];
            printf ("    Cumulative likelihood ratios (sampling count %d)\n",
                    scounts[c]);
            printf ("       estimated ratios:   left:sim = %6.4f,"
                    "  right:sim = %6.4f\n\n", leftratio, rightratio);
         }
         /* Fill in the matrices of likehood ratios */
         /* initialization first */

         for (j = 0; j < NTPos1; ++j)
            likeratios[c][k][j] = 0.0;
         if (k == 0)
         {
            likeratios[c][0][NTPos] = leftratio;
            likeratios[c][0][1] = rightratio;
         }
         else if (k == NTPos)
         {
            likeratios[c][k][k - 1] = leftratio;
            likeratios[c][k][0] = rightratio;
         }
         else
         {
            likeratios[c][k][k - 1] = leftratio;
            likeratios[c][k][k + 1] = rightratio;
         }

         printf ("    Average total log-probability of segregations:\n");
         printf ("       first locus (constant) %8.3f, and by map interval:"
                 "\n", -cmplist[c + 1].nmeioses * log (2.0));
         printf ("       interval        left          sim       right\n");
         for (j = 0; j < NMarks; ++j)
            printf ("            %3d    %8.3f     %8.3f    %8.3f\n",
                    j + 1, tot_log_left_seg[c][j] / tot_count,
                    tot_log_prob_seg[c][j] / tot_count,
                    tot_log_rght_seg[c][j] / tot_count);
         printf ("\n");
         printf ("    Average total log-probability of penetrances,"
                 " by locus:");
         for (j = 0; j <= NMarks; ++j)
         {
            if (!(j % 8))
               printf ("\n      ");
            if (chromo[0][j] == 0)
               printf ("  trait  ");
            else
               printf (" mark%2d  ", chromo[0][j]);
         }
         for (j = 0; j <= NMarks; ++j)
         {
            if (!(j % 8))
               printf ("\n    ");
            printf (" %8.3f", tot_log_prob_pen[c][chromo[0][j]] / tot_count);
         }
         printf ("\n\n");
         fflush (stdout);

         /* print out recombination counts  */
         printf ("    Recombination counts for map intervals:\n");
         printf ("       intv   M: total    recomb   non-rec "
                 "  F: total    recomb   non-rec\n");
         for (j = 0; j < NMarks; ++j)
         {
            printf ("       %4d ", j + 1);
            for (i = 0; i < 6; ++i)
            {
               if (i == 3)
                  printf (" ");
               printf ("%10d", recounts[c][j][i]);
            }
            printf ("\n");
         }
         printf ("\n");
         fflush (stdout);
      }

      /* dumping restart information */
      schnell_dump (likeratios, k, loci[0].va, additive);
   }

   /* no need for dumping any more */
   finish_dumping ();

   prntStatusFMC ();

   /* now we are done with simulating for all NMarks+2 simulation sets */
   print_lods (likeratios);

   /* {{{ Clean up */
   if (SamplSeedsOut)
   {
      getSeeds (SAMPL);
      if (!(oSeedF = open_fil (OSEED)))
         give_up (ERR_qit);
      writSeeds (oSeedF);
      fclose (oSeedF);
   }

   free_3d ((void *) &likeratios, n_compnts, NTPos1);

   /* free_schnell (NMarks); */
   free_loglik (&recounts, &tot_log_prob_seg, &temp_log_prob_seg,
                &tot_log_prob_pen);
   free_lod (&scounts, &lsflag, &l_ratio, &cumulik,
             &temp2_log_prob_seg, &tot_log_left_seg, &tot_log_rght_seg);
   free_lpflag (2);
   free_logprob ();
   free_msmpl ();
   free_gen_pen ();
   free_seg_pen (NMarks + 1, &prob);

   /* Free up memory used in peeling routines etc. */
   if (additive)
      free (additive);
   if (perm_sw)
      free (perm_sw);
   if (perm_loc)
      free (perm_loc);

   /* }}} */
   return EXIT_SUCCESS;
}

void free_schnell (int NMarks)
{
   free_allocate_loci ();
   free_xxgg ();

   /* memory allocated in input_data */
   free_input_data ();

   /* memory allocated in proc_drop */
   if (PTraitPars)
      free (PTraitPars);
   if (PTraitOut)
      free (PTraitOut);

   /* memory allocated in proc_auto */
   free_auto ();

   free_switches ();
   free_marek ();
   locus_peel_free ();
   free_allefrq ();
   free_fnams ();
   freeChkPed ();
   freeKey ();
   free_cmplists ();
   free_nghds ();
   free_ped ();
   free_dub ();
}
