/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */
#include "nghds.h"

/*
 * Function to form the components of a pedigree after the nghds have been
 * set.  It is assumed on entry that the "component" indicator of each
 * individuals is -1 (as now set by nghds).  An infection subroutine is used
 * to determine connected sets. The component indicator of each individual is
 * set.  The linked list of components, each being a list of individuals, is
 * set up. the total number of components is counted, and returned. 
 *
 * Written and installed; 09/1993  (EAT) 
 *
 * free_complists() which frees the Compoentns structures updated
 *   and moved here               (01/2005, EAT)
 *
 */
int neighbour (int j, int k)            /* has individual j a neighbour in
                                         * component k */
{
   Ind  *ipt;
   Marlist *mlpt;
   Indlist *ilpt;
   ipt = &nghd[j];
   if (!isNullNode (ipt->pa))
   {
      if (ipt->pa->component == k)
         return (1);
      if (ipt->ma->component == k)
         return (1);
   }
   for (mlpt = ipt->marriages; mlpt != NULL; mlpt = mlpt->next)
   {
      if (mlpt->current->spouse->component == k)
         return (1);
      for (ilpt = mlpt->current->kids; ilpt != NULL; ilpt = ilpt->next)
         if (ilpt->current->component == k)
            return (1);
   }
   return (0);
}
int infect (int compnt_no)
/* infection routine for component k; returns the size of the component */
{
   int   i, j, nc, nnc;
   Indlist *iptr;
   nc = 0;

   /* find the first member of the new component k */

   i = 0;
   while (i < pedsize)
   {
      if (nghd[i].component == -1)
         break;
      ++i;
   }
   if (i == pedsize)
      give_up_where (WHERE, ERN_cmp, "infect");

   nghd[i].component = compnt_no;
   nnc = 1;

   /* find other members of the component, if any  */

   while (nc < nnc)
   {
      nc = nnc;
      for (j = i + 1; j < pedsize; ++j)
      {
         if (nghd[j].component == -1)
         {
            if (neighbour (j, compnt_no) == 1)
            {
               nghd[j].component = compnt_no;
               ++nnc;
            }
         }
      }
   }
   /* Now have a component, size nc=nnc, label k; set up linked list  */
   /*  add in size and founder counts       EAT Oct 2004              */

   cmplist[compnt_no].index = compnt_no;
   cmplist[compnt_no].size = nnc;
   cmplist[compnt_no].nfou = 0;
   cmplist[compnt_no].naff = 0;
   cmplist[compnt_no].nunaff = 0;
   cmplist[compnt_no].nmeioses = 0;
   cmplist[compnt_no].meioses = NULL;
   cmplist[compnt_no].ind_aff = NULL;
   cmplist[compnt_no].ind_unaff = NULL;
   cmplist[compnt_no].people = (Indlist *) malloc (sizeof (Indlist));
   alloc_chk (WHERE, cmplist[compnt_no].people);
   iptr = cmplist[compnt_no].people;
   iptr->current = (Ind *) malloc (sizeof (Ind));
   alloc_chk (WHERE, iptr->current);

   for (i = 0; i < pedsize; ++i)
   {
      if (nghd[i].component == compnt_no)
      {
         iptr->next = (Indlist *) malloc (sizeof (Indlist));
         alloc_chk (WHERE, iptr->next);
         iptr = iptr->next;
         iptr->current = &nghd[i];
         if (isNullNode (nghd[i].pa))
            ++cmplist[compnt_no].nfou;
      }
   }
   iptr->next = NULL;

   return (nc);
}

int components (void)
{
   int   ncomp, nind, done;
   ncomp = 0;
   done = 0;

   while (done < pedsize)
   {
      ++ncomp;

      nind = infect (ncomp);
      done += nind;
      if (nind > max_cpsize)
         max_cpsize = nind;
   }
   return (ncomp);
}

/* ------- free_cmplists:  frees the component lists (Indlists etc.) --- */

void free_cmplists (void)
{
   int   i;
   Indlist *pcurr, *pnext;

   if (!cmplist)
      return;

   for (i = 1; i <= n_compnts; ++i)
   {
      pnext = cmplist[i].people;
      free ((void *) pnext->current);
      cmplist[i].people = NULL;
      while (pnext)
      {
         pcurr = pnext;
         pnext = pcurr->next;
         pcurr->current = NULL;
         pcurr->next = NULL;
         free ((void *) pcurr);
      }

      if (i == 1)
         if (cmplist[i].meioses)
            free ((void *) cmplist[i].meioses);

      if (cmplist[i].ind_aff)
         free ((void *) cmplist[i].ind_aff);

      if (cmplist[i].ind_unaff)
         free ((void *) cmplist[i].ind_unaff);
   }
   free ((void *) cmplist);
   cmplist = NULL;

   return;
}
