/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */

#include "parsie.h"
#include "parsedefs.h"
#include "parsepars.h"
#include "parseproc.h"
#include "errspars.h"
#include "parsekin.h"
#include "parseauto.h"
#include "parsedrop.h"
#include "parsepoly.h"
#include "parsefils.h"
#include "parseseeds.h"
#include "parsedisp.h"

/*
 * proc_pars() parses the parameter statements and stores the results in
 * intermediate tables.  Then proc_drop(), for the genedrop program,
 * proc_drop() and proc_auto() for the Autozyg programs, or proc_poly(),
 * for the polyEM programs, take this information and prepare global
 * variables for the specific MORGAN program.
 *                                                 MAJ Jul 1999, Dec 2001
 */

int proc_pars (int argc, char **argv, char *ppn)
{
   char  err_lev;

   /* gets name of calling program */
   for (pmx = Prog; pmx->pnam[0] != NUL && strcmp (ppn, pmx->pnam); ++pmx)
      ;
   set_defaults ();

   if (!get_cmd_fnams (argc, argv))     /* gets command line file names      */
      if (proc_err ())
         return FALSE;

   pfloop = Filtab;                     /* process one file each loop        */

   while (pfloop->type[0] != NUL)
   {
      if (pfloop->fname && pfloop->parstat && !pfloop->done)
      {
         if (!open_par_fil ())          /* opens input files                 */
            if (proc_err ())
               return FALSE;

         while (!pfloop->done)          /* processes 1 statement each loop   */
         {

            if (!get_tokens ())         /* gets list of tokens               */
            {
               if (proc_err ())
                  return FALSE;
            }
            else if (!get_aggr_key ())  /* gets the statement type           */
            {
               if (proc_err ())
                  return FALSE;
            }
            else
            {
               err_lev = get_data_list (); /* gets the parameter data       */
               if (err_lev && proc_err ())
                  return FALSE;

               if (err_lev != 'E')
                  if ((ppp->func) ())   /* calls statement-specific fncs     */
                     if (proc_err ())
                        return FALSE;
            }
         }
         if (!close_par_fil ())         /* closes input files                */
            if (proc_err ())
               return FALSE;

         fflush (stdout);
      }
      ++pfloop;                         /* goes on to next file type         */
   }
   return TRUE;
}

/* ----------- set_defaults:  sets default values to parameters ------------ */

void set_defaults (void)
{
   err_typ = 0;

   parentOrdIn = UNDEF;
   parentOrdOut = UNDEF;

   add_cor_flag = 0;
   res_cor_flag = 0;
   reset_fit_var (0);
   reset_fit_var (1);
   reset_fit_cov (0);
   reset_fit_cov (1);

   n_eigen = 0;
   eigen_set = N;
   eigen_compute = N;

   max_npar = MAXNTOKEN;
   token = NULL;
   tok_typ = NULL;
   j_par = NULL;
   u_par = NULL;
   d_par = NULL;
   m_par = NULL;
   realloc_arrs ();

   max_tchar = MAXNTOKEN * NOMTOKLEN;
   token[0] = NULL;
   realloc_toks (0);

   return;
}

/* ------ get_cmd_fnams:  gets the file names from the command line -------- */
/*                        checks the file types specified                    */
/*                        stores the names in Filtab[]                       */

int get_cmd_fnams (int argc, char **argv)
{
   int   i;

   if (argc < 2)                        /* tests for parameter file as       */
   {                                    /* first argument in command line    */
      err_typ = ERR_EIF;
      return FALSE;
   }
   else
   {
      pft = Filtab;
      pft->fname = alloc_copy (argv[1]);
   }
   for (i = 2; i < argc - 1; i += 2)    /* identifies other files named      */
   {                                    /* in command line                   */
      pft = get_filent (argv[i]);
      if (pft)
      {
         if (strchr (pft->pcods, pmx->pcod))
         {
            if (pft->reclo)             /* checks for rewritable file type   */
            {
               --pft;
               pft->rewrit = Y;
            }
            pft->command = Y;
            pft->fname = alloc_copy (argv[i + 1]);
         }
         else
         {                              /* checks for file used by program   */
            err_typ = ERR_PCL;
            err_txt = pft->type;
         }
      }
      else
      {
         err_typ = ERR_FNU;             /* reports bad file type             */
         err_txt = argv[i];
      }
   }
   if ((argc & 1) != 0)                 /* tests for odd no. of arguments    */
   {
      i = 0;
      err_typ = ERR_FCL;
      sprintf (errnumb, COM_LIN_FORM);
      while (pft = get_filtyp (pmx->pcod))
      {
         if (i && !(i % 3))
            sprintf (&errnumb[strlen (errnumb)], "\n                      ");
         sprintf (&errnumb[strlen (errnumb)], " [%s <%sfile>]",
                  pft->type, pft->type);
         ++i;
      }
      err_txt = errnumb;
   }
   if (err_typ)
      return FALSE;
   else
      return TRUE;
}

/* ---------------- open_par_fil:  opens the input file -------------------- */
/*                     gets the first line of input                          */

int open_par_fil (void)
{
   char *ptok, *pstr;

   /*                                    * opens the file                    */
   errno = 0;
   if (!(pfile = fopen (pfloop->fname, pfloop->fop)))
   {
      err_typ = ERR_OPF;
      sprintf (errnumb, "%s \"%s\":  %s", pfloop->fdesc,
               pfloop->fname, strerror (errno));
      err_txt = errnumb;
      pfloop->done = Y;
      return FALSE;
   }

   pstr = line_in;                      /* tests for file not empty          */
   while (get_line () && !(ptok = strtok (pstr, WHITESPACE)))
      ;
   if (err_typ)
      return FALSE;

   if (pfloop->done)
   {
      err_typ = ERR_SIF;
      err_txt = pfloop->fname;
      return FALSE;
   }
   else
      *strchr (ptok, NUL) = ' ';        /* gets first token                  */

   return TRUE;
}

/* ---------------- close_par_fil:  closes the input file ------------------ */

int close_par_fil (void)
{
   errno = 0;
   if (pfile)
      if (fclose (pfile) == EOF)
      {
         err_typ = ERR_CLF;
         sprintf (errnumb, "%s \"%s\":  %s", pfloop->fdesc,
                  pfloop->fname, strerror (errno));
         err_txt = errnumb;
      }
   pfile = NULL;

   if (err_typ)
      return FALSE;
   return TRUE;
}

/* ------ chk_stor:  checks for enough storage allocated for arrays -------  */

void chk_stor (int n_tok, int m_char)
{                                       /* n_tok:  no. tokens stored         */
   if (n_tok + 2 > max_npar)
   {
      max_npar *= 2;
      realloc_arrs ();
   }                                    /* m_char:  string length            */
   if (token[n_tok] - token[0] + m_char + 1 >= max_tchar)
   {
      max_tchar *= 2;
      realloc_toks (n_tok);
   }
   return;
}

/* ---------- realloc_arrs:  allocates or reallocates storage -------------  */
/*                            for arrays for input data                      */

void realloc_arrs (void)
{
   int   i;
   char **p;

   token = realloc (token, max_npar * sizeof (char *));
   alloc_chk (WHERE, token);
   tok_typ = realloc (tok_typ, max_npar * sizeof (int));
   alloc_chk (WHERE, tok_typ);
   j_par = realloc (j_par, max_npar * sizeof (int));
   alloc_chk (WHERE, j_par);
   u_par = realloc (u_par, max_npar * sizeof (uint32));
   alloc_chk (WHERE, u_par);
   d_par = realloc (d_par, max_npar * sizeof (double));
   alloc_chk (WHERE, d_par);

   p = m_par;
   m_par = realloc (m_par, max_npar * sizeof (char *));
   alloc_chk (WHERE, m_par);
   if (!p)
      m_par[0] = NULL;
   m_par[0] = realloc (m_par[0], max_npar * (MAXNAMLEN + 1) * sizeof (char));
   alloc_chk (WHERE, m_par[0]);
   for (i = 1; i < max_npar; ++i)
      m_par[i] = m_par[i - 1] + MAXNAMLEN + 1;

   return;
}

/* ----------- realloc_toks:  allocates or reallocates storage ------------- */
/*                             for array of tokens                           */

void realloc_toks (int n_tok)
{
   int   i;
   char *ptmp;

   ptmp = token[0];
   token[0] = (char *) realloc (token[0], max_tchar);
   alloc_chk (WHERE, token[0]);
   if (n_tok && token[0] != ptmp)
      for (i = 1; i <= n_tok; ++i)
         token[i] = token[0] + (token[i] - ptmp);

   return;
}

/* --------- free_arrs:  frees all of proc_pars's malloc'd arrays ---------- */

void free_arrs (void)
{
   free ((void *) token[0]);
   token[0] = NULL;
   free ((void *) token);
   token = NULL;
   free ((void *) tok_typ);
   tok_typ = NULL;
   free ((void *) j_par);
   j_par = NULL;
   free ((void *) u_par);
   u_par = NULL;
   free ((void *) d_par);
   d_par = NULL;
   free ((void *) m_par[0]);
   m_par[0] = NULL;
   free ((void *) m_par);
   m_par = NULL;

   return;
}

/* ------------ get_line:  gets the next line of input data  --------------- */
/*                         replaces first "#" with "\0" (for comment)        */
/*                         returns pointer to line,                          */
/*                         unless eof or line contains "****"                */
/*                         as first keyword, then                            */
/*                         sets file pointer to NULL and returns NULL        */

char *get_line (void)
{
   char *pt, *ps;

   if (!fgets (line_in, MAXLINLEN + 2, pfile))
   {
      pfloop->done = Y;
      return NULL;
   }
   else
   {
      ps = line_in;
      if (strlen (line_in) == MAXLINLEN + 1 && *(line_in + MAXLINLEN) != '\n')
      {
         err_typ = ERR_LTL;
         err_txt = line_in;
         pfloop->done = Y;
         return NULL;
      }
      if (pt = strchr (ps, COMMENT))
         *pt = NUL;
      if (pt = strtok (ps, WHITESPACE))
         if (!strncmp (ENDPAR, pt, 4))
         {
            pfloop->done = Y;
            return NULL;
         }
         else
            *strchr (pt, NUL) = ' ';
      return line_in;
   }
}

/* ---------------- get_tokens:  stores a list of tokens ------------------- */
/*                               returns the number found                    */

int get_tokens (void)
{
   char *ptok, *pstr;
   char  chkverb[5];
   int   end_statement;
   int   n, i, m;

   pstr = line_in;                      /* gets tokens for current line      */
   for (n = 0; (ptok = strtok (pstr, WHITESPACE)) != NULL; ++n)
   {
      m = strlen (ptok) + 1;
      chk_stor (n, m);

      strncpy (token[n], ptok, m);
      token[n + 1] = token[n] + m;
      pstr = NULL;
   }
   token[n][0] = NUL;

   end_statement = FALSE;               /* gets the next line                */
   while (!pfloop->done && !end_statement)
   {
      if (get_line ())
      {
         pstr = line_in;                /* tests for end of statement        */
         if ((ptok = strtok (pstr, WHITESPACE)) != NULL)
         {
            if (isalpha ((int) *ptok))
            {
               for (i = 0; i < 4 && *(ptok + i) != NUL; ++i)
                  chkverb[i] = tolower ((int) *(ptok + i));
               chkverb[i] = NUL;
               for (i = 0; strcmp (verb[i], ""); ++i)
                  if (!strncmp (verb[i], chkverb, 4))
                     end_statement = TRUE;
            }
            *strchr (ptok, NUL) = ' ';
            if (!end_statement)         /* gets tokens from the next line    */
            {
               pstr = line_in;
               for (; (ptok = strtok (pstr, WHITESPACE)) != NULL; ++n)
               {
                  m = strlen (ptok) + 1;
                  chk_stor (n, m);
                  strncpy (token[n], ptok, m);
                  token[n + 1] = token[n] + m;
                  pstr = NULL;
               }
               token[n][0] = NUL;
            }
         }
      }
   }
   if (err_typ)
      return 0;

   return n;
}

/* ---- get_aggr_key:  sets a pointer to the input statement prototype ----- */
/*                     returns key # from list                               */

int get_aggr_key (void)
{
   int   i, j, n;

   aggre_key[0] = NUL;
   for (i = 0, n = 0; token[i][0] != NUL; ++i)
   {
      if (isalpha ((int) token[i][0]))
      {
         tok_typ[i] = KEYWD;
         if (++n <= MAXNKEY)            /* gets aggregate key                */
            strncat (aggre_key, token[i], 4);
      }
      else
         tok_typ[i] = PDAT;
   }
   for (i = 0; aggre_key[i] != NUL; ++i) /* converts to lower case          */
      aggre_key[i] = tolower ((int) aggre_key[i]);

   /*                                    * searches for key in table         */
   for (ppp = proto, j = 0; ppp->key[0] != NUL
        && strcmp (aggre_key, ppp->key); ++ppp, ++j)
      ;
   if (ppp->key[0] != NUL)              /* found key;  return ptr            */
      return key_ix = j + 1;

   /*                                    * another attempt to find key;
    *                                      statement may contain names       */
   for (ppp = proto, j = 0; ppp->key[0] != NUL
        && (!strpbrk (ppp->par, nam_cod)
            || strncmp (aggre_key, ppp->key, strlen (ppp->key))); ++ppp, ++j)
      ;                                 /* continue                          */

   if (ppp->key[0] != NUL)              /* found key; reassign token types   */
   {
      aggre_key[0] = NUL;
      for (i = 0; token[i][0] != NUL; ++i)
         if (isalpha ((int) token[i][0])
             && strlen (aggre_key) < strlen (ppp->key))
         {
            tok_typ[i] = KEYWD;
            strncat (aggre_key, token[i], 4);
         }
         else
            tok_typ[i] = PDAT;

      return key_ix = j + 1;
   }
   else
   {
      err_typ = ERR_KEY;
      return key_ix = 0;
   }
}

/* ------- get_data_list:  converts tokens into Morgan internal data types
 *                         (integers, unsigned integers, doubles, or character
 *                         arrays)
 *                         converts all data from the current parameter
 *                         statement
 *                         returns the error level ('F', 'E', 'W') if any
 *                         errors detected, else returns the null character
 */

char get_data_list (void)
{
   int   i, j, k, n, s;
   char  c, *endp;

   prntd_flg = N;

   ni_par = nk_par = nh_par = nr_par = nx_par = nm_par = ns_par = 0;

   j = 0;

   for (i = 0; (c = ppp->par[i]) != NUL; ++i)
   {                                    /* processes all data (non-keywords)
                                           keywords) for the input statement */
      for (; token[j][0] != NUL && tok_typ[j] == KEYWD; ++j)
         ;
      if (token[j][0] == NUL)
      {
         err_typ = ERR_DAT;
         return (proc_errors + (int) err_typ)->level;
      }
      switch (c)
      {
          case 'i':                    /* expects single integer value       */
             j_par[ni_par + nk_par] = atoi (token[j]);
             ++ni_par;
             break;
          case 'k':                    /* expects list of int values         */
             for (; token[j][0] != NUL && tok_typ[j] == PDAT; ++j)
             {
                j_par[ni_par + nk_par] = atoi (token[j]);
                ++nk_par;
             }
             --j;
             break;
          case 'h':                    /* expects single hexadec value       */
             u_par[nh_par] = (uint32) strtoul (token[j], &endp, 0);
             if (*endp != NUL)
                err_typ = ERR_HEX;
             ++nh_par;
             break;
          case 'r':                    /* expects single real value          */
             d_par[nr_par + nx_par] = atof (token[j]);
             ++nr_par;
             break;
          case 'x':                    /* expects list of real values        */
             for (; token[j][0] != NUL && tok_typ[j] == PDAT; ++j)
             {
                d_par[nr_par + nx_par] = atof (token[j]);
                ++nx_par;
             }
             --j;
             break;
          case 's':                    /* expects a string                   */
             s = strlen (token[j]) - 2;
             if (s < 1)
                err_typ = ERR_STR;
             if (token[j][0] != '\"' && token[j][0] != '\'')
                err_typ = ERR_STR;
             if (token[j][s + 1] != '\"' && token[j][s + 1] != '\'')
                err_typ = ERR_STR;
             if (!err_typ)
             {
                strncpy (s_par, &token[j][1], s);
                s_par[s] = NUL;
             }
             ++ns_par;
             break;
          case 'n':                    /* expects a list of names            */
             for (; token[j][0] != NUL && tok_typ[j] == PDAT; ++j)
             {
                s = strlen (token[j]);
                if (s > MAXNAMLEN && !err_typ)
                {
                   err_typ = ERR_NML;
                   sprintf (errnumb, "%d", MAXNAMLEN);
                   err_txt = errnumb;
                }
                strncpy (m_par[nm_par], token[j], MAXNAMLEN);
                m_par[nm_par][MAXNAMLEN] = NUL;
                ++nm_par;
             }
             --j;
             break;
          case 'g':                    /* expects a list of pairs of names
                                           and integers                      */
             for (k = 0; token[j][0] != NUL && tok_typ[j] == PDAT; ++j, k = !k)
             {
                if (!k)                 /* expects name                      */
                {
                   s = strlen (token[j]);
                   if (s > MAXNAMLEN && !err_typ)
                   {
                      err_typ = ERR_NML;
                      sprintf (errnumb, "%d", MAXNAMLEN);
                      err_txt = errnumb;
                   }
                   strncpy (m_par[nm_par], token[j], MAXNAMLEN);
                   m_par[nm_par][MAXNAMLEN] = NUL;
                   ++nm_par;
                }
                else
                {                       /* expects an integer                */
                   if (!tok2int (token[j], &j_par[ni_par + nk_par]))
                   {
                      err_typ = ERR_BGM;
                      sprintf (errnumb, "%s", token[j]);
                      err_txt = errnumb;
                   }
                   ++nk_par;
                }
             }
             if (k)
                err_typ = ERR_PGP;
             --j;
             break;
          case 'a':                    /* expects an integer (the number of
                                           integer pairs in each set), then
                                           sets of a name plus a list of
                                           integers                          */
             n = j_par[ni_par + nk_par - 1];
             if (n < 1)
             {
                err_typ = ERR_BNN;
                sprintf (errnumb, "%d", j_par[ni_par + nk_par - 1]);
                err_txt = errnumb;
             }
             else
             {
                for (k = 0; token[j][0] != NUL && tok_typ[j] == PDAT; ++k, ++j)
                {
                   if (!(k % (2 * n + 1)))
                   {
                      s = strlen (token[j]); /* expects a name              */
                      if (s > MAXNAMLEN && !err_typ)
                      {
                         err_typ = ERR_NML;
                         sprintf (errnumb, "%d", MAXNAMLEN);
                         err_txt = errnumb;
                      }
                      strncpy (m_par[nm_par], token[j], MAXNAMLEN);
                      m_par[nm_par][MAXNAMLEN] = NUL;
                      ++nm_par;
                   }
                   else
                   {                    /* expects a list of integers        */
                      j_par[ni_par + nk_par] = atoi (token[j]);
                      ++nk_par;
                   }
                }
                if (k % (2 * n + 1))
                {
                   err_typ = ERR_AIL;
                   sprintf (errnumb, "%d", n);
                   err_txt = errnumb;
                }
                --j;
             }
             break;
      }
      ++j;
   }
   for (; token[j][0] != NUL; ++j)
      if (tok_typ[j] == PDAT)
         err_typ = ERR_XTR;

   if (err_typ)
      return (proc_errors + (int) err_typ)->level;
   else
      return NUL;
}

/* -------- there is one function for each parameter statement type -------  */
/*            they are named in a dispatch table in parsepars.h              */

int not_impl (void)
{
   chk_stat ();
   return err_typ;
}

int in_ped_fil (void)                   /* input pedigree file                   */
{
   if (chk_stat ())
      chk_fil (PED);

   return err_typ;
}

int in_pedsiz (void)                    /* input pedsize statement               */
{
   if (chk_stat ())
   {
      chk_repeat ();
      PedSize = j_par[0];
      if (PedSize < MIN_PED)
         err_typ = ERR_PED;
      else if (PedSize > pmx->max_ped)
      {
         err_typ = ERR_PEX;
         sprintf (errnumb, "%d", pmx->max_ped);
         err_txt = errnumb;
      }
   }
   return err_typ;
}

int allow_pedsiz (void)                 /* allow pedigree size statement     */
{
   if (chk_stat ())
   {
      chk_repeat ();
      allw_pedsiz = j_par[0];
      if (pmx->max_ped && allw_pedsiz > pmx->max_ped)
      {
         if (UNIVAR)
            err_typ = ERR_ALU;
         else
            err_typ = ERR_ALP;
         sprintf (errnumb, "%d", pmx->max_ped);
         err_txt = errnumb;
      }
   }
   return err_typ;
}

int allow_cmpsiz (void)                 /* allow component size statement    */
{
   if (chk_stat ())
   {
      chk_repeat ();
      allw_cmpsiz = j_par[0];
      if (pmx->max_cmp && allw_cmpsiz > pmx->max_cmp)
      {
         err_typ = ERR_ALC;
         sprintf (errnumb, "%d", pmx->max_cmp);
         err_txt = errnumb;
      }
   }
   return err_typ;
}

int allow_obsiz (void)                  /* allow observed individuals        */
{
   if (chk_stat ())
   {
      chk_repeat ();
      allw_obsiz = j_par[0];
      if (pmx->max_obs && allw_obsiz > pmx->max_obs)
      {
         err_typ = ERR_ALO;
         sprintf (errnumb, "%d", pmx->max_obs);
         err_txt = errnumb;
      }
   }
   return err_typ;
}

int in_ped_int (void)                   /* input pedigree record integer     */
{
   if (chk_stat ())
   {
      err_typ = ERR_PPI;

      chk_repeat ();
      NNams = 3;
      NInts = j_par[0] - NNams;
      if (NInts < 0 || NInts > FMAX_INT)
         err_typ = ERR_PF1;
      NReals = 0;
   }
   return err_typ;
}

int in_ped_int_rl (void)                /* input pedigree record integ real  */
{
   if (chk_stat ())
   {
      err_typ = ERR_PPR;

      chk_repeat ();
      NNams = 3;
      NInts = j_par[0] - NNams;
      NReals = j_par[1];
      if (NInts < 0 || NInts > FMAX_INT)
         err_typ = ERR_PF1;
      if (NReals < 0 || NReals > FMAX_REAL)
         err_typ = ERR_PF2;
   }
   return err_typ;
}

int in_ped_n (void)                     /* input pedigree record names       */
{
   if (chk_stat ())
   {
      chk_repeat ();
      NNams = j_par[0];
      if (NNams != 3)
         err_typ = ERR_PN3;
      else
      {
         NInts = 0;
         NReals = 0;
      }
   }
   return err_typ;
}

int in_ped_n_i (void)                   /* input pedigree record names int    */
{
   if (chk_stat ())
   {
      chk_repeat ();
      NNams = j_par[0];
      if (NNams != 3)
         err_typ = ERR_PN3;
      else
      {
         NInts = j_par[1];
         if (NInts < 0 || NInts > FMAX_INT - NNams)
            err_typ = ERR_PF1;
         NReals = 0;
      }
   }
   return err_typ;
}

int in_ped_n_i_r (void)                 /* input pedigree record nam int real */
{
   if (chk_stat ())
   {
      chk_repeat ();
      NNams = j_par[0];
      if (NNams != 3)
         err_typ = ERR_PN3;
      else
      {
         NInts = j_par[1];
         NReals = j_par[2];
         if (NInts < 0 || NInts > FMAX_INT - NNams)
            err_typ = ERR_PF1;
         if (NReals < 0 || NReals > FMAX_REAL)
            err_typ = ERR_PF2;
      }
   }
   return err_typ;
}

int in_ped_n_r (void)                   /* input pedigree record names real   */
{
   if (chk_stat ())
   {
      chk_repeat ();
      NNams = j_par[0];
      if (NNams != 3)
         err_typ = ERR_PN3;
      else
      {
         NInts = 0;
         NReals = j_par[1];
         if (NReals < 0 || NReals > FMAX_REAL)
            err_typ = ERR_PF2;
      }
   }
   return err_typ;
}

/*  * NEW ********** */
int in_ped_tr_int (void)                /* input pedigree reco traits integrs */
{
   TRTPED_PTR p_trtped;
   int   n, i;

   if (chk_stat ())
   {
      n = nk_par / 2;
      if (chk_trtped (INTDAT))
         for (i = 0; i < n; i++)
         {
            if (!(p_trtped = chk_trtped_node (j_par[i])))
               p_trtped = alloc_trtped_node (j_par[i]);
            stor_trtped_pos (INTDAT, j_par[i], &j_par[n + i], p_trtped);
         }
   }
   return err_typ;
}

int in_ped_tr_int2 (void)               /* input ped reco traits integr pairs */
{
   TRTPED_PTR p_trtped;
   int   n, i;

   if (chk_stat ())
   {
      n = nk_par / 3;
      if (chk_trtped (INTDAT2))
         for (i = 0; i < n; i++)
         {
            if (!(p_trtped = chk_trtped_node (j_par[i])))
               p_trtped = alloc_trtped_node (j_par[i]);
            stor_trtped_pos (INTDAT2, j_par[i], &j_par[n + 2 * i], p_trtped);
         }
   }
   return err_typ;
}

int in_ped_tr_real (void)               /* input pedigree reco traits reals   */
{
   TRTPED_PTR p_trtped;
   int   n, i;

   if (chk_stat ())
   {
      n = nk_par / 2;
      if (chk_trtped (REALDAT))
         for (i = 0; i < n; i++)
         {
            if (!(p_trtped = chk_trtped_node (j_par[i])))
               p_trtped = alloc_trtped_node (j_par[i]);
            stor_trtped_pos (REALDAT, j_par[i], &j_par[n + i], p_trtped);
         }
   }
   return err_typ;
}

int in_eig_fil (void)                   /* input eigenvalue file             */
{
   if (chk_stat ())
      chk_fil (IGEN);

   return err_typ;
}

int in_mrkdat_fil (void)                /* input marker data file            */
{
   if (chk_stat ())
      chk_fil (MDAT);

   return err_typ;
}

int in_peel_fil (void)                  /* input peeling file                */
{
   if (chk_stat ())
      chk_fil (PEEL);

   return err_typ;
}

int in_resc_fil (void)                  /* input rescue file                 */
{
   if (chk_stat ())
      chk_fil (IRESC);

   return err_typ;
}

int in_seed_fil (void)                  /* input seed file                   */
{
   if (chk_stat ())
      chk_fil (SEED);

   return err_typ;
}

int in_xtra_fil (void)                  /* input extra file                  */
{
   if (chk_stat ())
      chk_fil (XTRA);

   return err_typ;
}

int out_ped_fil (void)                  /* output pedgree file               */
{
   if (chk_stat ())
      chk_fil (OPED);

   return err_typ;
}

int out_rw_ped_fil (void)               /* output overwrite pedgree file     */
{
   if (chk_stat ())
      chk_fil (OPED);
   set_rewrt (OPED);

   return err_typ;
}

int out_resc_fil (void)                 /* output rescue file                */
{
   if (chk_stat ())
      chk_fil (ORESC);

   return err_typ;
}

int out_scor_fil (void)                 /* output scores file                */
{
   if (chk_stat ())
      chk_fil (OSCORE);

   return err_typ;
}

int out_rb_est_fil (void)               /* output r-b estimates file         */
{
   if (chk_stat ())
      chk_fil (ORBEST);

   return err_typ;
}

int out_seed_fil (void)                 /* output seed file                  */
{
   if (chk_stat ())
      chk_fil (OSEED);

   return err_typ;
}

int out_rw_seed_fil (void)              /* output overwrite seed file         */
{
   if (chk_stat ())
      chk_fil (OSEED);
   set_rewrt (OSEED);

   return err_typ;
}

int out_xtra_fil (void)                 /* output extra file                 */
{
   if (chk_stat ())
      chk_fil (OXTR);

   return err_typ;
}

int out_rw_xtra_fil (void)              /* output overwrite xtra file        */
{
   if (chk_stat ())
      chk_fil (OXTR);
   set_rewrt (OXTR);

   return err_typ;
}

int in_ord_fath (void)                  /* input record father mother        */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (parentOrdIn == MA1ST)
         err_typ = ERR_ORD;
      parentOrdIn = PA1ST;
   }
   return err_typ;
}

int in_ord_moth (void)                  /* input record mother father        */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (parentOrdIn == PA1ST)
         err_typ = ERR_ORD;
      parentOrdIn = MA1ST;
   }
   return err_typ;
}

int in_gend_abs (void)                  /* input record gender absent        */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (SexIn == Y)
         err_typ = ERR_GND;
      SexIn = N;
   }
   return err_typ;
}

int in_gend_pres (void)                 /* input record gender present       */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (SexIn == N)
         err_typ = ERR_GND;
      SexIn = Y;
   }
   return err_typ;
}

int in_obs_abs (void)                   /* input record observed absent      */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (ObsIn == Y)
         err_typ = ERR_OPA;
      ObsIn = N;
   }
   return err_typ;
}

int in_obs_pres (void)                  /* input record observed present     */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (ObsIn == N)
         err_typ = ERR_OPA;
      ObsIn = Y;
   }
   return err_typ;
}

int assum_all_obs (void)                /* assume all observed               */
{
   if (chk_stat ())
   {
      chk_repeat ();
      ObsOp = ALL;
   }
   return err_typ;
}

int assign_gend (void)                  /* assign gender                     */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (SexOp == IGNORE_M)
         err_typ = ERR_ASG;
      SexOp = ASSIGN;
   }
   return err_typ;
}

int ignor_gend (void)                   /*  ignore gender                    */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (SexOp == ASSIGN)
         err_typ = ERR_IGG;
      SexOp = IGNORE_M;
   }
   return err_typ;
}

int out_ord_fath (void)                 /* output record father mother       */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (parentOrdOut == MA1ST)
         err_typ = ERR_ORD;
      parentOrdOut = PA1ST;
   }
   return err_typ;
}

int out_ord_moth (void)                 /* output record mother father       */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (parentOrdOut == PA1ST)
         err_typ = ERR_ORD;
      parentOrdOut = MA1ST;
   }
   return err_typ;
}

int out_corr (void)                     /* output correlations               */
{
   if (chk_stat ())
   {
      chk_repeat ();
      IOCor = Y;
   }
   return err_typ;
}

int out_fin_pheno (void)                /* output final adjusted phenotypes  */
{
   if (chk_stat ())
   {
      chk_repeat ();
      FinPheno = Y;
   }
   return err_typ;
}

int out_covar (void)                    /* output covariances                */
{
   if (chk_stat ())
   {
      chk_repeat ();
      IOCor = N;
   }
   return err_typ;
}

int lim_breed (void)                    /* limit breeding iterations         */
{
   if (chk_stat ())
   {
      chk_repeat ();
      ItMo = j_par[0];
   }
   return err_typ;
}

int lim_em_iter (void)                  /* limit em iterations statement     */
{
   if (chk_stat ())
   {
      chk_repeat ();
      ItEM = j_par[0];
   }
   return err_typ;
}

int set_mc_iter (void)                  /* set mc iterations statement       */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (j_par[0] < 1)
         err_typ = ERR_SNI;
      NIt = j_par[0];
   }
   return err_typ;
}

int cmpu_scor_iter (void)               /* compute scores every I iterations */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (j_par[0] < 1)
         err_typ = ERR_SCI;
      ScorIt = j_par[0];
   }
   return err_typ;
}

int set_burn_iter (void)                /* set burn-in iterations               */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (j_par[0] < 0)
         err_typ = ERR_SBI;
      NBrnIt = j_par[0];
   }
   return err_typ;
}

int set_prel_iter (void)                /* set preliminary iterations           */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (j_par[0] < 0)
         err_typ = ERR_SPI;
      NPreIt = j_par[0];
   }
   return err_typ;
}

int set_lsmpl_prob (void)               /* set l-sampler probability         */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (d_par[0] < 0. || d_par[0] > 1.0)
         err_typ = ERA_BLS;

      LSamplProb = d_par[0];
   }
   return err_typ;
}
int set_pseudo_pr (void)                /* set pseudo-priors                 */
{
   if (chk_stat ())
   {
      if (chk_pseudo ())
      {
         if (!chk_repeat ())
            reset_set_pseudo ();
         set_set_pseudo ();
      }
   }
   return err_typ;
}

int set_tstpos_wind (void)              /* set test position window width    */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (j_par[0] < 0 || d_par[0] > 100)
         err_typ = ERA_TPW;

      WindWdth = j_par[0];
   }
   return err_typ;
}

int set_trait_geno (void)               /* set trait data genotypic           */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (PedDatTyp != UNDEF)
         err_typ = ERR_STT;
      PedDatTyp = GENOTYPIC;
      TraitTyp = GENOTYPIC;
   }
   return err_typ;
}

int set_trait_disc (void)               /* set trait data discrete            */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (PedDatTyp != UNDEF)
         err_typ = ERR_STT;
      PedDatTyp = DISCRETE;
      TraitTyp = DISCRETE;
   }
   return err_typ;
}

int set_trait_liab (void)               /* set trait discrete w liab classes  */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (PedDatTyp != UNDEF)
         err_typ = ERR_STT;
      PedDatTyp = DISCR_LIAB;
      TraitTyp = DISCRETE;
   }
   return err_typ;
}

int set_trait_quant (void)              /* set trait data quantitative        */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (PedDatTyp != UNDEF)
         err_typ = ERR_STT;
      PedDatTyp = QUANTITATIVE;
      TraitTyp = QUANTITATIVE;
   }
   return err_typ;
}

int use_lsmpl_setup (void)              /* use locus-by-locus samp for setup */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (SmplSetup == SEQIMP)
         err_typ = ERR_SSU;
      SmplSetup = INDEP_LOC;
   }
   return err_typ;
}

int use_sqimp_setup (void)              /* use sequential imputatn for setup */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (SmplSetup == INDEP_LOC)
         err_typ = ERR_SSU;
      SmplSetup = SEQIMP;
   }
   return err_typ;
}

int use_sqimp_realz (void)              /* use N sequential imp realizations */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (j_par[0] < 0)
         err_typ = ERR_NSI;
      NSeqImp = j_par[0];
   }
   return err_typ;
}

int set_sqimp_prop (void)               /* set sequential imputatn proposals */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (j_par[0] < 0)
         err_typ = ERR_SIP;
      SeqImpIt = j_par[0];
   }
   return err_typ;
}

int sampl_by_scan (void)                /* sample by scan                    */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (Scan == N)
         err_typ = ERR_SCN;
      Scan = Y;
   }
   return err_typ;
}
int sampl_by_step (void)                /* sample by step                    */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (Scan == Y)
         err_typ = ERR_SCN;
      Scan = N;
   }
   return err_typ;
}

int out_scor_iter (void)                /* output scores MC iter statement   */
{
   if (chk_stat ())
   {
      chk_repeat ();
      FilIt = j_par[0];
   }
   return err_typ;
}

int out_space (void)                    /* output spacing statement          */
{
   if (chk_stat ())
   {
      chk_repeat ();
      ItWt = j_par[0];
   }
   return err_typ;
}

int use_full_em (void)                  /* use full (regular) em             */
{
   if (chk_stat ())
   {
      chk_repeat ();
      IPtEM = 0;
   }
   return err_typ;
}

int use_part_em (void)                  /* use partitioned em                */
{
   if (chk_stat ())
   {
      chk_repeat ();
      IPtEM = Y;
   }
   return err_typ;
}

int set_br_conv (void)                  /* set breeding convergence          */
{
   if (chk_stat ())
   {
      chk_repeat ();
      Diff = d_par[0];
   }
   return err_typ;
}

int fit_no_gen (void)                   /* fit environmental model           */
{
   if (chk_stat ())
   {
      chk_repeat ();
      IEn = 1;
   }
   return err_typ;
}

int sel_trait (void)                    /* select trait statement            */
{
   if (chk_stat ())
   {
      if (chk_int_trait ())
      {
         if (!chk_sel_tr ())
            reset_sel_tr ();
         set_sel_tr (0);
      }
   }
   return err_typ;
}
int sel_tr_eff (void)                   /* select trait... effect            */
{
   if (chk_stat ())
   {
      if (chk_trait ())
      {
         if (!chk_sel_tr ())
            reset_sel_tr ();
         if (chk_ff (0))
            set_sel_tr (1);
      }
   }
   return err_typ;
}
int start_mean (void)                   /* start trait mean statement        */
{
   if (chk_stat ())
   {
      if (chk_trait ())
      {
         if (!chk_start_ff (0))
            reset_start_ff (0);
         set_start_ff (0);
      }
   }
   return err_typ;
}
int start_effect (void)                 /* start trait.. effect..            */
{
   if (chk_stat ())
   {
      if (chk_trait ())
      {
         if (chk_ff (1))
         {
            if (!chk_start_ff (1))
               reset_start_ff (1);
            if (chk_levels_ff ())
               set_start_ff (1);
         }
      }
   }
   return err_typ;
}
int start_add_var (void)                /* start additive variance..         */
{
   if (chk_stat ())
   {
      if (!chk_repeat ())
         reset_start_var (0);
      if (chk_var ())
         set_start_var (0);

   }
   return err_typ;
}
int start_add_cor (void)                /* start additive correlation..      */
{
   if (chk_stat ())
   {
      if (!chk_repeat ())
         reset_start_cov (0);
      if (chk_covar ())
         set_start_cor (0);
   }
   return err_typ;
}
int start_add_cov (void)                /* start additive covariance..       */
{
   if (chk_stat ())
   {
      if (!chk_repeat ())
         reset_start_cov (0);
      if (chk_covar ())
         set_start_cov (0);
   }
   return err_typ;
}
int start_res_var (void)                /* start residual variance..         */
{
   if (chk_stat ())
   {
      if (!chk_repeat ())
         reset_start_var (1);
      if (chk_var ())
         set_start_var (1);
   }
   return err_typ;
}
int start_res_cor (void)                /* start residual correlation..      */
{
   if (chk_stat ())
   {
      if (!chk_repeat ())
         reset_start_cov (1);
      if (chk_covar ())
         set_start_cor (1);
   }
   return err_typ;
}
int start_res_cov (void)                /* start residual covariance..       */
{
   if (chk_stat ())
   {
      if (!chk_repeat ())
         reset_start_cov (1);
      if (chk_covar ())
         set_start_cov (1);
   }
   return err_typ;
}
int fit_addi (void)                     /* fit additive covariance..         */
{
   if (chk_stat ())
   {
      if (!chk_repeat ())
         reset_fit_cov (0);
      if (chk_covar_i ())
         set_fit_cov (0);
   }
   return err_typ;
}
int fit_resi (void)                     /* fit residual covariance..         */
{
   if (chk_stat ())
   {
      if (!chk_repeat ())
         reset_fit_cov (1);
      if (chk_covar_i ())
         set_fit_cov (1);
   }
   return err_typ;
}
int set_eigen (void)                    /* set eigenvalues ..                */
{
   if (chk_stat ())
   {
      if (!chk_repeat ())
         reset_set_eigen ();
      set_set_eigen ();
   }
   return err_typ;
}
int comp_eigen (void)                   /* compute eigenvalues statement     */
{
   if (chk_stat ())
   {
      chk_repeat ();
      ICh = 1;
   }
   return err_typ;
}
int check_eigen (void)                  /* check eigenvalues                 */
{
   if (chk_stat ())
   {
      chk_repeat ();
      ChkEig = Y;
   }
   return err_typ;
}
int check_ginv (void)                   /* check ginverse                    */
{
   if (chk_stat ())
   {
      chk_repeat ();
      ChkGnv = Y;
   }
   return err_typ;
}
int check_gmat (void)                   /* check gmatrix                     */
{
   if (chk_stat ())
   {
      chk_repeat ();
      ChkGmt = Y;
   }
   return err_typ;
}

int check_trace (void)                  /* check trace                       */
{
   if (chk_stat ())
   {
      chk_repeat ();
      ChkTrc = Y;
   }
   return err_typ;
}

int check_eig_cmp (void)                /* check eigenvalue computation      */
{
   if (chk_stat ())
   {
      chk_repeat ();
      ChkEigC = Y;
   }
   return err_typ;
}

int check_mc_iter (void)                /* check MC iterations statement     */
{
   if (chk_stat ())
   {
      chk_repeat ();
      ChkIt = j_par[0];
   }
   return err_typ;
}

/* ------ functions for "genedrop" and "lm_auto" parameter statements ------- */

int sim_mark (void)                     /* simulate markers                  */
{
   SIM_PTR p_sim;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_sim (NCH, MK, NTR))
      {
         if (!(p_sim = chk_sim_node (NCH))) /* tests for existing node       */
            p_sim = alloc_sim_node (NCH);
         stor_sim_data (p_sim, NCH, MK, NTR); /* stores data in node         */
      }
   }
   return err_typ;
}

int sim_cmark (void)                    /* simulate markers on chromosome    */
{
   SIM_PTR p_sim;

   if (chk_stat ())
      if (chk_sim (CH, MK, NTR))
      {
         if (!(p_sim = chk_sim_node (CH))) /* tests for existing node        */
            p_sim = alloc_sim_node (CH);
         stor_sim_data (p_sim, CH, MK, NTR); /* stores data in node          */
      }
   return err_typ;
}

int sim_mark_trait (void)               /* simulate markers and traits       */
{
   SIM_PTR p_sim;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_sim (NCH, MK, TR))
      {
         if (!(p_sim = chk_sim_node (NCH))) /* tests for existing node       */
            p_sim = alloc_sim_node (NCH);
         stor_sim_data (p_sim, NCH, MK, TR); /* stores data in node          */
      }
   }
   return err_typ;
}

int sim_cmark_trait (void)              /* sim markers and traits on chrom    */
{
   SIM_PTR p_sim;

   if (chk_stat ())
      if (chk_sim (CH, MK, TR))
      {
         if (!(p_sim = chk_sim_node (CH))) /* tests for existing node        */
            p_sim = alloc_sim_node (CH);
         stor_sim_data (p_sim, CH, MK, TR); /* stores data in node           */
      }
   return err_typ;
}

int sim_cmark_uinhr (void)              /* sim markers on chrom using inher   */
{
   SIM_PTR p_sim;

   PedDatTyp = INHERITANCE;

   if (chk_stat ())
      if (chk_sim (CH, MK, TRL))
      {
         if (!(p_sim = chk_sim_node (CH))) /* tests for existing node        */
            p_sim = alloc_sim_node (CH);
         stor_sim_data (p_sim, CH, MK, TRL); /* stores data in node           */
      }
   return err_typ;
}

int sim_cmark_utrt (void)               /* sim markers on chrom using trait  */
{
   SIM_PTR p_sim;

   if (chk_stat ())
      if (chk_sim (CH, MK, TRL))
      {
         if (!(p_sim = chk_sim_node (CH))) /* tests for existing node        */
            p_sim = alloc_sim_node (CH);
         stor_sim_data (p_sim, CH, MK, TRL); /* stores data in node           */
      }
   return err_typ;
}

int sim_mark_uinhr (void)               /* sim markers using inheritance     */
{
   SIM_PTR p_sim;

   PedDatTyp = INHERITANCE;

   if (chk_stat ())
      if (chk_sim (NCH, MK, TRL))
      {
         if (!(p_sim = chk_sim_node (NCH))) /* tests for existing node       */
            p_sim = alloc_sim_node (NCH);
         stor_sim_data (p_sim, NCH, MK, TRL); /* stores data in node          */
      }
   return err_typ;
}

int sim_mark_utrt (void)                /* sim markers using trait           */
{
   SIM_PTR p_sim;

   if (chk_stat ())
      if (chk_sim (NCH, MK, TRL))
      {
         if (!(p_sim = chk_sim_node (NCH))) /* tests for existing node       */
            p_sim = alloc_sim_node (NCH);
         stor_sim_data (p_sim, NCH, MK, TRL); /* stores data in node          */
      }
   return err_typ;
}

int sela_mark (void)                    /* select markers                     */
{
   SIM_PTR p_sim;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_sim (NCH, AMK, NTR))
      {
         if (p_sim = chk_sim_node (NCH)) /* tests for existing node          */
            free_mark_list (p_sim);
         else
            p_sim = alloc_sim_node (NCH);
         stor_sim_data (p_sim, NCH, AMK, NTR); /* stores data in node        */
      }
   }
   return err_typ;
}

int sela_cmark (void)                   /* select markers on chromosome       */
{
   SIM_PTR p_sim;

   if (chk_stat ())
      if (chk_sim (CH, AMK, NTR))
      {
         if (p_sim = chk_sim_node (CH)) /* tests for existing node            */
            free_mark_list (p_sim);
         else
            p_sim = alloc_sim_node (CH);
         stor_sim_data (p_sim, CH, AMK, NTR); /* stores data in node         */
      }
   return err_typ;
}

int sela_mark_tr (void)                 /* select markers and trait           */
{
   SIM_PTR p_sim;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_sim (NCH, AMK, TR))
      {
         if (p_sim = chk_sim_node (NCH)) /* tests for existing node          */
            free_mark_list (p_sim);
         else
            p_sim = alloc_sim_node (NCH);
         stor_sim_data (p_sim, NCH, AMK, TR); /* stores data in node         */
      }
   }
   return err_typ;
}

int sela_cmark_tr (void)                /* sel markers and trait on chrom     */
{
   SIM_PTR p_sim;

   if (chk_stat ())
      if (chk_sim (CH, AMK, TR))
      {
         if (p_sim = chk_sim_node (CH)) /* tests for existing node            */
            free_mark_list (p_sim);
         else
            p_sim = alloc_sim_node (CH);
         stor_sim_data (p_sim, CH, AMK, TR); /* stores data in node          */
      }
   return err_typ;
}

int sel_mark (void)                     /* select markers                     */
{
   SIM_PTR p_sim;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_sim (NCH, MKL, NTR))
      {
         if (p_sim = chk_sim_node (NCH)) /* tests for existing node          */
            free_mark_list (p_sim);
         else
            p_sim = alloc_sim_node (NCH);
         stor_sim_data (p_sim, NCH, MKL, NTR); /* stores data in node        */
         alloc_mark_list (p_sim);
         stor_mark_list (p_sim, NCH);
      }
   }
   return err_typ;
}

int sel_cmark (void)                    /* select markers on chrom            */
{
   SIM_PTR p_sim;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_sim (CH, MKL, NTR))
      {
         if (p_sim = chk_sim_node (CH)) /* tests for existing node            */
            free_mark_list (p_sim);
         else
            p_sim = alloc_sim_node (CH);
         stor_sim_data (p_sim, CH, MKL, NTR); /* stores data in node         */
         alloc_mark_list (p_sim);
         stor_mark_list (p_sim, CH);
      }
   }
   return err_typ;
}

int sel_mark_tr (void)                  /* select markers and trait           */
{
   SIM_PTR p_sim;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_sim (NCH, MKL, TR))
      {
         if (p_sim = chk_sim_node (NCH)) /* tests for existing node          */
            free_mark_list (p_sim);
         else
            p_sim = alloc_sim_node (NCH);
         stor_sim_data (p_sim, NCH, MKL, TR); /* stores data in node         */
         alloc_mark_list (p_sim);
         stor_mark_list (p_sim, NCH);
      }
   }
   return err_typ;
}

int sel_cmark_tr (void)                 /* select markers and trait on chrom  */
{
   SIM_PTR p_sim;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_sim (CH, MKL, TR))
      {
         if (p_sim = chk_sim_node (CH)) /* tests for existing node            */
            free_mark_list (p_sim);
         else
            p_sim = alloc_sim_node (CH);
         stor_sim_data (p_sim, CH, MKL, TR); /* stores data in node          */
         alloc_mark_list (p_sim);
         stor_mark_list (p_sim, CH);
      }
   }
   return err_typ;
}

int sim_unlinked (void)                 /* simulate unlinked traits           */
{
   UNLNKD_PTR p_unlnkd;
   int   i;

   if (chk_stat ())
   {
      if (!chk_repeat ())               /* tests for existing trait list      */
         free_unlnkd ();
      if (chk_unlnkd ())
         for (i = 0; i < nk_par; i++)
         {
            p_unlnkd = alloc_unlnkd_node (j_par[i]);
            stor_unlnkd (j_par[i], p_unlnkd);
         }                              /* stores trait list                  */
   }
   return err_typ;
}

int map_markr (void)                    /* map markers                       */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_map (NCH, RM, MD))
      {                                 /* tests for existing node           */
         if (p_map = chk_map_node (NCH, MD, AVG))
            free_dist (p_map);
         else
            p_map = add_map_node (NCH, MD, AVG);
         alloc_dist (p_map, MD);
         stor_map (p_map, NCH, RM, MD); /* stores map                        */
      }
   }
   return err_typ;
}

int map_markd (void)                    /* map markers                       */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_map (NCH, DM, MD))
      {                                 /* tests for existing node           */
         if (p_map = chk_map_node (NCH, MD, AVG))
            free_dist (p_map);
         else
            p_map = add_map_node (NCH, MD, AVG);
         alloc_dist (p_map, MD);
         stor_map (p_map, NCH, DM, MD); /* stores map                        */
      }
   }
   return err_typ;
}

int map_markp (void)                    /* map markers                       */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_map (NCH, PM, MD))
      {                                 /* tests for existing node           */
         if (p_map = chk_map_node (NCH, MD, AVG))
            free_dist (p_map);
         else
            p_map = add_map_node (NCH, MD, AVG);
         alloc_dist (p_map, MD);
         stor_map (p_map, NCH, PM, MD); /* stores map                        */
      }
   }
   return err_typ;
}

int map_cmarkr (void)                   /* map markers on chromosome         */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      if (chk_map (CH, RM, MD))
      {                                 /* tests for existing node           */
         if (p_map = chk_map_node (CH, MD, AVG))
            free_dist (p_map);
         else
            p_map = add_map_node (CH, MD, AVG);
         alloc_dist (p_map, MD);
         stor_map (p_map, CH, RM, MD);  /* stores map                        */
      }
   }
   return err_typ;
}

int map_cmarkd (void)                   /* map markers on chromosome         */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      if (chk_map (CH, DM, MD))
      {                                 /* tests for existing node           */
         if (p_map = chk_map_node (CH, MD, AVG))
            free_dist (p_map);
         else
            p_map = add_map_node (CH, MD, AVG);
         alloc_dist (p_map, MD);
         stor_map (p_map, CH, DM, MD);  /* stores map                        */
      }
   }
   return err_typ;
}

int map_cmarkp (void)                   /* map markers on chromosome         */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      if (chk_map (CH, PM, MD))
      {                                 /* tests for existing node           */
         if (p_map = chk_map_node (CH, MD, AVG))
            free_dist (p_map);
         else
            p_map = add_map_node (CH, MD, AVG);
         alloc_dist (p_map, MD);
         stor_map (p_map, CH, PM, MD);  /* stores map                        */
      }
   }
   return err_typ;
}

int map_traitd (void)                   /* map traits                        */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_map (NCH, DM, TD))
      {                                 /* tests for existing node           */
         if (p_map = chk_map_node (NCH, TD, AVG))
            free_dist (p_map);
         else
            p_map = add_map_node (NCH, TD, AVG);
         alloc_dist (p_map, TD);
         stor_map (p_map, NCH, DM, TD); /* stores map                        */
      }
   }
   return err_typ;
}

int map_traitr (void)                   /* map traits                        */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_map (NCH, RM, TD))
      {                                 /* tests for existing node           */
         if (p_map = chk_map_node (NCH, TD, AVG))
            free_dist (p_map);
         else
            p_map = add_map_node (NCH, TD, AVG);
         alloc_dist (p_map, TD);
         stor_map (p_map, NCH, RM, TD); /*stores map                         */
      }
   }
   return err_typ;
}

int map_ctraitd (void)                  /* map traits on chromosomes         */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      if (chk_map (CH, DM, TD))
      {                                 /* tests for existing node           */
         if (p_map = chk_map_node (CH, TD, AVG))
            free_dist (p_map);
         else
            p_map = add_map_node (CH, TD, AVG);
         alloc_dist (p_map, TD);
         stor_map (p_map, CH, DM, TD);  /* stores map                        */
      }
   }
   return err_typ;
}

int map_ctraitr (void)                  /* map traits on chromosomes         */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      if (chk_map (CH, RM, TD))
      {                                 /* tests for existing node           */
         if (p_map = chk_map_node (CH, TD, AVG))
            free_dist (p_map);
         else
            p_map = add_map_node (CH, TD, AVG);
         alloc_dist (p_map, TD);
         stor_map (p_map, CH, RM, TD);  /* stores map                        */
      }
   }
   return err_typ;
}

/*                                       * next 20 functions are for gender
 *                                       * specific maps
 *                                       * ...females first
 */
int map_markr_f (void)                  /* map markers (f)                   */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_map (NCH, RM, MD))
      {                                 /* tests for existing node           */
         if (p_map = chk_map_node (NCH, MD, FEM))
            free_dist (p_map);
         else
            p_map = add_map_node (NCH, MD, FEM);
         alloc_dist (p_map, MD);
         stor_map (p_map, NCH, RM, MD); /* stores map                        */
      }
   }
   return err_typ;
}

int map_markd_f (void)                  /* map markers (f)                   */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_map (NCH, DM, MD))
      {                                 /* tests for existing node           */
         if (p_map = chk_map_node (NCH, MD, FEM))
            free_dist (p_map);
         else
            p_map = add_map_node (NCH, MD, FEM);
         alloc_dist (p_map, MD);
         stor_map (p_map, NCH, DM, MD); /* stores map                        */
      }
   }
   return err_typ;
}

int map_markp_f (void)                  /* map markers (f)                   */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_map (NCH, PM, MD))
      {                                 /* tests for existing node           */
         if (p_map = chk_map_node (NCH, MD, FEM))
            free_dist (p_map);
         else
            p_map = add_map_node (NCH, MD, FEM);
         alloc_dist (p_map, MD);
         stor_map (p_map, NCH, PM, MD); /* stores map                        */
      }
   }
   return err_typ;
}

int map_cmarkr_f (void)                 /* map markers on chromosome (f)     */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      if (chk_map (CH, RM, MD))
      {                                 /* tests for existing node           */
         if (p_map = chk_map_node (CH, MD, FEM))
            free_dist (p_map);
         else
            p_map = add_map_node (CH, MD, FEM);
         alloc_dist (p_map, MD);
         stor_map (p_map, CH, RM, MD);  /* stores map                        */
      }
   }
   return err_typ;
}

int map_cmarkd_f (void)                 /* map markers on chromosome (f)     */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      if (chk_map (CH, DM, MD))
      {                                 /* tests for existing node           */
         if (p_map = chk_map_node (CH, MD, FEM))
            free_dist (p_map);
         else
            p_map = add_map_node (CH, MD, FEM);
         alloc_dist (p_map, MD);
         stor_map (p_map, CH, DM, MD);  /* stores map                        */
      }
   }
   return err_typ;
}

int map_cmarkp_f (void)                 /* map markers on chromosome (f)     */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      if (chk_map (CH, PM, MD))
      {                                 /* tests for existing node           */
         if (p_map = chk_map_node (CH, MD, FEM))
            free_dist (p_map);
         else
            p_map = add_map_node (CH, MD, FEM);
         alloc_dist (p_map, MD);
         stor_map (p_map, CH, PM, MD);  /* stores map                        */
      }
   }
   return err_typ;
}

int map_traitd_f (void)                 /* map traits (f)                    */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_map (NCH, DM, TD))
      {                                 /* tests for existing node           */
         if (p_map = chk_map_node (NCH, TD, FEM))
            free_dist (p_map);
         else
            p_map = add_map_node (NCH, TD, FEM);
         alloc_dist (p_map, TD);
         stor_map (p_map, NCH, DM, TD); /* stores map                        */
      }
   }
   return err_typ;
}

int map_traitr_f (void)                 /* map traits (f)                    */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_map (NCH, RM, TD))
      {                                 /* tests for existing node           */
         if (p_map = chk_map_node (NCH, TD, FEM))
            free_dist (p_map);
         else
            p_map = add_map_node (NCH, TD, FEM);
         alloc_dist (p_map, TD);
         stor_map (p_map, NCH, RM, TD); /* stores map                        */
      }
   }
   return err_typ;
}

int map_ctraitd_f (void)                /* map traits on chromosomes (f)     */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      if (chk_map (CH, DM, TD))
      {                                 /* tests for existing node           */
         if (p_map = chk_map_node (CH, TD, FEM))
            free_dist (p_map);
         else
            p_map = add_map_node (CH, TD, FEM);
         alloc_dist (p_map, TD);
         stor_map (p_map, CH, DM, TD);  /* stores map                        */
      }
   }
   return err_typ;
}

int map_ctraitr_f (void)                /* map traits on chromosomes (f)     */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      if (chk_map (CH, RM, TD))
      {                                 /* tests for existing node           */
         if (p_map = chk_map_node (CH, TD, FEM))
            free_dist (p_map);
         else
            p_map = add_map_node (CH, TD, FEM);
         alloc_dist (p_map, TD);
         stor_map (p_map, CH, RM, TD);  /* stores map                        */
      }
   }
   return err_typ;
}

/*                                       * gender specific maps 
 *                                       * ...males next
 */
int map_markr_m (void)                  /* map markers (m)                   */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_map (NCH, RM, MD))
      {                                 /* tests for existing node           */
         if (p_map = chk_map_node (NCH, MD, MAL))
            free_dist (p_map);
         else
            p_map = add_map_node (NCH, MD, MAL);
         alloc_dist (p_map, MD);
         stor_map (p_map, NCH, RM, MD); /* stores map                        */
      }
   }
   return err_typ;
}

int map_markd_m (void)                  /* map markers (m)                   */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_map (NCH, DM, MD))
      {                                 /* tests for existing node           */
         if (p_map = chk_map_node (NCH, MD, MAL))
            free_dist (p_map);
         else
            p_map = add_map_node (NCH, MD, MAL);
         alloc_dist (p_map, MD);
         stor_map (p_map, NCH, DM, MD); /* stores map                        */
      }
   }
   return err_typ;
}

int map_markp_m (void)                  /* map markers  (m)                  */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_map (NCH, PM, MD))
      {                                 /* tests for existing node           */
         if (p_map = chk_map_node (NCH, MD, MAL))
            free_dist (p_map);
         else
            p_map = add_map_node (NCH, MD, MAL);
         alloc_dist (p_map, MD);
         stor_map (p_map, NCH, PM, MD); /* stores map                        */
      }
   }
   return err_typ;
}

int map_cmarkr_m (void)                 /* map markers on chromosome (m)     */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      if (chk_map (CH, RM, MD))
      {                                 /* tests for existing node           */
         if (p_map = chk_map_node (CH, MD, MAL))
            free_dist (p_map);
         else
            p_map = add_map_node (CH, MD, MAL);
         alloc_dist (p_map, MD);
         stor_map (p_map, CH, RM, MD);  /* stores map                        */
      }
   }
   return err_typ;
}

int map_cmarkd_m (void)                 /* map markers on chromosome (m)     */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      if (chk_map (CH, DM, MD))
      {                                 /* tests for existing node           */
         if (p_map = chk_map_node (CH, MD, MAL))
            free_dist (p_map);
         else
            p_map = add_map_node (CH, MD, MAL);
         alloc_dist (p_map, MD);
         stor_map (p_map, CH, DM, MD);  /* stores map                        */
      }
   }
   return err_typ;
}

int map_cmarkp_m (void)                 /* map markers on chromosome (m)     */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      if (chk_map (CH, PM, MD))
      {                                 /* tests for existing node           */
         if (p_map = chk_map_node (CH, MD, MAL))
            free_dist (p_map);
         else
            p_map = add_map_node (CH, MD, MAL);
         alloc_dist (p_map, MD);
         stor_map (p_map, CH, PM, MD);  /* stores map                        */
      }
   }
   return err_typ;
}

int map_traitd_m (void)                 /* map traits (m)                    */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_map (NCH, DM, TD))
      {                                 /* tests for existing node           */
         if (p_map = chk_map_node (NCH, TD, MAL))
            free_dist (p_map);
         else
            p_map = add_map_node (NCH, TD, MAL);
         alloc_dist (p_map, TD);
         stor_map (p_map, NCH, DM, TD); /* stores map                        */
      }
   }
   return err_typ;
}

int map_traitr_m (void)                 /* map traits (m)                    */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_map (NCH, RM, TD))
      {                                 /* tests for existing node           */
         if (p_map = chk_map_node (NCH, TD, MAL))
            free_dist (p_map);
         else
            p_map = add_map_node (NCH, TD, MAL);
         alloc_dist (p_map, TD);
         stor_map (p_map, NCH, RM, TD); /* stores map                        */
      }
   }
   return err_typ;
}

int map_ctraitd_m (void)                /* map traits on chromosomes (m)     */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      if (chk_map (CH, DM, TD))
      {                                 /* tests for existing node           */
         if (p_map = chk_map_node (CH, TD, MAL))
            free_dist (p_map);
         else
            p_map = add_map_node (CH, TD, MAL);
         alloc_dist (p_map, TD);
         stor_map (p_map, CH, DM, TD);  /* stores stores map                 */
      }
   }
   return err_typ;
}

int map_ctraitr_m (void)                /* map traits on chromosomes (m)     */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      if (chk_map (CH, RM, TD))
      {                                 /* tests for existing node           */
         if (p_map = chk_map_node (CH, TD, MAL))
            free_dist (p_map);
         else
            p_map = add_map_node (CH, TD, MAL);
         alloc_dist (p_map, TD);
         stor_map (p_map, CH, RM, TD);  /* stores map                        */
      }
   }
   return err_typ;
}

int map_ctr_extd_f (void)               /* map extrn distance on chrom (f)    */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
      if (chk_vmap (CH, XDM))
      {
         if (!(p_vmap = chk_vmap_node (CH, FEM))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (CH, FEM);
         else
         {
            if (chk_vext (p_vmap->p_beg, CH)) /* checks for beginning map    */
               free_vdist (p_vmap->p_beg);
            if (chk_vext (p_vmap->p_end, CH)) /* checks for ending map       */
               free_vdist (p_vmap->p_end);
         }
         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();
         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_beg, XDM, qfunc_doubdec);
         alloc_vdist (p_vmap->p_end);
         stor_vdist (p_vmap->p_end, XDM, qfunc_doub);
      }
   return err_typ;
}

int map_ctr_extr_f (void)               /* map extrn recomb frac on chrom (f) */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
      if (chk_vmap (CH, XRM))
      {
         if (!(p_vmap = chk_vmap_node (CH, FEM))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (CH, FEM);
         else
         {
            if (chk_vext (p_vmap->p_beg, CH)) /* checks for beginning map    */
               free_vdist (p_vmap->p_beg);
            if (chk_vext (p_vmap->p_end, CH)) /* checks for ending map       */
               free_vdist (p_vmap->p_end);
         }
         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();
         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_beg, XRM, qfunc_doubdec);
         alloc_vdist (p_vmap->p_end);
         stor_vdist (p_vmap->p_end, XRM, qfunc_doub);
      }
   return err_typ;
}

int map_ctr_extd_m (void)               /* map extrn distance on chrom (m)   */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
      if (chk_vmap (CH, XDM))
      {
         if (!(p_vmap = chk_vmap_node (CH, MAL))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (CH, MAL);
         else
         {
            if (chk_vext (p_vmap->p_beg, CH)) /* checks for beginning map    */
               free_vdist (p_vmap->p_beg);
            if (chk_vext (p_vmap->p_end, CH)) /* checks for ending map       */
               free_vdist (p_vmap->p_end);
         }
         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();
         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_beg, XDM, qfunc_doubdec);
         alloc_vdist (p_vmap->p_end);
         stor_vdist (p_vmap->p_end, XDM, qfunc_doub);
      }
   return err_typ;
}

int map_ctr_extr_m (void)               /* map extrn recomb frac on chrom (m) */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
      if (chk_vmap (CH, XRM))
      {
         if (!(p_vmap = chk_vmap_node (CH, MAL))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (CH, MAL);
         else
         {
            if (chk_vext (p_vmap->p_beg, CH)) /* checks for beginning map    */
               free_vdist (p_vmap->p_beg);
            if (chk_vext (p_vmap->p_end, CH)) /* checks for ending map       */
               free_vdist (p_vmap->p_end);
         }
         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();
         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_beg, XRM, qfunc_doubdec);
         alloc_vdist (p_vmap->p_end);
         stor_vdist (p_vmap->p_end, XRM, qfunc_doub);
      }
   return err_typ;
}

int map_ctr_extd (void)                 /* map extrn distances on chrom      */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
      if (chk_vmap (CH, XDM))
      {
         if (!(p_vmap = chk_vmap_node (CH, AVG))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (CH, AVG);
         else
         {
            if (chk_vext (p_vmap->p_beg, CH)) /* checks for beginning map    */
               free_vdist (p_vmap->p_beg);
            if (chk_vext (p_vmap->p_end, CH)) /* checks for ending map       */
               free_vdist (p_vmap->p_end);
         }
         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();
         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_beg, XDM, qfunc_doubdec);
         alloc_vdist (p_vmap->p_end);
         stor_vdist (p_vmap->p_end, XDM, qfunc_doub);
      }
   return err_typ;
}

int map_ctr_extr (void)                 /* map extrn recomb fracts on chrom  */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
      if (chk_vmap (CH, XRM))
      {
         if (!(p_vmap = chk_vmap_node (CH, AVG))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (CH, AVG);
         else
         {
            if (chk_vext (p_vmap->p_beg, CH)) /* checks for beginning map    */
               free_vdist (p_vmap->p_beg);
            if (chk_vext (p_vmap->p_end, CH)) /* checks for ending map       */
               free_vdist (p_vmap->p_end);
         }
         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();
         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_beg, XRM, qfunc_doubdec);
         alloc_vdist (p_vmap->p_end);
         stor_vdist (p_vmap->p_end, XRM, qfunc_doub);
      }
   return err_typ;
}

int map_tr_extd_f (void)                /* map extrn distances (f)           */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_vmap (NCH, XDM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, FEM))) /* if no vmap node, allcs */
            p_vmap = add_vmap_node (NCH, FEM);
         else
         {
            if (chk_vext (p_vmap->p_beg, NCH)) /* checks for beginning map   */
               free_vdist (p_vmap->p_beg);
            if (chk_vext (p_vmap->p_end, NCH)) /* checks for ending map      */
               free_vdist (p_vmap->p_end);
         }
         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();
         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_beg, XDM, qfunc_doubdec);
         alloc_vdist (p_vmap->p_end);
         stor_vdist (p_vmap->p_end, XDM, qfunc_doub);
      }
   }
   return err_typ;
}

int map_tr_extr_f (void)                /* map extrn recomb fracts (f)       */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_vmap (NCH, XRM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, FEM))) /* if no vmap node, allcs */
            p_vmap = add_vmap_node (NCH, FEM);
         else
         {
            if (chk_vext (p_vmap->p_beg, NCH)) /* checks for beginning map   */
               free_vdist (p_vmap->p_beg);
            if (chk_vext (p_vmap->p_end, NCH)) /* checks for ending map      */
               free_vdist (p_vmap->p_end);
         }
         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();
         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_beg, XRM, qfunc_doubdec);
         alloc_vdist (p_vmap->p_end);
         stor_vdist (p_vmap->p_end, XRM, qfunc_doub);
      }
   }
   return err_typ;
}

int map_tr_extd_m (void)                /* map extrn distances (m)           */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_vmap (NCH, XDM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, MAL))) /* if no vmap node, allcs */
            p_vmap = add_vmap_node (NCH, MAL);
         else
         {
            if (chk_vext (p_vmap->p_beg, NCH)) /* checks for beginning map   */
               free_vdist (p_vmap->p_beg);
            if (chk_vext (p_vmap->p_end, NCH)) /* checks for ending map      */
               free_vdist (p_vmap->p_end);
         }
         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();
         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_beg, XDM, qfunc_doubdec);
         alloc_vdist (p_vmap->p_end);
         stor_vdist (p_vmap->p_end, XDM, qfunc_doub);
      }
   }
   return err_typ;
}

int map_tr_extr_m (void)                /* map extrn recomb fracts (m)       */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_vmap (NCH, XRM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, MAL))) /* if no vmap node, allcs */
            p_vmap = add_vmap_node (NCH, MAL);
         else
         {
            if (chk_vext (p_vmap->p_beg, NCH)) /* checks for beginning map   */
               free_vdist (p_vmap->p_beg);
            if (chk_vext (p_vmap->p_end, NCH)) /* checks for ending map      */
               free_vdist (p_vmap->p_end);
         }
         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();
         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_beg, XRM, qfunc_doubdec);
         alloc_vdist (p_vmap->p_end);
         stor_vdist (p_vmap->p_end, XRM, qfunc_doub);
      }
   }
   return err_typ;
}

int map_tr_extd (void)                  /* map extrn distances               */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_vmap (NCH, XDM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, AVG))) /* if no vmap node, allcs */
            p_vmap = add_vmap_node (NCH, AVG);
         else
         {
            if (chk_vext (p_vmap->p_beg, NCH)) /* checks for beginning map   */
               free_vdist (p_vmap->p_beg);
            if (chk_vext (p_vmap->p_end, NCH)) /* checks for ending map      */
               free_vdist (p_vmap->p_end);
         }
         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();
         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_beg, XDM, qfunc_doubdec);
         alloc_vdist (p_vmap->p_end);
         stor_vdist (p_vmap->p_end, XDM, qfunc_doub);
      }
   }
   return err_typ;
}

int map_tr_extr (void)                  /* map extrn recomb fracts           */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_vmap (NCH, XRM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, AVG))) /* if no vmap node, allcs */
            p_vmap = add_vmap_node (NCH, AVG);
         else
         {
            if (chk_vext (p_vmap->p_beg, NCH)) /* checks for beginning map   */
               free_vdist (p_vmap->p_beg);
            if (chk_vext (p_vmap->p_end, NCH)) /* checks for ending map      */
               free_vdist (p_vmap->p_end);
         }
         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();
         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_beg, XRM, qfunc_doubdec);
         alloc_vdist (p_vmap->p_end);
         stor_vdist (p_vmap->p_end, XRM, qfunc_doub);
      }
   }
   return err_typ;
}

int map_ctr_begd_f (void)               /* map start distances on chrom (f)  */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
      if (chk_vmap (CH, BDM))
      {
         if (!(p_vmap = chk_vmap_node (CH, FEM))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (CH, FEM);
         else if (chk_vext (p_vmap->p_beg, CH)) /* checks for beginning map  */
            free_vdist (p_vmap->p_beg);

         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_beg, BDM, qfunc_doubdec);
      }
   return err_typ;
}

int map_ctr_begr_f (void)               /* map start recomb frct on chrom (f) */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
      if (chk_vmap (CH, BRM))
      {
         if (!(p_vmap = chk_vmap_node (CH, FEM))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (CH, FEM);
         else if (chk_vext (p_vmap->p_beg, CH)) /* checks for beginning map  */
            free_vdist (p_vmap->p_beg);

         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_beg, BRM, qfunc_doubdec);
      }
   return err_typ;
}

int map_ctr_begd_m (void)               /* map start distances on chrom (m)  */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
      if (chk_vmap (CH, BDM))
      {
         if (!(p_vmap = chk_vmap_node (CH, MAL))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (CH, MAL);
         else if (chk_vext (p_vmap->p_beg, CH)) /* checks for beginning map  */
            free_vdist (p_vmap->p_beg);

         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_beg, BDM, qfunc_doubdec);
      }
   return err_typ;
}

int map_ctr_begr_m (void)               /* map start recomb frct on chrom (m) */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
      if (chk_vmap (CH, BRM))
      {
         if (!(p_vmap = chk_vmap_node (CH, MAL))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (CH, MAL);
         else if (chk_vext (p_vmap->p_beg, CH)) /* checks for beginning map  */
            free_vdist (p_vmap->p_beg);

         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_beg, BRM, qfunc_doubdec);
      }
   return err_typ;
}

int map_ctr_begd (void)                 /* map start distances on chrom      */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
      if (chk_vmap (CH, BDM))
      {
         if (!(p_vmap = chk_vmap_node (CH, AVG))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (CH, AVG);
         else if (chk_vext (p_vmap->p_beg, CH)) /* checks for beginning map  */
            free_vdist (p_vmap->p_beg);

         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_beg, BDM, qfunc_doubdec);
      }
   return err_typ;
}

int map_ctr_begr (void)                 /* map start recomb fracts on chrom  */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
      if (chk_vmap (CH, BRM))
      {
         if (!(p_vmap = chk_vmap_node (CH, AVG))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (CH, AVG);
         else if (chk_vext (p_vmap->p_beg, CH)) /* checks for beginning map  */
            free_vdist (p_vmap->p_beg);

         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_beg, BRM, qfunc_doubdec);
      }
   return err_typ;
}

int map_tr_begd_f (void)                /* map start distances (f)           */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_vmap (NCH, BDM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, FEM))) /* if no vmap node, allcs */
            p_vmap = add_vmap_node (NCH, FEM);
         else if (chk_vext (p_vmap->p_beg, NCH)) /* checks for beginning map */
            free_vdist (p_vmap->p_beg);

         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_beg, BDM, qfunc_doubdec);
      }
   }
   return err_typ;
}

int map_tr_begr_f (void)                /* map start recomb fracts (f)       */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_vmap (NCH, BRM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, FEM))) /* if no vmap node, allcs */
            p_vmap = add_vmap_node (NCH, FEM);
         else if (chk_vext (p_vmap->p_beg, NCH)) /* checks for beginning map */
            free_vdist (p_vmap->p_beg);

         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_beg, BRM, qfunc_doubdec);
      }
   }
   return err_typ;
}

int map_tr_begd_m (void)                /* map start distances (m)           */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_vmap (NCH, BDM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, MAL))) /* if no vmap node, allcs */
            p_vmap = add_vmap_node (NCH, MAL);
         else if (chk_vext (p_vmap->p_beg, NCH)) /* checks for beginning map */
            free_vdist (p_vmap->p_beg);

         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_beg, BDM, qfunc_doubdec);
      }
   }
   return err_typ;
}

int map_tr_begr_m (void)                /* map start recomb fracts (m)       */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_vmap (NCH, BRM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, MAL))) /* if no vmap node, allcs */
            p_vmap = add_vmap_node (NCH, MAL);
         else if (chk_vext (p_vmap->p_beg, NCH)) /* checks for beginning map */
            free_vdist (p_vmap->p_beg);

         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_beg, BRM, qfunc_doubdec);
      }
   }
   return err_typ;
}

int map_tr_begd (void)                  /* map start distances               */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_vmap (NCH, BDM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, AVG))) /* if no vmap node, allcs */
            p_vmap = add_vmap_node (NCH, AVG);
         else if (chk_vext (p_vmap->p_beg, NCH)) /* checks for beginning map */
            free_vdist (p_vmap->p_beg);

         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_beg, BDM, qfunc_doubdec);
      }
   }
   return err_typ;
}

int map_tr_begr (void)                  /* map start recomb fracts           */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_vmap (NCH, BRM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, AVG))) /* if no vmap node, allcs */
            p_vmap = add_vmap_node (NCH, AVG);
         else if (chk_vext (p_vmap->p_beg, NCH)) /* checks for beginning map */
            free_vdist (p_vmap->p_beg);

         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_beg, BRM, qfunc_doubdec);
      }
   }
   return err_typ;
}

int map_ctr_endd_f (void)               /* map end distances on chrom (f)     */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
      if (chk_vmap (CH, EDM))
      {
         if (!(p_vmap = chk_vmap_node (CH, FEM))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (CH, FEM);
         else if (chk_vext (p_vmap->p_end, CH)) /* checks for ending map     */
            free_vdist (p_vmap->p_end);

         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_end);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_end, EDM, qfunc_doub);
      }
   return err_typ;
}

int map_ctr_endr_f (void)               /* map end recomb frct on chrom (f)  */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
      if (chk_vmap (CH, ERM))
      {
         if (!(p_vmap = chk_vmap_node (CH, FEM))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (CH, FEM);
         else if (chk_vext (p_vmap->p_end, CH)) /* checks for ending map     */
            free_vdist (p_vmap->p_end);

         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_end);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_end, ERM, qfunc_doub);
      }
   return err_typ;
}

int map_ctr_endd_m (void)               /* map end distances on chrom (m)    */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
      if (chk_vmap (CH, EDM))
      {
         if (!(p_vmap = chk_vmap_node (CH, MAL))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (CH, MAL);
         else if (chk_vext (p_vmap->p_end, CH)) /* checks for ending map     */
            free_vdist (p_vmap->p_end);

         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_end);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_end, EDM, qfunc_doub);
      }
   return err_typ;
}

int map_ctr_endr_m (void)               /* map end recomb frct on chrom (m)  */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
      if (chk_vmap (CH, ERM))
      {
         if (!(p_vmap = chk_vmap_node (CH, MAL))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (CH, MAL);
         else if (chk_vext (p_vmap->p_end, CH)) /* checks for ending map     */
            free_vdist (p_vmap->p_end);

         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_end);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_end, ERM, qfunc_doub);
      }
   return err_typ;
}

int map_ctr_endd (void)                 /* map end distances on chrom        */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
      if (chk_vmap (CH, EDM))
      {
         if (!(p_vmap = chk_vmap_node (CH, AVG))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (CH, AVG);
         else if (chk_vext (p_vmap->p_end, CH)) /* checks for ending map     */
            free_vdist (p_vmap->p_end);

         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_end);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_end, EDM, qfunc_doub);
      }
   return err_typ;
}

int map_ctr_endr (void)                 /* map end recomb fracts on chrom    */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
      if (chk_vmap (CH, ERM))
      {
         if (!(p_vmap = chk_vmap_node (CH, AVG))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (CH, AVG);
         else if (chk_vext (p_vmap->p_end, CH)) /* checks for ending map     */
            free_vdist (p_vmap->p_end);

         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_end);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_end, ERM, qfunc_doub);
      }
   return err_typ;
}

int map_tr_endd_f (void)                /* map end distances (f)             */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_vmap (NCH, EDM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, FEM))) /* if no vmap node, allcs */
            p_vmap = add_vmap_node (NCH, FEM);
         else if (chk_vext (p_vmap->p_end, NCH)) /* checks for ending map    */
            free_vdist (p_vmap->p_end);

         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_end);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_end, EDM, qfunc_doub);
      }
   }
   return err_typ;
}

int map_tr_endr_f (void)                /* map end recomb fracts (f)         */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_vmap (NCH, ERM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, FEM))) /* if no vmap node, allcs */
            p_vmap = add_vmap_node (NCH, FEM);
         else if (chk_vext (p_vmap->p_end, NCH)) /* checks for ending map    */
            free_vdist (p_vmap->p_end);

         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_end);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_end, ERM, qfunc_doub);
      }
   }
   return err_typ;
}

int map_tr_endd_m (void)                /* map end distances (m)             */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_vmap (NCH, EDM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, MAL))) /* if no vmap node, allcs */
            p_vmap = add_vmap_node (NCH, MAL);
         else if (chk_vext (p_vmap->p_end, NCH)) /* checks for ending map    */
            free_vdist (p_vmap->p_end);

         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_end);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_end, EDM, qfunc_doub);
      }
   }
   return err_typ;
}

int map_tr_endr_m (void)                /* map end recomb fracts (m)         */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_vmap (NCH, ERM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, MAL))) /* if no vmap node, allcs */
            p_vmap = add_vmap_node (NCH, MAL);
         else if (chk_vext (p_vmap->p_end, NCH)) /* checks for ending map    */
            free_vdist (p_vmap->p_end);

         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_end);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_end, ERM, qfunc_doub);
      }
   }
   return err_typ;
}

int map_tr_endd (void)                  /* map end distances                 */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_vmap (NCH, EDM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, AVG))) /* if no vmap node, allcs */
            p_vmap = add_vmap_node (NCH, AVG);
         else if (chk_vext (p_vmap->p_end, NCH)) /* checks for ending map    */
            free_vdist (p_vmap->p_end);

         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_end);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_end, EDM, qfunc_doub);
      }
   }
   return err_typ;
}

int map_tr_endr (void)                  /* map end recomb fracts             */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_vmap (NCH, ERM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, AVG))) /* if no vmap node, allcs */
            p_vmap = add_vmap_node (NCH, AVG);
         else if (chk_vext (p_vmap->p_end, NCH)) /* checks for ending map    */
            free_vdist (p_vmap->p_end);

         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_end);   /* allocs, stores dists & recomb frcs */
         stor_vdist (p_vmap->p_end, ERM, qfunc_doub);
      }
   }
   return err_typ;
}

int map_ctr_antv_f (void)               /* map all trait intvls on chrom (f)  */
{
   VMAP_PTR p_vmap;
   NTMAP_PTR p_vint;

   if (chk_stat ())
      if (chk_vmap (CH, AM))
      {
         if (!(p_vmap = chk_vmap_node (CH, FEM))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (CH, FEM);
         if (p_vint = chk_vint (p_vmap, CH, AM, 0)) /* checks for intv "0"   */
            free_vprop (p_vint);
         else
            p_vint = add_vint (p_vmap, 0); /* adds prop list node for "0"    */
         alloc_vprop (p_vint);          /* allocs and stores ist              */
         stor_vprop (p_vint);
      }
   return err_typ;
}

int map_ctr_antv_m (void)               /* map all trait intvls on chrom (m) */
{
   VMAP_PTR p_vmap;
   NTMAP_PTR p_vint;

   if (chk_stat ())
      if (chk_vmap (CH, AM))
      {
         if (!(p_vmap = chk_vmap_node (CH, MAL))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (CH, MAL);
         if (p_vint = chk_vint (p_vmap, CH, AM, 0)) /* checks for intv "0"   */
            free_vprop (p_vint);
         else
            p_vint = add_vint (p_vmap, 0); /* adds prop list node for "0"    */
         alloc_vprop (p_vint);          /* allocs and stores ist              */
         stor_vprop (p_vint);
      }
   return err_typ;
}

int map_ctr_antv (void)                 /* map all trait intervals on chrom  */
{
   VMAP_PTR p_vmap;
   NTMAP_PTR p_vint;

   if (chk_stat ())
      if (chk_vmap (CH, AM))
      {
         if (!(p_vmap = chk_vmap_node (CH, AVG))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (CH, AVG);
         if (p_vint = chk_vint (p_vmap, CH, AM, 0)) /* checks for intv "0"   */
            free_vprop (p_vint);
         else
            p_vint = add_vint (p_vmap, 0); /* adds prop list node for "0"    */
         alloc_vprop (p_vint);          /* allocs, stores list                */
         stor_vprop (p_vint);
      }
   return err_typ;
}

int map_tr_antv_f (void)                /* map all trait intervals (f)       */
{
   VMAP_PTR p_vmap;
   NTMAP_PTR p_vint;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_vmap (NCH, AM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, FEM))) /* if no vmap node, allcs */
            p_vmap = add_vmap_node (NCH, FEM);
         if (p_vint = chk_vint (p_vmap, NCH, AM, 0)) /* checks for intv "0"  */
            free_vprop (p_vint);
         else
            p_vint = add_vint (p_vmap, 0); /* adds prop list node for "0"    */
         alloc_vprop (p_vint);          /* allocs and stores ist              */
         stor_vprop (p_vint);
      }
   }
   return err_typ;
}

int map_tr_antv_m (void)                /* map all trait intervals (m)       */
{
   VMAP_PTR p_vmap;
   NTMAP_PTR p_vint;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_vmap (NCH, AM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, MAL))) /* if no vmap node, allcs */
            p_vmap = add_vmap_node (NCH, MAL);
         if (p_vint = chk_vint (p_vmap, NCH, AM, 0)) /* checks for intv "0"  */
            free_vprop (p_vint);
         else
            p_vint = add_vint (p_vmap, 0); /* adds prop list node for "0"    */
         alloc_vprop (p_vint);          /* allocs and stores ist              */
         stor_vprop (p_vint);
      }
   }
   return err_typ;
}

int map_tr_antv (void)                  /* map all trait intervals           */
{
   VMAP_PTR p_vmap;
   NTMAP_PTR p_vint;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_vmap (NCH, AM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, AVG))) /* if no vmap node, allcs */
            p_vmap = add_vmap_node (NCH, AVG);
         if (p_vint = chk_vint (p_vmap, NCH, AM, 0)) /* checks for intv "0"  */
            free_vprop (p_vint);
         else
            p_vint = add_vint (p_vmap, 0); /* adds prop list node for "0"    */
         alloc_vprop (p_vint);          /* allocs and stores ist              */
         stor_vprop (p_vint);
      }
   }
   return err_typ;
}

int map_ctr_intv_f (void)               /* map trait intvl prop on chrom (f) */
{
   int   i;
   VMAP_PTR p_vmap;
   NTMAP_PTR p_vint;

   if (chk_stat ())
      if (chk_vmap (CH, IM))
      {
         if (!(p_vmap = chk_vmap_node (CH, FEM))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (CH, FEM);

         for (i = 1; i <= nk_par; ++i)  /* for each specified map interval,   */
         {                              /* checks for existing list of props  */
            if (p_vint = chk_vint (p_vmap, CH, IM, j_par[i + 1]))
               free_vprop (p_vint);
            else
               p_vint = add_vint (p_vmap, j_par[i + 1]); /* adds interval    */
            alloc_vprop (p_vint);       /* allocs, stores proportion list     */
            stor_vprop (p_vint);
         }
      }
   return err_typ;
}

int map_ctr_intv_m (void)               /* map trait intvl prop on chrom (m) */
{
   int   i;
   VMAP_PTR p_vmap;
   NTMAP_PTR p_vint;

   if (chk_stat ())
      if (chk_vmap (CH, IM))
      {
         if (!(p_vmap = chk_vmap_node (CH, MAL))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (CH, MAL);

         for (i = 1; i <= nk_par; ++i)  /* for each specified map interval,   */
         {                              /* checks for existing list of props  */
            if (p_vint = chk_vint (p_vmap, CH, IM, j_par[i + 1]))
               free_vprop (p_vint);
            else
               p_vint = add_vint (p_vmap, j_par[i + 1]); /* adds interval    */
            alloc_vprop (p_vint);       /* allocs, stores proportion list     */
            stor_vprop (p_vint);
         }
      }
   return err_typ;
}

int map_ctr_intv (void)                 /* map trait interval prop on chrom  */
{
   int   i;
   VMAP_PTR p_vmap;
   NTMAP_PTR p_vint;

   if (chk_stat ())
      if (chk_vmap (CH, IM))
      {
         if (!(p_vmap = chk_vmap_node (CH, AVG))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (CH, AVG);

         for (i = 1; i <= nk_par; ++i)  /* for each specified map interval,   */
         {                              /* checks for existing list of props  */
            if (p_vint = chk_vint (p_vmap, CH, IM, j_par[i + 1]))
               free_vprop (p_vint);
            else
               p_vint = add_vint (p_vmap, j_par[i + 1]); /* adds interval    */
            alloc_vprop (p_vint);       /* allocs, stores proportion list     */
            stor_vprop (p_vint);
         }
      }
   return err_typ;
}

int map_tr_intv_f (void)                /* map trait intervl proportions (f) */
{
   int   i;
   VMAP_PTR p_vmap;
   NTMAP_PTR p_vint;

   if (chk_stat ())
      if (chk_vmap (NCH, IM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, FEM))) /* if no vmap node, allcs */
            p_vmap = add_vmap_node (NCH, FEM);

         for (i = 0; i < nk_par; ++i)   /* for each specified map interval,   */
         {                              /* checks for existing list of props  */
            if (p_vint = chk_vint (p_vmap, NCH, IM, j_par[i + 1]))
               free_vprop (p_vint);
            else
               p_vint = add_vint (p_vmap, j_par[i + 1]); /* adds interval    */
            alloc_vprop (p_vint);       /* allocs, stores proportion list     */
            stor_vprop (p_vint);
         }
      }
   return err_typ;
}

int map_tr_intv_m (void)                /* map trait intervl proportions (m) */
{
   int   i;
   VMAP_PTR p_vmap;
   NTMAP_PTR p_vint;

   if (chk_stat ())
      if (chk_vmap (NCH, IM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, MAL))) /* if no vmap node, allcs */
            p_vmap = add_vmap_node (NCH, MAL);

         for (i = 0; i < nk_par; ++i)   /* for each specified map interval,   */
         {                              /* checks for existing list of props  */
            if (p_vint = chk_vint (p_vmap, NCH, IM, j_par[i + 1]))
               free_vprop (p_vint);
            else
               p_vint = add_vint (p_vmap, j_par[i + 1]); /* adds interval    */
            alloc_vprop (p_vint);       /* allocs, stores proportion list     */
            stor_vprop (p_vint);
         }
      }
   return err_typ;
}

int map_tr_intv (void)                  /* map trait interval proportions    */
{
   int   i;
   VMAP_PTR p_vmap;
   NTMAP_PTR p_vint;

   if (chk_stat ())
      if (chk_vmap (NCH, IM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, AVG))) /* if no vmap node, allcs */
            p_vmap = add_vmap_node (NCH, AVG);

         for (i = 0; i < nk_par; ++i)   /* for each specified map interval,   */
         {                              /* checks for existing list of props  */
            if (p_vint = chk_vint (p_vmap, NCH, IM, j_par[i + 1]))
               free_vprop (p_vint);
            else
               p_vint = add_vint (p_vmap, j_par[i + 1]); /* adds interval    */
            alloc_vprop (p_vint);       /* allocs, stores proportion list     */
            stor_vprop (p_vint);
         }
      }
   return err_typ;
}

/*                                       * distance in the following functions
 *                                       * was specified as Kosambi, rather
 *                                       * than Haldane  
 */
int map_markKd (void)                   /* map markers (Kosambi)             */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_map (NCH, KDM, MD))
      {                                 /* tests for existing node           */
         if (p_map = chk_map_node (NCH, MD, AVG))
            free_dist (p_map);
         else
            p_map = add_map_node (NCH, MD, AVG);
         alloc_dist (p_map, MD);
         stor_map (p_map, NCH, KDM, MD); /* stores map                       */
      }
   }
   return err_typ;
}

int map_markKp (void)                   /* map markers (Kosambi)             */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_map (NCH, KPM, MD))
      {                                 /* tests for existing node           */
         if (p_map = chk_map_node (NCH, MD, AVG))
            free_dist (p_map);
         else
            p_map = add_map_node (NCH, MD, AVG);
         alloc_dist (p_map, MD);
         stor_map (p_map, NCH, KPM, MD); /* stores map                       */
      }
   }
   return err_typ;
}

int map_cmarkKd (void)                  /* map markers on chromosome         */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      if (chk_map (CH, KDM, MD))
      {                                 /* tests for existing node           */
         if (p_map = chk_map_node (CH, MD, AVG))
            free_dist (p_map);
         else
            p_map = add_map_node (CH, MD, AVG);
         alloc_dist (p_map, MD);
         stor_map (p_map, CH, KDM, MD); /* stores map                        */
      }
   }
   return err_typ;
}

int map_cmarkKp (void)                  /* map markers on chromosome         */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      if (chk_map (CH, KPM, MD))
      {                                 /* tests for existing node           */
         if (p_map = chk_map_node (CH, MD, AVG))
            free_dist (p_map);
         else
            p_map = add_map_node (CH, MD, AVG);
         alloc_dist (p_map, MD);
         stor_map (p_map, CH, KPM, MD); /* stores map                        */
      }
   }
   return err_typ;
}

int map_traitKd (void)                  /* map traits                        */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_map (NCH, KDM, TD))
      {                                 /* tests for existing node           */
         if (p_map = chk_map_node (NCH, TD, AVG))
            free_dist (p_map);
         else
            p_map = add_map_node (NCH, TD, AVG);
         alloc_dist (p_map, TD);
         stor_map (p_map, NCH, KDM, TD); /* stores map                       */
      }
   }
   return err_typ;
}

int map_ctraitKd (void)                 /* map traits on chromosomes         */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      if (chk_map (CH, KDM, TD))
      {                                 /* tests for existing node           */
         if (p_map = chk_map_node (CH, TD, AVG))
            free_dist (p_map);
         else
            p_map = add_map_node (CH, TD, AVG);
         alloc_dist (p_map, TD);
         stor_map (p_map, CH, KDM, TD); /* stores map                        */
      }
   }
   return err_typ;
}

/*                                       * next functions are for gender
 *                                       * specific maps
 *                                       * ...females first
 */

int map_markKd_f (void)                 /* map markers (f)                   */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_map (NCH, KDM, MD))
      {                                 /* tests for existing node           */
         if (p_map = chk_map_node (NCH, MD, FEM))
            free_dist (p_map);
         else
            p_map = add_map_node (NCH, MD, FEM);
         alloc_dist (p_map, MD);
         stor_map (p_map, NCH, KDM, MD); /* stores map                       */
      }
   }
   return err_typ;
}

int map_markKp_f (void)                 /* map markers (f)                   */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_map (NCH, KPM, MD))
      {                                 /* tests for existing node           */
         if (p_map = chk_map_node (NCH, MD, FEM))
            free_dist (p_map);
         else
            p_map = add_map_node (NCH, MD, FEM);
         alloc_dist (p_map, MD);
         stor_map (p_map, NCH, KPM, MD); /* stores map                       */
      }
   }
   return err_typ;
}

int map_cmarkKd_f (void)                /* map markers on chromosome (f)     */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      if (chk_map (CH, KDM, MD))
      {                                 /* tests for existing node           */
         if (p_map = chk_map_node (CH, MD, FEM))
            free_dist (p_map);
         else
            p_map = add_map_node (CH, MD, FEM);
         alloc_dist (p_map, MD);
         stor_map (p_map, CH, KDM, MD); /* stores map                        */
      }
   }
   return err_typ;
}

int map_cmarkKp_f (void)                /* map markers on chromosome (f)     */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      if (chk_map (CH, KPM, MD))
      {                                 /* tests for existing node           */
         if (p_map = chk_map_node (CH, MD, FEM))
            free_dist (p_map);
         else
            p_map = add_map_node (CH, MD, FEM);
         alloc_dist (p_map, MD);
         stor_map (p_map, CH, KPM, MD); /* stores map                        */
      }
   }
   return err_typ;
}

int map_traitKd_f (void)                /* map traits (f)                    */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_map (NCH, KDM, TD))
      {                                 /* tests for existing node           */
         if (p_map = chk_map_node (NCH, TD, FEM))
            free_dist (p_map);
         else
            p_map = add_map_node (NCH, TD, FEM);
         alloc_dist (p_map, TD);
         stor_map (p_map, NCH, KDM, TD); /* stores map                       */
      }
   }
   return err_typ;
}

int map_ctraitKd_f (void)               /* map traits on chromosomes (f)     */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      if (chk_map (CH, KDM, TD))
      {                                 /* tests for existing node           */
         if (p_map = chk_map_node (CH, TD, FEM))
            free_dist (p_map);
         else
            p_map = add_map_node (CH, TD, FEM);
         alloc_dist (p_map, TD);
         stor_map (p_map, CH, KDM, TD); /* stores map                        */
      }
   }
   return err_typ;
}

/*                                       * gender specific maps 
 *                                       * ...males next
 */
int map_markKd_m (void)                 /* map markers (m)                   */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_map (NCH, KDM, MD))
      {                                 /* tests for existing node           */
         if (p_map = chk_map_node (NCH, MD, MAL))
            free_dist (p_map);
         else
            p_map = add_map_node (NCH, MD, MAL);
         alloc_dist (p_map, MD);
         stor_map (p_map, NCH, KDM, MD); /* stores map                       */
      }
   }
   return err_typ;
}

int map_markKp_m (void)                 /* map markers  (m)                  */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_map (NCH, KPM, MD))
      {                                 /* tests for existing node           */
         if (p_map = chk_map_node (NCH, MD, MAL))
            free_dist (p_map);
         else
            p_map = add_map_node (NCH, MD, MAL);
         alloc_dist (p_map, MD);
         stor_map (p_map, NCH, KPM, MD); /* stores map                       */
      }
   }
   return err_typ;
}

int map_cmarkKd_m (void)                /* map markers on chromosome (m)     */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      if (chk_map (CH, KDM, MD))
      {                                 /* tests for existing node           */
         if (p_map = chk_map_node (CH, MD, MAL))
            free_dist (p_map);
         else
            p_map = add_map_node (CH, MD, MAL);
         alloc_dist (p_map, MD);
         stor_map (p_map, CH, KDM, MD); /* stores map                        */
      }
   }
   return err_typ;
}

int map_cmarkKp_m (void)                /* map markers on chromosome (m)     */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      if (chk_map (CH, KPM, MD))
      {                                 /* tests for existing node           */
         if (p_map = chk_map_node (CH, MD, MAL))
            free_dist (p_map);
         else
            p_map = add_map_node (CH, MD, MAL);
         alloc_dist (p_map, MD);
         stor_map (p_map, CH, KPM, MD); /* stores map                        */
      }
   }
   return err_typ;
}

int map_traitKd_m (void)                /* map traits (m)                    */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_map (NCH, KDM, TD))
      {                                 /* tests for existing node           */
         if (p_map = chk_map_node (NCH, TD, MAL))
            free_dist (p_map);
         else
            p_map = add_map_node (NCH, TD, MAL);
         alloc_dist (p_map, TD);
         stor_map (p_map, NCH, KDM, TD); /* stores map                       */
      }
   }
   return err_typ;
}

int map_ctraitKd_m (void)               /* map traits on chromosomes (m)     */
{
   MAP_PTR p_map;

   if (chk_stat ())
   {
      if (chk_map (CH, KDM, TD))
      {                                 /* tests for existing node           */
         if (p_map = chk_map_node (CH, TD, MAL))
            free_dist (p_map);
         else
            p_map = add_map_node (CH, TD, MAL);
         alloc_dist (p_map, TD);
         stor_map (p_map, CH, KDM, TD); /* stores stores map                 */
      }
   }
   return err_typ;
}

int map_ctr_extKd_f (void)              /* map extrn distance on chrom (f)   */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
      if (chk_vmap (CH, KXDM))
      {
         if (!(p_vmap = chk_vmap_node (CH, FEM))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (CH, FEM);
         else
         {
            if (chk_vext (p_vmap->p_beg, CH)) /* checks for beginning map    */
               free_vdist (p_vmap->p_beg);
            if (chk_vext (p_vmap->p_end, CH)) /* checks for ending map       */
               free_vdist (p_vmap->p_end);
         }
         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();
         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb fcs */
         stor_vdist (p_vmap->p_beg, KXDM, qfunc_doubdec);
         alloc_vdist (p_vmap->p_end);
         stor_vdist (p_vmap->p_end, KXDM, qfunc_doub);
      }
   return err_typ;
}

int map_ctr_extKd_m (void)              /* map extrn distance on chrom (m)   */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
      if (chk_vmap (CH, KXDM))
      {
         if (!(p_vmap = chk_vmap_node (CH, MAL))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (CH, MAL);
         else
         {
            if (chk_vext (p_vmap->p_beg, CH)) /* checks for beginning map    */
               free_vdist (p_vmap->p_beg);
            if (chk_vext (p_vmap->p_end, CH)) /* checks for ending map       */
               free_vdist (p_vmap->p_end);
         }
         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();
         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb fcs */
         stor_vdist (p_vmap->p_beg, KXDM, qfunc_doubdec);
         alloc_vdist (p_vmap->p_end);
         stor_vdist (p_vmap->p_end, KXDM, qfunc_doub);
      }
   return err_typ;
}

int map_ctr_extKd (void)                /* map extrn distances on chrom      */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
      if (chk_vmap (CH, KXDM))
      {
         if (!(p_vmap = chk_vmap_node (CH, AVG))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (CH, AVG);
         else
         {
            if (chk_vext (p_vmap->p_beg, CH)) /* checks for beginning map    */
               free_vdist (p_vmap->p_beg);
            if (chk_vext (p_vmap->p_end, CH)) /* checks for ending map       */
               free_vdist (p_vmap->p_end);
         }
         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();
         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb fcs */
         stor_vdist (p_vmap->p_beg, KXDM, qfunc_doubdec);
         alloc_vdist (p_vmap->p_end);
         stor_vdist (p_vmap->p_end, KXDM, qfunc_doub);
      }
   return err_typ;
}

int map_tr_extKd_f (void)               /* map extrn distances (f)           */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_vmap (NCH, KXDM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, FEM))) /* if no vmap node, allcs */
            p_vmap = add_vmap_node (NCH, FEM);
         else
         {
            if (chk_vext (p_vmap->p_beg, NCH)) /* checks for beginning map   */
               free_vdist (p_vmap->p_beg);
            if (chk_vext (p_vmap->p_end, NCH)) /* checks for ending map      */
               free_vdist (p_vmap->p_end);
         }
         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();
         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb fcs */
         stor_vdist (p_vmap->p_beg, KXDM, qfunc_doubdec);
         alloc_vdist (p_vmap->p_end);
         stor_vdist (p_vmap->p_end, KXDM, qfunc_doub);
      }
   }
   return err_typ;
}

int map_tr_extKd_m (void)               /* map extrn distances (m)           */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_vmap (NCH, KXDM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, MAL))) /* if no vmap node, allcs */
            p_vmap = add_vmap_node (NCH, MAL);
         else
         {
            if (chk_vext (p_vmap->p_beg, NCH)) /* checks for beginning map   */
               free_vdist (p_vmap->p_beg);
            if (chk_vext (p_vmap->p_end, NCH)) /* checks for ending map      */
               free_vdist (p_vmap->p_end);
         }
         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();
         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb fcs */
         stor_vdist (p_vmap->p_beg, KXDM, qfunc_doubdec);
         alloc_vdist (p_vmap->p_end);
         stor_vdist (p_vmap->p_end, KXDM, qfunc_doub);
      }
   }
   return err_typ;
}

int map_tr_extKd (void)                 /* map extrn distances               */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_vmap (NCH, KXDM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, AVG))) /* if no vmap node, allcs */
            p_vmap = add_vmap_node (NCH, AVG);
         else
         {
            if (chk_vext (p_vmap->p_beg, NCH)) /* checks for beginning map   */
               free_vdist (p_vmap->p_beg);
            if (chk_vext (p_vmap->p_end, NCH)) /* checks for ending map      */
               free_vdist (p_vmap->p_end);
         }
         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();
         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb fcs */
         stor_vdist (p_vmap->p_beg, KXDM, qfunc_doubdec);
         alloc_vdist (p_vmap->p_end);
         stor_vdist (p_vmap->p_end, KXDM, qfunc_doub);
      }
   }
   return err_typ;
}

int map_ctr_begKd_f (void)              /* map start distances on chrom (f)  */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
      if (chk_vmap (CH, KBDM))
      {
         if (!(p_vmap = chk_vmap_node (CH, FEM))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (CH, FEM);
         else if (chk_vext (p_vmap->p_beg, CH)) /* checks for beginning map  */
            free_vdist (p_vmap->p_beg);

         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb fcs */
         stor_vdist (p_vmap->p_beg, KBDM, qfunc_doubdec);
      }
   return err_typ;
}

int map_ctr_begKd_m (void)              /* map start distances on chrom (m)  */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
      if (chk_vmap (CH, KBDM))
      {
         if (!(p_vmap = chk_vmap_node (CH, MAL))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (CH, MAL);
         else if (chk_vext (p_vmap->p_beg, CH)) /* checks for beginning map  */
            free_vdist (p_vmap->p_beg);

         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb fcs */
         stor_vdist (p_vmap->p_beg, KBDM, qfunc_doubdec);
      }
   return err_typ;
}

int map_ctr_begKd (void)                /* map start distances on chrom      */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
      if (chk_vmap (CH, KBDM))
      {
         if (!(p_vmap = chk_vmap_node (CH, AVG))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (CH, AVG);
         else if (chk_vext (p_vmap->p_beg, CH)) /* checks for beginning map  */
            free_vdist (p_vmap->p_beg);

         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb fcs */
         stor_vdist (p_vmap->p_beg, KBDM, qfunc_doubdec);
      }
   return err_typ;
}

int map_tr_begKd_f (void)               /* map start distances (f)           */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_vmap (NCH, KBDM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, FEM))) /* if no vmap node, allcs */
            p_vmap = add_vmap_node (NCH, FEM);
         else if (chk_vext (p_vmap->p_beg, NCH)) /* checks for beginning map */
            free_vdist (p_vmap->p_beg);

         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb fcs */
         stor_vdist (p_vmap->p_beg, KBDM, qfunc_doubdec);
      }
   }
   return err_typ;
}

int map_tr_begKd_m (void)               /* map start distances (m)           */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_vmap (NCH, KBDM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, MAL))) /* if no vmap node, allcs */
            p_vmap = add_vmap_node (NCH, MAL);
         else if (chk_vext (p_vmap->p_beg, NCH)) /* checks for beginning map */
            free_vdist (p_vmap->p_beg);

         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb fcs */
         stor_vdist (p_vmap->p_beg, KBDM, qfunc_doubdec);
      }
   }
   return err_typ;
}

int map_tr_begKd (void)                 /* map start distances               */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_vmap (NCH, KBDM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, AVG))) /* if no vmap node, allcs */
            p_vmap = add_vmap_node (NCH, AVG);
         else if (chk_vext (p_vmap->p_beg, NCH)) /* checks for beginning map */
            free_vdist (p_vmap->p_beg);

         if (!p_vmap->p_beg)
            p_vmap->p_beg = alloc_vext ();

         alloc_vdist (p_vmap->p_beg);   /* allocs, stores dists & recomb fcs */
         stor_vdist (p_vmap->p_beg, KBDM, qfunc_doubdec);
      }
   }
   return err_typ;
}

int map_ctr_endKd_f (void)              /* map end distances on chrom (f)    */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
      if (chk_vmap (CH, KEDM))
      {
         if (!(p_vmap = chk_vmap_node (CH, FEM))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (CH, FEM);
         else if (chk_vext (p_vmap->p_end, CH)) /* checks for ending map     */
            free_vdist (p_vmap->p_end);

         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_end);   /* allocs, stores dists & recomb fcs */
         stor_vdist (p_vmap->p_end, KEDM, qfunc_doub);
      }
   return err_typ;
}

int map_ctr_endKd_m (void)              /* map end distances on chrom (m)    */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
      if (chk_vmap (CH, KEDM))
      {
         if (!(p_vmap = chk_vmap_node (CH, MAL))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (CH, MAL);
         else if (chk_vext (p_vmap->p_end, CH)) /* checks for ending map     */
            free_vdist (p_vmap->p_end);

         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_end);   /* allocs, stores dists & recomb fcs */
         stor_vdist (p_vmap->p_end, KEDM, qfunc_doub);
      }
   return err_typ;
}

int map_ctr_endKd (void)                /* map end distances on chrom        */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
      if (chk_vmap (CH, KEDM))
      {
         if (!(p_vmap = chk_vmap_node (CH, AVG))) /* if no vmap node, allocs */
            p_vmap = add_vmap_node (CH, AVG);
         else if (chk_vext (p_vmap->p_end, CH)) /* checks for ending map     */
            free_vdist (p_vmap->p_end);

         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_end);   /* allocs, stores dists & recomb fcs */
         stor_vdist (p_vmap->p_end, KEDM, qfunc_doub);
      }
   return err_typ;
}

int map_tr_endKd_f (void)               /* map end distances (f)             */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_vmap (NCH, KEDM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, FEM))) /* if no vmap node, allcs */
            p_vmap = add_vmap_node (NCH, FEM);
         else if (chk_vext (p_vmap->p_end, NCH)) /* checks for ending map    */
            free_vdist (p_vmap->p_end);

         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_end);   /* allocs, stores dists & recomb fcs */
         stor_vdist (p_vmap->p_end, KEDM, qfunc_doub);
      }
   }
   return err_typ;
}

int map_tr_endKd_m (void)               /* map end distances (m)             */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_vmap (NCH, KEDM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, MAL))) /* if no vmap node, allcs */
            p_vmap = add_vmap_node (NCH, MAL);
         else if (chk_vext (p_vmap->p_end, NCH)) /* checks for ending map    */
            free_vdist (p_vmap->p_end);

         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_end);   /* allocs, stores dists & recomb fcs */
         stor_vdist (p_vmap->p_end, KEDM, qfunc_doub);
      }
   }
   return err_typ;
}

int map_tr_endKd (void)                 /* map end distances                 */
{
   VMAP_PTR p_vmap;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_vmap (NCH, KEDM))
      {
         if (!(p_vmap = chk_vmap_node (NCH, AVG))) /* if no vmap node, allcs */
            p_vmap = add_vmap_node (NCH, AVG);
         else if (chk_vext (p_vmap->p_end, NCH)) /* checks for ending map    */
            free_vdist (p_vmap->p_end);

         if (!p_vmap->p_end)
            p_vmap->p_end = alloc_vext ();

         alloc_vdist (p_vmap->p_end);   /* allocs, stores dists & recomb fcs */
         stor_vdist (p_vmap->p_end, KEDM, qfunc_doub);
      }
   }
   return err_typ;
}

int set_norm_frq (void)                 /* set normalized frequencies        */
{
   if (chk_stat ())
   {
      chk_repeat ();
      norm_frq_spec = Y;
   }
   return err_typ;
}

int set_mark_frq (void)                 /* set marker frequencies            */
{
   LOCUS_PTR p_locus;
   FREQ_PTR p_fnode;
   int   i;

   if (chk_stat ())
   {
      if (chk_locus (NCH, MD))
      {
         p_fnode = alloc_frq ();
         stor_frq (p_fnode);
         for (i = 0; i < nk_par; i++)
         {
            if (p_locus = chk_locus_node (j_par[i], NCH, MD))
               free_freq (p_locus);
            else
               p_locus = alloc_locus_node (j_par[i], NCH, MD);
            stor_locus_data (j_par[i], p_locus, p_fnode, NCH, MD);
         }
         chk_locus_norm ();
      }
   }
   return err_typ;
}

int set_cmark_frq (void)                /* set chrom/marker frequencies      */
{
   LOCUS_PTR p_locus;
   FREQ_PTR p_fnode;
   int   i;

   if (chk_stat ())
   {
      if (chk_locus (CH, MD))
      {
         p_fnode = alloc_frq ();
         stor_frq (p_fnode);
         for (i = 0; i < nk_par; i++)
         {
            if (p_locus = chk_locus_node (j_par[i + 1], CH, MD))
               free_freq (p_locus);
            else
               p_locus = alloc_locus_node (j_par[i + 1], CH, MD);
            stor_locus_data (j_par[i + 1], p_locus, p_fnode, CH, MD);
         }
         chk_locus_norm ();
      }
   }
   return err_typ;
}

int set_trait_frq (void)                /* set trait frequencies             */
{
   LOCUS_PTR p_locus;
   FREQ_PTR p_fnode;
   int   i;

   if (chk_stat ())
   {
      if (chk_locus (NCH, TD))
      {
         p_fnode = alloc_frq ();
         stor_frq (p_fnode);
         for (i = 0; i < nk_par; i++)
         {
            if (p_locus = chk_locus_node (j_par[i], NCH, TD))
               free_freq (p_locus);
            else
               p_locus = alloc_locus_node (j_par[i], NCH, TD);
            stor_locus_data (j_par[i], p_locus, p_fnode, NCH, TD);
         }
         chk_locus_norm ();
      }
   }
   return err_typ;
}

int set_geno_mean (void)                /* set trait genotype means          */
{                                       /* (must be 3 means)                 */
   MEANS_PTR p_means;
   int   i;

   if (chk_stat ())
   {
      if (chk_means ())
         for (i = 0; i < nk_par; i++)
         {
            if (!(p_means = chk_means_node (j_par[i])))
               p_means = alloc_means_node (j_par[i]);
            stor_mean_data (j_par[i], p_means);
         }
   }
   return err_typ;
}

int set_addi_var (void)                 /* set additive variances            */
{
   VARI_PTR p_vari;
   int   i;

   if (chk_stat ())
   {
      if (chk_vari ())
         for (i = 0; i < nk_par; i++)
         {
            if (!(p_vari = chk_vari_node (j_par[i], AV)))
               p_vari = alloc_vari_node (j_par[i], AV);
            stor_vari_data (i, p_vari);
         }
   }
   return err_typ;
}

int set_resi_var (void)                 /* set residual variances            */
{
   VARI_PTR p_vari;
   int   i;

   if (chk_stat ())
   {
      if (chk_vari ())
         for (i = 0; i < nk_par; i++)
         {
            if (!(p_vari = chk_vari_node (j_par[i], RV)))
               p_vari = alloc_vari_node (j_par[i], RV);
            stor_vari_data (i, p_vari);
         }
   }
   return err_typ;
}

int set_mark_seeds (void)               /* set marker seeds                  */
{
   if (chk_stat ())
   {
      chk_repeat ();
      chk_seeds ();
      MarkSeedSet = Y;
      MarkSeedC = u_par[0];
      MarkSeedT = u_par[1];
   }
   return err_typ;
}

int set_trait_seeds (void)              /* set trait seeds                   */
{
   if (chk_stat ())
   {
      chk_repeat ();
      chk_seeds ();
      TraitSeedSet = Y;
      TraitSeedC = u_par[0];
      TraitSeedT = u_par[1];
   }
   return err_typ;
}

int set_sampl_seeds (void)              /* set sampler seeds                 */
{
   if (chk_stat ())
   {
      chk_repeat ();
      chk_seeds ();
      SamplSeedSet = Y;
      SamplSeedC = u_par[0];
      SamplSeedT = u_par[1];
   }
   return err_typ;
}

int set_perm_seeds (void)               /* set permutation seeds             */
{
   if (chk_stat ())
   {
      chk_repeat ();
      chk_seeds ();
      PermSeedSet = Y;
      PermSeedC = u_par[0];
      PermSeedT = u_par[1];
   }
   return err_typ;
}

int out_mark_seeds (void)               /* output marker seeds only          */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (TraitSeedsOut)
      {
         TraitSeedsOut = N;
         err_typ = ERD_OTS;
      }
      MarkSeedsOut = Y;
   }
   return err_typ;
}

int out_trait_seeds (void)              /* output trait seeds only           */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (MarkSeedsOut)
      {
         MarkSeedsOut = N;
         err_typ = ERD_OMS;
      }
      TraitSeedsOut = Y;
   }
   return err_typ;
}

int out_samp_seeds (void)               /* output sampler seeds only         */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (PermSeedsOut)
      {
         PermSeedsOut = N;
         err_typ = ERA_OPS;
      }
      SamplSeedsOut = Y;
   }
   return err_typ;
}

int out_perm_seeds (void)               /* output permutation seeds only     */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (SamplSeedsOut)
      {
         SamplSeedsOut = N;
         err_typ = ERA_OSS;
      }
      PermSeedsOut = Y;
   }
   return err_typ;
}

int out_unobs (void)                    /* output unobserved variables       */
{
   if (chk_stat ())
   {
      chk_repeat ();
      ObsOut = Y;
   }
   return err_typ;
}

int set_mark_dat (void)                 /* set marker data                   */
{
   MARKDAT_PTR p_mdnode;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_mrkdat (NCH))
      {                                 /* tests for existing node            */
         if (!(p_mdnode = chk_mrkdat_node (NCH)))
            p_mdnode = alloc_mrkdat_node (NCH);
         else
            free_mrkdat_list (p_mdnode);
         alloc_mrkdat_list (p_mdnode);
         stor_mrkdat (p_mdnode, NCH);   /* stores data in node and in list    */
      }
   }
   return err_typ;
}

int set_cmark_dat (void)                /* set marker data for chromosome    */
{
   MARKDAT_PTR p_mdnode;

   if (chk_stat ())
      if (chk_mrkdat (CH))
      {                                 /* tests for existing node            */
         if (!(p_mdnode = chk_mrkdat_node (CH)))
            p_mdnode = alloc_mrkdat_node (CH);
         else
            free_mrkdat_list (p_mdnode);
         alloc_mrkdat_list (p_mdnode);
         stor_mrkdat (p_mdnode, CH);    /*stores data in node and in list     */
      }
   return err_typ;
}

int set_cmark_nams (void)               /* set marker names for chromosome   */
{
   MARKNAM_PTR p_mnnode;

   if (chk_stat ())
      if (chk_mrknam (CH))
      {                                 /* tests for existing node            */
         if (!(p_mnnode = chk_mrknam_node (CH)))
            p_mnnode = alloc_mrknam_node (CH);
         else
            free_mrknam_list (p_mnnode);
         alloc_mrknam_list (p_mnnode);
         stor_mrknam (p_mnnode, CH);    /*stores names in node and in list    */
      }
   return err_typ;
}

int set_mark_nams (void)                /* set marker names for chromosome    */
{
   MARKNAM_PTR p_mnnode;

   if (chk_stat ())
      if (chk_mrknam (NCH))
      {                                 /* tests for existing node            */
         if (!(p_mnnode = chk_mrknam_node (NCH)))
            p_mnnode = alloc_mrknam_node (NCH);
         else
            free_mrknam_list (p_mnnode);
         alloc_mrknam_list (p_mnnode);
         stor_mrknam (p_mnnode, NCH);   /*stores names in node and in list    */
      }
   return err_typ;
}

int set_prbgam (void)                   /* set proband gametes                */
{
   PROBGAM_PTR p_pgnode;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_prbgam (NCMP, NSC))
      {
         if (!(p_pgnode = chk_prbgam_node (NCMP, NSC)))
            p_pgnode = alloc_prbgam_node (NCMP, NSC);
         else
            free_prbgam_list (p_pgnode);
         alloc_prbgam_list (p_pgnode);
         stor_prbgam (p_pgnode, NCMP, NSC);
      }
   }
   return err_typ;
}

int set_cp_prbgam (void)                /* set component proband gametes     */
{
   PROBGAM_PTR p_pgnode;

   if (chk_stat ())
      if (chk_prbgam (CMP, NSC))
      {
         if (!(p_pgnode = chk_prbgam_node (CMP, NSC)))
            p_pgnode = alloc_prbgam_node (CMP, NSC);
         else
            free_prbgam_list (p_pgnode);
         alloc_prbgam_list (p_pgnode);
         stor_prbgam (p_pgnode, CMP, NSC);
      }
   return err_typ;
}

int set_sc_prbgam (void)                /* set scoreset proband gametes   */
{
   PROBGAM_PTR p_pgnode;

   if (chk_stat ())
      if (chk_prbgam (NCMP, SC))
      {
         if (!(p_pgnode = chk_prbgam_node (NCMP, SC)))
            p_pgnode = alloc_prbgam_node (NCMP, SC);
         else
            free_prbgam_list (p_pgnode);
         alloc_prbgam_list (p_pgnode);
         stor_prbgam (p_pgnode, NCMP, SC);
      }
   return err_typ;
}

int set_cs_prbgam (void)                /* set component scoreset proband */
{                                       /* gametes                            */
   PROBGAM_PTR p_pgnode;

   if (chk_stat ())
      if (chk_prbgam (CMP, SC))
      {
         if (!(p_pgnode = chk_prbgam_node (CMP, SC)))
            p_pgnode = alloc_prbgam_node (CMP, SC);
         else
            free_prbgam_list (p_pgnode);
         alloc_prbgam_list (p_pgnode);
         stor_prbgam (p_pgnode, CMP, SC);
      }
   return err_typ;
}

int set_affind_tr (void)                /* set affected individuals trait     */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_int_trait ())
         AffTrIn = j_par[0];
   }
   return err_typ;
}

int set_cp_affnam (void)                /* set component affected indivds */
{
   INDNAM_PTR p_affindnode;

   if (chk_stat ())
      if (chk_indnam (CMP))
      {
         if (!(p_affindnode = chk_indnam_node (CMP, AFFIND)))
            p_affindnode = alloc_indnam_node (CMP, AFFIND);
         else
            free_indnam_list (p_affindnode);
         alloc_indnam_list (p_affindnode);
         stor_indnam (p_affindnode, CMP);
      }
   return err_typ;
}

int set_affnam (void)                   /* set affected individuals       */
{
   INDNAM_PTR p_affindnode;

   if (chk_stat ())
      if (chk_indnam (NCMP))
      {
         if (!(p_affindnode = chk_indnam_node (NCMP, AFFIND)))
            p_affindnode = alloc_indnam_node (NCMP, AFFIND);
         else
            free_indnam_list (p_affindnode);
         alloc_indnam_list (p_affindnode);
         stor_indnam (p_affindnode, NCMP);
      }
   return err_typ;
}

int set_loc_wind (void)                 /* set locus window               */
{
   LOCWND_PTR p_wnd;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_wind (NCH))
      {                                 /* checks for node present            */
         if (!(p_wnd = chk_wind_node (NCH)))
            p_wnd = alloc_wind_node (NCH);
         stor_wind (p_wnd, NCH);        /* saves window size                  */
      }
   }
   return err_typ;
}

int set_cloc_wind (void)                /* set chromosome locus window    */
{
   LOCWND_PTR p_wnd;

   if (chk_stat ())
      if (chk_wind (CH))
      {                                 /* checks for node present for chrom  */
         if (!(p_wnd = chk_wind_node (CH)))
            p_wnd = alloc_wind_node (CH);
         stor_wind (p_wnd, CH);         /* saves window size                     */
      }
   return err_typ;
}

int set_windpat (void)                  /* set window patterns            */
{
   WNDPAT_PTR p_wpnode;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_wndpat (NCMP, NSC))
      {
         if (!(p_wpnode = chk_wndpat_node (NCMP, NSC)))
            p_wpnode = alloc_wndpat_node (NCMP, NSC);
         else
            free_wndpat_set (p_wpnode);
         alloc_wndpat_set (p_wpnode);
         stor_wndpat (p_wpnode, NCMP, NSC);
      }
   }
   return err_typ;
}

int set_cp_windpat (void)               /* set component window patterns  */
{
   WNDPAT_PTR p_wpnode;

   if (chk_stat ())
      if (chk_wndpat (CMP, NSC))
      {
         if (!(p_wpnode = chk_wndpat_node (CMP, NSC)))
            p_wpnode = alloc_wndpat_node (CMP, NSC);
         else
            free_wndpat_set (p_wpnode);
         alloc_wndpat_set (p_wpnode);
         stor_wndpat (p_wpnode, CMP, NSC);
      }
   return err_typ;
}

int set_sc_windpat (void)               /* set scoreset window patterns   */
{
   WNDPAT_PTR p_wpnode;

   if (chk_stat ())
      if (chk_wndpat (NCMP, SC))
      {
         if (!(p_wpnode = chk_wndpat_node (NCMP, SC)))
            p_wpnode = alloc_wndpat_node (NCMP, SC);
         else
            free_wndpat_set (p_wpnode);
         alloc_wndpat_set (p_wpnode);
         stor_wndpat (p_wpnode, NCMP, SC);
      }
   return err_typ;
}

int set_cs_windpat (void)               /* set component scoreset         */
{                                       /* window patterns                    */
   WNDPAT_PTR p_wpnode;

   if (chk_stat ())
      if (chk_wndpat (CMP, SC))
      {
         if (!(p_wpnode = chk_wndpat_node (CMP, SC)))
            p_wpnode = alloc_wndpat_node (CMP, SC);
         else
            free_wndpat_set (p_wpnode);
         alloc_wndpat_set (p_wpnode);
         stor_wndpat (p_wpnode, CMP, SC);
      }
   return err_typ;
}

int set_incmplt_pen (void)              /* set incomplete penetrances          */
{
   int   i;

   if (chk_stat ())
   {
      chk_repeat ();
      if (nx_par != 3)
         err_typ = ERA_NPV;
      else
      {
         chk_penetr ();
         NPenVals = 3;
         for (i = 0; i < 3; ++i)
            PenVal[i] = d_par[i];
      }
   }
   return err_typ;
}

int cmpu_kinshp (void)                  /* compute kinship coeff         */
{
   KINS *p_kinshp;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_kinshp (NCMP, 2))
      {
         if (!(p_kinshp = chk_kinshp_node (NCMP, KINSHP)))
            p_kinshp = alloc_kinshp_node (NCMP, KINSHP);
         else
            free_nams_list (p_kinshp);
         alloc_nams_list (p_kinshp);
         stor_kinshp (p_kinshp, NCMP, 2);
      }
   }
   return err_typ;
}

int cmpu_inbreed (void)                 /* compute inbreeding coeff      */
{
   KINS *p_inbr;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_kinshp (NCMP, 1))
      {
         if (!(p_inbr = chk_kinshp_node (NCMP, INBRD)))
            p_inbr = alloc_kinshp_node (NCMP, INBRD);
         else
            free_nams_list (p_inbr);
         alloc_nams_list (p_inbr);
         stor_kinshp (p_inbr, NCMP, 1);
      }
   }
   return err_typ;
}

int cmpu_2_loc (void)                   /* compute 2-locus inbreed coeff */
{
   KINS *p_inbr2;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_kinshp (NCMP, 1))
      {
         if (!(p_inbr2 = chk_kinshp_node (NCMP, INBRD2)))
            p_inbr2 = alloc_kinshp_node (NCMP, INBRD2);
         else
            free_nams_list (p_inbr2);
         alloc_nams_list (p_inbr2);
         stor_kinshp (p_inbr2, NCMP, 1);
      }
   }
   return err_typ;
}

int cmpu_ckinshp (void)                 /* compute kinship coeff         */
{
   KINS *p_kinshp;

   if (chk_stat ())
      if (chk_kinshp (CMP, 2))
      {
         if (!(p_kinshp = chk_kinshp_node (CMP, KINSHP)))
            p_kinshp = alloc_kinshp_node (CMP, KINSHP);
         else
            free_nams_list (p_kinshp);
         alloc_nams_list (p_kinshp);
         stor_kinshp (p_kinshp, CMP, 2);
      }
   return err_typ;
}

int cmpu_cinbreed (void)                /* compute inbreeding coeff      */
{
   KINS *p_inbr;

   if (chk_stat ())
      if (chk_kinshp (CMP, 1))
      {
         if (!(p_inbr = chk_kinshp_node (CMP, INBRD)))
            p_inbr = alloc_kinshp_node (CMP, INBRD);
         else
            free_nams_list (p_inbr);
         alloc_nams_list (p_inbr);
         stor_kinshp (p_inbr, CMP, 1);
      }
   return err_typ;
}

int cmpu_c2_loc (void)                  /* compute 2-locus inbreed coeff */
{
   KINS *p_inbr2;

   if (chk_stat ())
      if (chk_kinshp (CMP, 1))
      {
         if (!(p_inbr2 = chk_kinshp_node (CMP, INBRD2)))
            p_inbr2 = alloc_kinshp_node (CMP, INBRD2);
         else
            free_nams_list (p_inbr2);
         alloc_nams_list (p_inbr2);
         stor_kinshp (p_inbr2, CMP, 1);
      }
   return err_typ;
}

int set_reco_frq (void)                 /* set recombination frequencies */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_reco_frq ())
      {
         if (!NInbreedRF)
            InbreedRF = alloc_reco_frq ();
         else
            free_reco_frq (&InbreedRF);

         stor_reco_frq (&NInbreedRF, InbreedRF);
      }
   }
   return err_typ;
}

int cmpu_est_n (void)                   /* compute estimates I times         */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (j_par[0] < 1)
         err_typ = ERA_CET;
      Redo = j_par[0];
   }
   return err_typ;
}

int lim_rf (void)                       /* limit recombination fraction      */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (d_par[0] <= 0. || d_par[0] >= .1)
         err_typ = ERA_LBR;
      RF_bound = d_par[0];
   }
   return err_typ;
}

int set_ascent_iter (void)              /* set ascent iterations             */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (j_par[0] < 1)
         err_typ = ERA_NAI;
      NAscentIt = j_par[0];
   }
   return err_typ;
}

int set_curv_iter (void)                /* set curvature iterations          */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (j_par[0] < 1)
         err_typ = ERA_NCI;
      NCurvIt = j_par[0];
   }
   return err_typ;
}

int set_grad_iter (void)                /* set gradient iterations           */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (j_par[0] < 1)
         err_typ = ERA_NGI;
      NGradIt = j_par[0];
   }
   return err_typ;
}

int set_lrt_iter (void)                 /* set LRT statistics iterations     */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (j_par[0] < 1)
         err_typ = ERA_NRI;
      NChiSqIt = j_par[0];
   }
   return err_typ;
}

int set_em_conv (void)                  /* set EM convergence                */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (d_par[0] <= 0. || d_par[0] >= 1.)
         err_typ = ERA_PCE;
      EM_bound = d_par[0];
   }
   return err_typ;
}

int set_sa_conv (void)                  /* set SA convergence                */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (d_par[0] <= 0. || d_par[0] >= 1.)
         err_typ = ERA_PCS;
      SA_bound = d_par[0];
   }
   return err_typ;
}

int out_maps_avg (void)                 /* output maps gender averaged       */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (MapTyp == SEX_SPEC || MapTyp == BOTH)
         err_typ = ERA_MTR;
      MapTyp = SEX_AVG;
   }
   return err_typ;
}

int out_maps_spc (void)                 /* output maps gender specific       */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (MapTyp == SEX_AVG || MapTyp == BOTH)
         err_typ = ERA_MTR;
      MapTyp = SEX_SPEC;
   }
   return err_typ;
}

int out_maps_both (void)                /* output maps both                  */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (MapTyp == SEX_SPEC || MapTyp == SEX_AVG)
         err_typ = ERA_MTR;
      MapTyp = BOTH;
   }
   return err_typ;
}

int cmpu_ibdstat (void)                 /* compute ibd statistics             */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (IBDStats != UNDEF || LikRatStats != UNDEF)
         err_typ = ERA_RST;
      IBDStats = 1;
      LikRatStats = 0;
   }
   return err_typ;
}

int cmpu_likstat (void)                 /* compute likelihood-ratio stats     */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (IBDStats != UNDEF || LikRatStats != UNDEF)
         err_typ = ERA_RST;
      IBDStats = 0;
      LikRatStats = 1;
   }
   return err_typ;
}

int set_ibd_meas1 (void)                /* set ibd measures Spairs            */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (SpairsOn != UNDEF || RobdomOn != UNDEF || SAffUnaffOn != UNDEF
          || LambdaOn != UNDEF || MaxFGLOn != UNDEF)
         err_typ = ERA_RIS;
      SpairsOn = 1;
      RobdomOn = 0;
      SAffUnaffOn = 0;
      LambdaOn = 0;
      MaxFGLOn = 0;
   }
   return err_typ;
}

int set_ibd_meas12 (void)               /* set ibd meas Spairs Srobdom        */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (SpairsOn != UNDEF || RobdomOn != UNDEF || SAffUnaffOn != UNDEF
          || LambdaOn != UNDEF || MaxFGLOn != UNDEF)
         err_typ = ERA_RIS;
      SpairsOn = 1;
      RobdomOn = 1;
      SAffUnaffOn = 0;
      LambdaOn = 0;
      MaxFGLOn = 0;
   }
   return err_typ;
}

int set_ibd_meas123 (void)              /* set ibd meas Spairs Srobdom Saff   */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (SpairsOn != UNDEF || RobdomOn != UNDEF || SAffUnaffOn != UNDEF
          || LambdaOn != UNDEF || MaxFGLOn != UNDEF)
         err_typ = ERA_RIS;
      SpairsOn = 1;
      RobdomOn = 1;
      SAffUnaffOn = 1;
      LambdaOn = 0;
      MaxFGLOn = 0;
   }
   return err_typ;
}

int set_ibd_measall (void)              /* set ibd meas Sp Sr Sa Sl           */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (SpairsOn != UNDEF || RobdomOn != UNDEF || SAffUnaffOn != UNDEF
          || LambdaOn != UNDEF || MaxFGLOn != UNDEF)
         err_typ = ERA_RIS;
      SpairsOn = 1;
      RobdomOn = 1;
      SAffUnaffOn = 1;
      LambdaOn = 1;
      MaxFGLOn = 0;
   }
   return err_typ;
}

int set_ibd_meas124 (void)              /* set ibd meas Spairs Srobdom Slamb  */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (SpairsOn != UNDEF || RobdomOn != UNDEF || SAffUnaffOn != UNDEF
          || LambdaOn != UNDEF || MaxFGLOn != UNDEF)
         err_typ = ERA_RIS;
      SpairsOn = 1;
      RobdomOn = 1;
      SAffUnaffOn = 0;
      LambdaOn = 1;
      MaxFGLOn = 0;
   }
   return err_typ;
}

int set_ibd_meas13 (void)               /* set ibd meas Spairs Saffect        */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (SpairsOn != UNDEF || RobdomOn != UNDEF || SAffUnaffOn != UNDEF
          || LambdaOn != UNDEF || MaxFGLOn != UNDEF)
         err_typ = ERA_RIS;
      SpairsOn = 1;
      RobdomOn = 0;
      SAffUnaffOn = 1;
      LambdaOn = 0;
      MaxFGLOn = 0;
   }
   return err_typ;
}

int set_ibd_meas134 (void)              /* set ibd meas Spairs Saffect Slamb  */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (SpairsOn != UNDEF || RobdomOn != UNDEF || SAffUnaffOn != UNDEF
          || LambdaOn != UNDEF || MaxFGLOn != UNDEF)
         err_typ = ERA_RIS;
      SpairsOn = 1;
      RobdomOn = 0;
      SAffUnaffOn = 1;
      LambdaOn = 1;
      MaxFGLOn = 0;
   }
   return err_typ;
}

int set_ibd_meas14 (void)               /* set ibd meas Spairs Slambda        */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (SpairsOn != UNDEF || RobdomOn != UNDEF || SAffUnaffOn != UNDEF
          || LambdaOn != UNDEF || MaxFGLOn != UNDEF)
         err_typ = ERA_RIS;
      SpairsOn = 1;
      RobdomOn = 0;
      SAffUnaffOn = 0;
      LambdaOn = 1;
      MaxFGLOn = 0;
   }
   return err_typ;
}

int set_ibd_meas2 (void)                /* set ibd meas Srobdom               */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (SpairsOn != UNDEF || RobdomOn != UNDEF || SAffUnaffOn != UNDEF
          || LambdaOn != UNDEF || MaxFGLOn != UNDEF)
         err_typ = ERA_RIS;
      SpairsOn = 0;
      RobdomOn = 1;
      SAffUnaffOn = 0;
      LambdaOn = 0;
      MaxFGLOn = 0;
   }
   return err_typ;
}

int set_ibd_meas23 (void)               /* set ibd meas Srobdom Saffect       */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (SpairsOn != UNDEF || RobdomOn != UNDEF || SAffUnaffOn != UNDEF
          || LambdaOn != UNDEF || MaxFGLOn != UNDEF)
         err_typ = ERA_RIS;
      SpairsOn = 0;
      RobdomOn = 1;
      SAffUnaffOn = 1;
      LambdaOn = 0;
      MaxFGLOn = 0;
   }
   return err_typ;
}

int set_ibd_meas234 (void)              /* set ibd meas Srobdom Saff Slambda  */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (SpairsOn != UNDEF || RobdomOn != UNDEF || SAffUnaffOn != UNDEF
          || LambdaOn != UNDEF || MaxFGLOn != UNDEF)
         err_typ = ERA_RIS;
      SpairsOn = 0;
      RobdomOn = 1;
      SAffUnaffOn = 1;
      LambdaOn = 1;
      MaxFGLOn = 0;
   }
   return err_typ;
}

int set_ibd_meas24 (void)               /* set ibd meas Srobdom Slambda       */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (SpairsOn != UNDEF || RobdomOn != UNDEF || SAffUnaffOn != UNDEF
          || LambdaOn != UNDEF || MaxFGLOn != UNDEF)
         err_typ = ERA_RIS;
      SpairsOn = 0;
      RobdomOn = 1;
      SAffUnaffOn = 0;
      LambdaOn = 1;
      MaxFGLOn = 0;
   }
   return err_typ;
}

int set_ibd_meas3 (void)                /* set ibd meas Saffect               */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (SpairsOn != UNDEF || RobdomOn != UNDEF || SAffUnaffOn != UNDEF
          || LambdaOn != UNDEF || MaxFGLOn != UNDEF)
         err_typ = ERA_RIS;
      SpairsOn = 0;
      RobdomOn = 0;
      SAffUnaffOn = 1;
      LambdaOn = 0;
      MaxFGLOn = 0;
   }
   return err_typ;
}

int set_ibd_meas34 (void)               /* set ibd meas Saffect Slambda       */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (SpairsOn != UNDEF || RobdomOn != UNDEF || SAffUnaffOn != UNDEF
          || LambdaOn != UNDEF || MaxFGLOn != UNDEF)
         err_typ = ERA_RIS;
      SpairsOn = 0;
      RobdomOn = 0;
      SAffUnaffOn = 1;
      LambdaOn = 1;
      MaxFGLOn = 0;
   }
   return err_typ;
}

int set_ibd_meas4 (void)                /* set ibd meas Slambda               */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (SpairsOn != UNDEF || RobdomOn != UNDEF || SAffUnaffOn != UNDEF
          || LambdaOn != UNDEF || MaxFGLOn != UNDEF)
         err_typ = ERA_RIS;
      SpairsOn = 0;
      RobdomOn = 0;
      SAffUnaffOn = 0;
      LambdaOn = 1;
      MaxFGLOn = 0;
   }
   return err_typ;
}

int set_ibd_meas5 (void)                /* set ibd meas maxFGL                */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (SpairsOn != UNDEF || RobdomOn != UNDEF || SAffUnaffOn != UNDEF
          || LambdaOn != UNDEF || MaxFGLOn != UNDEF)
         err_typ = ERA_RIS;
      SpairsOn = 0;
      RobdomOn = 0;
      SAffUnaffOn = 0;
      LambdaOn = 0;
      MaxFGLOn = 1;
   }
   return err_typ;
}

int set_ibd_meas51 (void)               /* set ibd meas maxFGL Spairs         */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (SpairsOn != UNDEF || RobdomOn != UNDEF || SAffUnaffOn != UNDEF
          || LambdaOn != UNDEF || MaxFGLOn != UNDEF)
         err_typ = ERA_RIS;
      SpairsOn = 1;
      RobdomOn = 0;
      SAffUnaffOn = 0;
      LambdaOn = 0;
      MaxFGLOn = 1;
   }
   return err_typ;
}

int set_ibd_meas512 (void)              /* set ibd meas maxFGL Spairs Srobdom */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (SpairsOn != UNDEF || RobdomOn != UNDEF || SAffUnaffOn != UNDEF
          || LambdaOn != UNDEF || MaxFGLOn != UNDEF)
         err_typ = ERA_RIS;
      SpairsOn = 1;
      RobdomOn = 1;
      SAffUnaffOn = 0;
      LambdaOn = 0;
      MaxFGLOn = 1;
   }
   return err_typ;
}

int set_ibd_meas52 (void)               /* set ibd meas  maxFGL Srobdom       */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (SpairsOn != UNDEF || RobdomOn != UNDEF || SAffUnaffOn != UNDEF
          || LambdaOn != UNDEF || MaxFGLOn != UNDEF)
         err_typ = ERA_RIS;
      SpairsOn = 0;
      RobdomOn = 1;
      SAffUnaffOn = 0;
      LambdaOn = 0;
      MaxFGLOn = 1;
   }
   return err_typ;
}

int set_like_meas6 (void)               /* set like measures delta            */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (DeltaOn != UNDEF || Lambda_pOn != UNDEF)
         err_typ = ERA_RLS;
      DeltaOn = 1;
      Lambda_pOn = 0;
   }
   return err_typ;
}

int set_like_meas67 (void)              /* set like measiures delta lambda-p   */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (DeltaOn != UNDEF || Lambda_pOn != UNDEF)
         err_typ = ERA_RLS;
      DeltaOn = 1;
      Lambda_pOn = 1;
   }
   return err_typ;
}

int set_like_meas7 (void)               /* set likelihood measures lambda-p   */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (DeltaOn != UNDEF || Lambda_pOn != UNDEF)
         err_typ = ERA_RLS;
      DeltaOn = 0;
      Lambda_pOn = 1;
   }
   return err_typ;
}

int set_ibd_test1 (void)                /* set ibd tests normal              */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (NormalEnable != UNDEF || PermutEnable != UNDEF)
         err_typ = ERA_RIT;
      NormalEnable = 1;
      PermutEnable = 0;
   }
   return err_typ;
}

int set_ibd_test12 (void)               /* set ibd tests normal              */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (NormalEnable != UNDEF || PermutEnable != UNDEF)
         err_typ = ERA_RIT;
      NormalEnable = 1;
      PermutEnable = 1;
   }
   return err_typ;
}

int set_ibd_test2 (void)                /* set ibd tests permutation         */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (NormalEnable != UNDEF || PermutEnable != UNDEF)
         err_typ = ERA_RIT;
      NormalEnable = 0;
      PermutEnable = 1;
   }
   return err_typ;
}

int set_like_test3 (void)               /* set likehd-rat tests chi-squared  */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (ChiSqEnable != UNDEF)
         err_typ = ERA_RLT;
      ChiSqEnable = 1;
   }
   return err_typ;
}

int set_ibd_perm (void)                 /* set ibd permutations              */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (j_par[0] < 1)
         err_typ = ERA_SPI;
      else if (j_par[0] < 50)
         err_typ = ERA_RPI;

      NPermutIt = j_par[0];
   }
   return err_typ;
}

int set_like_grdpts (void)              /* set likehd-rat lambda-p gridpoints */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (j_par[0] < 1 || j_par[1] < 1)
         err_typ = ERA_BGP;

      NumOfSegL = j_par[0];
      NumOfSegP = j_par[1];
   }
   return err_typ;
}

int out_found_labs (void)               /* output founder labels             */
{
   if (chk_stat ())
      chk_repeat ();
   LabelsOut = Y;

   return err_typ;
}

int out_tr_vars (void)                  /* output latent trait variables     */
{
   if (chk_stat ())
      chk_repeat ();
   LatentOut = Y;

   return err_typ;
}

int out_chron (void)                    /* output pedigree chronological     */
{
   if (chk_stat ())
      chk_repeat ();
   ChronOut = Y;

   return err_typ;
}

int disp_deflt (void)                   /* display (default)                 */
{
   if (chk_stat ())
      chk_repeat ();
   GDispOpt = DEFLT_DISP;

   return err_typ;
}

int disp_segrg (void)                   /* display segregations              */
{
   if (chk_stat ())
      chk_repeat ();
   GDispOpt = SEGR_DISP;

   return err_typ;
}

int disp_labls (void)                   /* display labels                    */
{
   if (chk_stat ())
      chk_repeat ();
   GDispOpt = LABL_DISP;

   return err_typ;
}

int disp_iter (void)                    /* display every I iterations         */
{
   if (chk_stat ())
   {
      chk_repeat ();
      GDispIt = j_par[0];
   }
   return err_typ;
}

int disp_cp_indvs (void)                /* display component individuals     */
{
   INDNAM_PTR p_dispnode;

   if (chk_stat ())
      if (chk_indnam (CMP))
      {
         if (!(p_dispnode = chk_indnam_node (CMP, DISPIND)))
            p_dispnode = alloc_indnam_node (CMP, DISPIND);
         else
            free_indnam_list (p_dispnode);
         alloc_indnam_list (p_dispnode);
         stor_indnam (p_dispnode, CMP);
      }
   return err_typ;
}

int disp_indvs (void)                   /* display individuals */
{
   INDNAM_PTR p_dispnode;

   if (chk_stat ())
      if (chk_indnam (NCMP))
      {
         if (!(p_dispnode = chk_indnam_node (NCMP, DISPIND)))
            p_dispnode = alloc_indnam_node (NCMP, DISPIND);
         else
            free_indnam_list (p_dispnode);
         alloc_indnam_list (p_dispnode);
         stor_indnam (p_dispnode, NCMP);
      }
   return err_typ;
}

int set_debugmain (void)                /* set debug flag for main program   */
{
   if (chk_stat ())
      chk_repeat ();
   DebugMain = Y;

   return err_typ;
}

int set_debugcmf (void)                 /* set debug flag for CMF library    */
{
   if (chk_stat ())
      chk_repeat ();
   DebugCMF = Y;

   return err_typ;
}

int set_debuggldisp (void)              /* set debug flag for GLDisp library */
{
   if (chk_stat ())
      chk_repeat ();
   DebugGLDisp = Y;

   return err_typ;
}

int set_debugmarkrs (void)              /* set debug flag for Markers libr   */
{
   if (chk_stat ())
      chk_repeat ();
   DebugMarkers = Y;

   return err_typ;
}

int set_debugnghds (void)               /* set debug flag for Nghds library  */
{
   if (chk_stat ())
      chk_repeat ();
   DebugNghds = Y;

   return err_typ;
}

int set_debugpars (void)                /* set debug flag for Pars library   */
{
   if (chk_stat ())
      chk_repeat ();
   DebugPars = Y;

   return err_typ;
}

int set_debugpedchk (void)              /* set debug flag for Pedchk library */
{
   if (chk_stat ())
      chk_repeat ();
   DebugPedchk = Y;

   return err_typ;
}

int set_debugpeel (void)                /* set debug flag for Peel library   */
{
   if (chk_stat ())
      chk_repeat ();
   DebugPeel = Y;

   return err_typ;
}

int set_debugquant (void)               /* set debug flag for Quant library  */
{
   if (chk_stat ())
      chk_repeat ();
   DebugQuant = Y;

   return err_typ;
}

int set_debugrans (void)                /* set debug flag for Rans library   */
{
   if (chk_stat ())
      chk_repeat ();
   DebugQuant = Y;

   return err_typ;
}

int set_debugsample (void)              /* set debug flag for Sample library */
{
   if (chk_stat ())
      chk_repeat ();
   DebugSample = Y;

   return err_typ;
}

int set_debugstuff (void)               /* set debug flag for Stuff library  */
{
   if (chk_stat ())
      chk_repeat ();
   DebugStuff = Y;

   return err_typ;
}

/* -------------- end of functions for parameter statements ---------------- */
#include "proc_parf.c"
#include "proc_err.c"
#include "proc_kin.c"
#include "proc_drop.c"
#include "proc_auto.c"
#include "proc_poly.c"
#include "proc_fils.c"
#include "proc_end.c"
#include "proc_seeds.c"
#include "proc_disp.c"
#include "check_pars.c"
#include "input_eigen.c"
#include "proc_free.c"
