/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */

void  setupPoly (int, char **, char *);

static double **quant = NULL;

/**
 * Calls functions to do the setup steps for the Morgan programs
 * which use a Monte Carlo method.
 * This includes setting up parameters and data structures for the
 * marker and trait model,
 * reading the pedigree file and setting up the pedigree structure,
 * storing observed marker and trait data for pedigree members.
 */
void setupPoly (int argc, char **argv, char *ppn)
{
   /* initialize exit procedure, print headings. 
    * fncs:  atexit(), print_head() 
    * dirs:  Stuff 
    */
   initMorgProg (argv);

   /* process parameter statements, set up globals resulting from these.
    * fncs:  proc_pars(), prntStatusPP(), proc_poly(), proc_fils(), 
    *        proc_seeds(), prntStatusCP() 
    * vars:  globals in globs.h
    * dirs:  Pars, Stuff
    */
   procParStat (argc, argv, ppn);

   /* check, and massage if necessary, pedigree file.
    * fncs:  chk_pedfil(), prntStatusCPF(), mak_opedfil()
    * vars:  indKeys[PedSize], compntList[nCompnt]
    * dirs:  PedComp, Stuff
    */
   chkPedFil ();

   /* read in pedigree data and set up pedigree structure.
    * fncs:  alloc_ped(), open_ped(), input_ped(), nghds(), count_founders()
    * vars:  int data[PedSize][NColData], Ind nghd[PedSize+1]
    *        double quant[NTraits][PedSize], Ind nghd[].q_dat[2*(NMarks+1)]
    * dirs:  Nghds, Quant, Stuff
    */
   quant = allocQPed ();
   setPed (quant);

   setQuantEff (quant);
   free_qdat (quant);

   return;
}
