/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */
/*****************************************************************************
  routine name: sampling.c

  Take out the MCMC sampling portion of lm_auto program, 
  to separate from scoring of results, and accommodate burn-in etc.
                                                                   EAT Sep 2002

  set up permutation arrays for loci and switches moved from main  YJS Jan 2005

  free_perm() added
******************************************************************************/

#include "lsample.h"
#include "msample.h"

static int *perm_sw = NULL;             /* permutation arrays of switches */
static int *perm_loc = NULL;            /* and loci */

void sampling (int t, int mm, int nloc, int nsw, int ngenes, double **prob)
/* mm=0 if L-sample samples all loci, mm=1 if markers only  */
/*      now also for M-sampler provided chromo[0]==0  EAT  3/2005  */
{
   int   k, l, i, jj, sx, c;
   static int allocated = 0;

   /*** set up permutation arrays for loci and switches ***/
   /*** (moved from main                 YJS Jan 2005)  ***/

   if (Scan && (nsw != allocated))
   {
      perm_sw = realloc (perm_sw, nsw * sizeof (int));
      alloc_chk (WHERE, perm_sw);
      for (i = 0; i < nsw; ++i)
         perm_sw[i] = i;

      perm_loc = realloc (perm_loc, (nloc + 1) * sizeof (int));
      alloc_chk (WHERE, perm_loc);
      for (i = 0; i <= nloc; ++i)
         perm_loc[i] = i;

      allocated = nsw;
   }

   if (rans () < LSamplProb)
   {                                    /* ***** Do L-sample *****  */
      if (!((t + 1) % ChkIt))
      {
         printf (" Starting L-sampler iteration %d\n", t + 1);
         fflush (stdout);
      }
      if (Scan)                         /* ****   scan   **** */
      {
         permute (perm_loc, nloc + 1);  /* permute loci for L-sampler */
         for (k = 0; k <= nloc; ++k)
         {                              /* do l-sampling for ea. locus */
            jj = perm_loc[k];
            if (jj >= mm)               /* if mm=1, don't do jj=0  */
               (void) do_lsmpl (nloc, jj, prob, ngenes);
         }
      }
      else
      {                                 /* ****   step   **** */

         jj = intran (mm, nloc);        /* choose a random locus
                                         * do l-sampling for this locus */
         (void) do_lsmpl (nloc, jj, prob, ngenes);

      }
   }
   else
   {                                    /* ***** Do M-sample *****  */
      if (!((t + 1) % ChkIt))
      {
         printf (" Starting M-sampler iteration %d\n", t + 1);
         fflush (stdout);
      }
      if (Scan)                         /* ****   scan   **** */
      {
         permute (perm_sw, nsw);        /* permute switches for M-sampler */
         for (k = 0; k < nsw; ++k)
         {                              /* for ea. switch, get meiosis */
            l = perm_sw[k] + 1;         /* and component */
            i = (meiosis[l].ind).index;
            sx = meiosis[l].sex;
            c = nghd[i].component - 1;
            /*                        * do m-sampling for the meiosis */
            do_msmpl (nloc, mm, c, i, sx, prob, ngenes);
         }
      }
      else
      {                                 /* ****   step   **** */

         l = intran (0, nsw - 1) + 1;   /* choose a random switch */
         i = (meiosis[l].ind).index;
         sx = meiosis[l].sex;           /* get meiosis and component */
         c = nghd[i].component - 1;
         /*                           * do m-sampling for this meiosis */
         do_msmpl (nloc, mm, c, i, sx, prob, ngenes);
      }
   }
}

void free_perm (void)
{
   if (perm_sw)
   {
      free (perm_sw);
      perm_sw = NULL;
   }
   if (perm_loc)
   {
      free (perm_loc);
      perm_loc = NULL;
   }
   return;
}
